import json
import pathlib
from enum import Enum
from random import random, randint

from qgis._analysis import QgsRasterCalculatorEntry, QgsRasterCalculator
from qgis._core import QgsApplication, QgsRasterLayer, QgsCoordinateReferenceSystem, QgsRectangle, \
    QgsCoordinateTransformContext, QgsRasterFileWriter, QgsRasterPipe

from queries import insert_case, insert_ts, insert_ts_value

WATER_DISCHARGE_LEVEL_LOW_MIN = 0
WATER_DISCHARGE_LEVEL_LOW_MAX = 316
WATER_DISCHARGE_LEVEL_MEDIUM_MIN = 316
WATER_DISCHARGE_LEVEL_MEDIUM_MAX = 800
WATER_DISCHARGE_LEVEL_HIGH_MIN = 800
WATER_DISCHARGE_LEVEL_HIGH_MAX = 4200

RAINFALL_LOW_MIN = 0
RAINFALL_LOW_MAX = 53
RAINFALL_MEDIUM_MIN = 53
RAINFALL_MEDIUM_MAX = 149
RAINFALL_HIGH_MIN = 149
RAINFALL_HIGH_MAX = 270

FORECAST_LOW_MIN = 0
FORECAST_LOW_MAX = 53
FORECAST_MEDIUM_MIN = 53
FORECAST_MEDIUM_MAX = 149
FORECAST_HIGH_MIN = 152.54772
FORECAST_HIGH_MAX = 387.047422

WATER_DISCHARGE_LEVEL_LOW_FACTOR = 0.99
WATER_DISCHARGE_LEVEL_MEDIUM_FACTOR = 1
WATER_DISCHARGE_LEVEL_HIGH_FACTOR = 1.05

RAINFALL_GAUGE_LOW_FACTOR = 0.99
RAINFALL_GAUGE_MEDIUM_FACTOR = 1
RAINFALL_GAUGE_HIGH_FACTOR = 1.15

RAINFALL_FORECAST_LOW_FACTOR = 0.99
RAINFALL_FORECAST_MEDIUM_FACTOR = 1.05
RAINFALL_FORECAST_HIGH_FACTOR = 1.2

# Factor 1
RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR1 = 0.99
RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR1 = 1.15
RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR1 = 1.2
# Factor 2
RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR2 = 0.9999999
RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR2 = 1.02
RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR2 = 1.03
# Factor 3
RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR3 = 0.9999999
RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR3 = 1.052
RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR3 = 1.098
# Factor 4
RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR4 = 0.9999999
RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR4 = 1.068
RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR4 = 1.132
# Factor 5
RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR5 = 0.9999999
RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR5 = 1.084
RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR5 = 1.166
# Factor 6
RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR6 = 0.9999999
RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR6 = 1.1
RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR6 = 1.2

# Water discharge level
# Water discharge level low
WATER_DISCHARGE_LEVEL_LOW_ASC_INIT = WATER_DISCHARGE_LEVEL_LOW_MIN
WATER_DISCHARGE_LEVEL_LOW_ASC_END = WATER_DISCHARGE_LEVEL_LOW_MAX
WATER_DISCHARGE_LEVEL_LOW_CONST_INIT = (WATER_DISCHARGE_LEVEL_LOW_MIN + WATER_DISCHARGE_LEVEL_LOW_MAX) / 2
WATER_DISCHARGE_LEVEL_LOW_CONST_END = (WATER_DISCHARGE_LEVEL_LOW_MIN + WATER_DISCHARGE_LEVEL_LOW_MAX) / 2
WATER_DISCHARGE_LEVEL_LOW_DESC_INIT = WATER_DISCHARGE_LEVEL_LOW_MAX
WATER_DISCHARGE_LEVEL_LOW_DESC_END = WATER_DISCHARGE_LEVEL_LOW_MIN

# Water discharge level medium
WATER_DISCHARGE_LEVEL_MEDIUM_ASC_INIT = WATER_DISCHARGE_LEVEL_MEDIUM_MIN
WATER_DISCHARGE_LEVEL_MEDIUM_ASC_END = WATER_DISCHARGE_LEVEL_MEDIUM_MAX
WATER_DISCHARGE_LEVEL_MEDIUM_CONST_INIT = (WATER_DISCHARGE_LEVEL_MEDIUM_ASC_END + WATER_DISCHARGE_LEVEL_MEDIUM_MIN) / 2
WATER_DISCHARGE_LEVEL_MEDIUM_CONST_END = (WATER_DISCHARGE_LEVEL_MEDIUM_ASC_END + WATER_DISCHARGE_LEVEL_MEDIUM_MIN) / 2
WATER_DISCHARGE_LEVEL_MEDIUM_DESC_INIT = WATER_DISCHARGE_LEVEL_MEDIUM_MAX
WATER_DISCHARGE_LEVEL_MEDIUM_DESC_END = WATER_DISCHARGE_LEVEL_MEDIUM_MIN

# Water discharge level high
WATER_DISCHARGE_LEVEL_HIGH_ASC_INIT = WATER_DISCHARGE_LEVEL_HIGH_MIN
WATER_DISCHARGE_LEVEL_HIGH_ASC_END = WATER_DISCHARGE_LEVEL_HIGH_MAX
WATER_DISCHARGE_LEVEL_HIGH_CONST_INIT = (WATER_DISCHARGE_LEVEL_HIGH_ASC_INIT + WATER_DISCHARGE_LEVEL_HIGH_MIN) / 2
WATER_DISCHARGE_LEVEL_HIGH_CONST_END = (WATER_DISCHARGE_LEVEL_HIGH_ASC_INIT + WATER_DISCHARGE_LEVEL_HIGH_MIN) / 2
WATER_DISCHARGE_LEVEL_HIGH_DESC_INIT = WATER_DISCHARGE_LEVEL_HIGH_MAX
WATER_DISCHARGE_LEVEL_HIGH_DESC_END = WATER_DISCHARGE_LEVEL_HIGH_MIN

# Rainfall gauge
# Rainfall gauge low
RAINFALL_GAUGE_LOW_ASC_INIT = RAINFALL_LOW_MIN
RAINFALL_GAUGE_LOW_ASC_END = RAINFALL_LOW_MAX
RAINFALL_GAUGE_LOW_CONST_INIT = (RAINFALL_LOW_MAX + RAINFALL_LOW_MIN) / 2
RAINFALL_GAUGE_LOW_CONST_END = (RAINFALL_LOW_MAX + RAINFALL_LOW_MIN) / 2
RAINFALL_GAUGE_LOW_DESC_INIT = RAINFALL_LOW_MAX
RAINFALL_GAUGE_LOW_DESC_END = RAINFALL_LOW_MIN

# Rainfall gauge medium
RAINFALL_GAUGE_MEDIUM_ASC_INIT = RAINFALL_MEDIUM_MIN
RAINFALL_GAUGE_MEDIUM_ASC_END = RAINFALL_MEDIUM_MAX
RAINFALL_GAUGE_MEDIUM_CONST_INIT = (RAINFALL_MEDIUM_MAX + RAINFALL_MEDIUM_MIN) / 2
RAINFALL_GAUGE_MEDIUM_CONST_END = (RAINFALL_MEDIUM_MAX + RAINFALL_MEDIUM_MIN) / 2
RAINFALL_GAUGE_MEDIUM_DESC_INIT = RAINFALL_MEDIUM_MAX
RAINFALL_GAUGE_MEDIUM_DESC_END = RAINFALL_MEDIUM_MIN

# Rainfall forecast high
RAINFALL_GAUGE_HIGH_ASC_INIT = RAINFALL_HIGH_MIN
RAINFALL_GAUGE_HIGH_ASC_END = RAINFALL_HIGH_MAX
RAINFALL_GAUGE_HIGH_CONST_INIT = (RAINFALL_HIGH_MAX + RAINFALL_MEDIUM_MIN) / 2
RAINFALL_GAUGE_HIGH_CONST_END = (RAINFALL_HIGH_MAX + RAINFALL_MEDIUM_MIN) / 2
RAINFALL_GAUGE_HIGH_DESC_INIT = RAINFALL_HIGH_MAX
RAINFALL_GAUGE_HIGH_DESC_END = RAINFALL_HIGH_MIN

# Rainfall forecast
# Rainfall forecast low
RAINFALL_FORECAST_LOW_ASC_INIT = RAINFALL_LOW_MIN
RAINFALL_FORECAST_LOW_ASC_END = RAINFALL_LOW_MAX
RAINFALL_FORECAST_LOW_CONST_INIT = (RAINFALL_LOW_MAX + RAINFALL_LOW_MIN) / 2
RAINFALL_FORECAST_LOW_CONST_END = (RAINFALL_LOW_MAX + RAINFALL_LOW_MIN) / 2
RAINFALL_FORECAST_LOW_DESC_INIT = RAINFALL_LOW_MAX
RAINFALL_FORECAST_LOW_DESC_END = RAINFALL_LOW_MIN

# Rainfall forecast medium
RAINFALL_FORECAST_MEDIUM_ASC_INIT = RAINFALL_MEDIUM_MIN
RAINFALL_FORECAST_MEDIUM_ASC_END = RAINFALL_MEDIUM_MAX
RAINFALL_FORECAST_MEDIUM_CONST_INIT = (RAINFALL_MEDIUM_MAX + RAINFALL_MEDIUM_MIN) / 2
RAINFALL_FORECAST_MEDIUM_CONST_END = (RAINFALL_MEDIUM_MAX + RAINFALL_MEDIUM_MIN) / 2
RAINFALL_FORECAST_MEDIUM_DESC_INIT = RAINFALL_MEDIUM_MAX
RAINFALL_FORECAST_MEDIUM_DESC_END = RAINFALL_MEDIUM_MIN

# Rainfall forecast high
RAINFALL_FORECAST_HIGH_ASC_INIT = RAINFALL_HIGH_MIN
RAINFALL_FORECAST_HIGH_ASC_END = RAINFALL_HIGH_MAX
RAINFALL_FORECAST_HIGH_CONST_INIT = (RAINFALL_HIGH_MAX + RAINFALL_HIGH_MIN) / 2
RAINFALL_FORECAST_HIGH_CONST_END = (RAINFALL_HIGH_MAX + RAINFALL_HIGH_MIN) / 2
RAINFALL_FORECAST_HIGH_DESC_INIT = RAINFALL_HIGH_MAX
RAINFALL_FORECAST_HIGH_DESC_END = RAINFALL_HIGH_MIN


class Riesgo(Enum):
    ALTO = 1
    MEDIO = 2
    BAJO = 3


class Evolucion(Enum):
    ASCENDENTE = 1
    CONSTANTE = 2
    DESCENDENTE = 3


class Tipo(Enum):
    WDL = 1
    RG = 2
    FORECAST = 3


def generate_ts_values(intervalos, evo, valor_inicial, valor_final):
    generated_ts = []

    variacion = 0.05
    tinit = 0
    tfin = intervalos
    for t in range(tinit, tfin):
        if evo == Evolucion.ASCENDENTE:
            p = t - tinit
            delta = (valor_final - valor_inicial) * variacion
            coef = 2 * delta * random() + 1 - delta
            v = coef * ((valor_final - valor_inicial) * p / intervalos) + valor_inicial
            generated_ts.append(max(0, v))
        if evo == Evolucion.CONSTANTE:
            v = valor_inicial + ((valor_inicial * ((random()) * 2 - 1)) / 10)
            generated_ts.append(max(0, v))
        if evo == Evolucion.DESCENDENTE:
            p = t - tinit
            delta = abs((valor_final - valor_inicial) * variacion)
            coef = 2 * delta * random() + 1 - delta
            v = coef * ((valor_final - valor_inicial) * p / intervalos) + valor_inicial
            generated_ts.append(max(0, v))

    return generated_ts


def get_intervalos(tipo):
    if tipo == Tipo.RG:
        return 5
    if tipo == Tipo.WDL:
        return 9
    if tipo == Tipo.FORECAST:
        return 9


# rgs = ["VC-RG1", "VC-RG2"]
rgs = ["VC-RG1", "VC-RG2", "VC-RG3", "VC-RG4", "VC-RG5", "VC-RG6"]
# sensor identifier values for wdls
wdls = ["VC-WDL1"]
# sensor identifier values for rfs
rfs = ["VC-RF1"]

location = "Venero Claro"


def get_init_value(tipo, riesgo, evo):
    if tipo == Tipo.WDL:
        if riesgo == Riesgo.BAJO:
            if evo == Evolucion.CONSTANTE:
                return WATER_DISCHARGE_LEVEL_LOW_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return WATER_DISCHARGE_LEVEL_LOW_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return WATER_DISCHARGE_LEVEL_LOW_DESC_INIT
        elif riesgo == Riesgo.MEDIO:
            if evo == Evolucion.CONSTANTE:
                return WATER_DISCHARGE_LEVEL_MEDIUM_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return WATER_DISCHARGE_LEVEL_MEDIUM_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return WATER_DISCHARGE_LEVEL_MEDIUM_DESC_INIT
        elif riesgo == Riesgo.ALTO:
            if evo == Evolucion.CONSTANTE:
                return WATER_DISCHARGE_LEVEL_HIGH_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return WATER_DISCHARGE_LEVEL_HIGH_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return WATER_DISCHARGE_LEVEL_HIGH_DESC_INIT
    elif tipo == Tipo.RG:
        if riesgo == Riesgo.BAJO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_GAUGE_LOW_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_GAUGE_LOW_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_GAUGE_LOW_DESC_INIT
        elif riesgo == Riesgo.MEDIO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_GAUGE_MEDIUM_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_GAUGE_MEDIUM_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_GAUGE_MEDIUM_DESC_INIT
        elif riesgo == Riesgo.ALTO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_GAUGE_HIGH_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_GAUGE_HIGH_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_GAUGE_HIGH_DESC_INIT
    elif tipo == Tipo.FORECAST:
        if riesgo == Riesgo.BAJO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_FORECAST_LOW_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_FORECAST_LOW_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_FORECAST_LOW_DESC_INIT
        elif riesgo == Riesgo.MEDIO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_FORECAST_MEDIUM_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_FORECAST_MEDIUM_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_FORECAST_MEDIUM_DESC_INIT
        elif riesgo == Riesgo.ALTO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_FORECAST_HIGH_CONST_INIT
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_FORECAST_HIGH_ASC_INIT
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_FORECAST_MEDIUM_DESC_INIT


def get_end_value(tipo, riesgo, evo):
    if tipo == Tipo.WDL:
        if riesgo == Riesgo.BAJO:
            if evo == Evolucion.CONSTANTE:
                return WATER_DISCHARGE_LEVEL_LOW_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return WATER_DISCHARGE_LEVEL_LOW_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return WATER_DISCHARGE_LEVEL_LOW_DESC_END
        elif riesgo == Riesgo.MEDIO:
            if evo == Evolucion.CONSTANTE:
                return WATER_DISCHARGE_LEVEL_MEDIUM_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return WATER_DISCHARGE_LEVEL_MEDIUM_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return WATER_DISCHARGE_LEVEL_MEDIUM_DESC_END
        elif riesgo == Riesgo.ALTO:
            if evo == Evolucion.CONSTANTE:
                return WATER_DISCHARGE_LEVEL_HIGH_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return WATER_DISCHARGE_LEVEL_HIGH_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return WATER_DISCHARGE_LEVEL_HIGH_DESC_END
    elif tipo == Tipo.RG:
        if riesgo == Riesgo.BAJO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_GAUGE_LOW_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_GAUGE_LOW_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_GAUGE_LOW_DESC_END
        elif riesgo == Riesgo.MEDIO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_GAUGE_MEDIUM_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_GAUGE_MEDIUM_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_GAUGE_MEDIUM_DESC_END
        elif riesgo == Riesgo.ALTO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_GAUGE_HIGH_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_GAUGE_HIGH_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_GAUGE_HIGH_DESC_END
    elif tipo == Tipo.FORECAST:
        if riesgo == Riesgo.BAJO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_FORECAST_LOW_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_FORECAST_LOW_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_FORECAST_LOW_DESC_END
        elif riesgo == Riesgo.MEDIO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_FORECAST_MEDIUM_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_FORECAST_MEDIUM_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_FORECAST_MEDIUM_DESC_END
        elif riesgo == Riesgo.ALTO:
            if evo == Evolucion.CONSTANTE:
                return RAINFALL_FORECAST_HIGH_CONST_END
            elif evo == Evolucion.ASCENDENTE:
                return RAINFALL_FORECAST_HIGH_ASC_END
            elif evo == Evolucion.DESCENDENTE:
                return RAINFALL_FORECAST_MEDIUM_DESC_END


def get_risk_factor(riesgoWDL, riesgoRG, riesgoRF, evoRF, prediction_horizon):
    init_factor_value = 1
    if riesgoWDL == Riesgo.BAJO:
        init_factor_value = init_factor_value * WATER_DISCHARGE_LEVEL_LOW_FACTOR
    elif riesgoWDL == Riesgo.MEDIO:
        init_factor_value = init_factor_value * WATER_DISCHARGE_LEVEL_MEDIUM_FACTOR
    elif riesgoWDL == Riesgo.ALTO:
        init_factor_value = init_factor_value * WATER_DISCHARGE_LEVEL_HIGH_FACTOR

    if riesgoRG == Riesgo.BAJO:
        init_factor_value = init_factor_value * RAINFALL_GAUGE_LOW_FACTOR
    elif riesgoRG == Riesgo.MEDIO:
        init_factor_value = init_factor_value * RAINFALL_GAUGE_MEDIUM_FACTOR
    elif riesgoRG == Riesgo.ALTO:
        init_factor_value = init_factor_value * RAINFALL_GAUGE_HIGH_FACTOR

    if riesgoRF == Riesgo.BAJO:
        init_factor_value = init_factor_value * RAINFALL_FORECAST_LOW_FACTOR
    elif riesgoRF == Riesgo.MEDIO:
        init_factor_value = init_factor_value * RAINFALL_FORECAST_MEDIUM_FACTOR
    elif riesgoRF == Riesgo.ALTO:
        init_factor_value = init_factor_value * RAINFALL_FORECAST_HIGH_FACTOR

    if evoRF == Evolucion.ASCENDENTE:
        init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR1
    elif evoRF == Evolucion.DESCENDENTE:
        init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR1
    elif evoRF == Evolucion.CONSTANTE:
        init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR1

    if prediction_horizon >= 2:
        if evoRF == Evolucion.ASCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR2
        elif evoRF == Evolucion.DESCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR2
        elif evoRF == Evolucion.CONSTANTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR2
    if prediction_horizon >= 3:
        if evoRF == Evolucion.ASCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR3
        elif evoRF == Evolucion.DESCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR3
        elif evoRF == Evolucion.CONSTANTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR3
    if prediction_horizon >= 4:
        if evoRF == Evolucion.ASCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR4
        elif evoRF == Evolucion.DESCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR4
        elif evoRF == Evolucion.CONSTANTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR4
    if prediction_horizon >= 5:
        if evoRF == Evolucion.ASCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR5
        elif evoRF == Evolucion.DESCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR5
        elif evoRF == Evolucion.CONSTANTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR5
    if prediction_horizon == 6:
        if evoRF == Evolucion.ASCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_HIGH_FACTOR6
        elif evoRF == Evolucion.DESCENDENTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_LOW_FACTOR6
        elif evoRF == Evolucion.CONSTANTE:
            init_factor_value = init_factor_value * RAINFALL_FORECAST_EVOLUTION_MEDIUM_FACTOR6

    return init_factor_value


# Para un riesgo, un intervalo y una iteración dadas, retorna un mapa representando la inundación.
def get_map(riesgo):
    # Linux prefix path
    prefix_path = "usr/bin/qgis"

    QgsApplication.setPrefixPath(prefix_path, True)

    # Referencia a QgsApplication. False para mantener desactivada la Interfaz gráfica.
    qgs = QgsApplication([], False)
    # Inicia qgs
    qgs.initQgis()

    # Capa base a utilizar. Contiene la elevación en cada punto
    dem_path = './maps/N40W005.hgt'
    dem_raster = QgsRasterLayer(dem_path, 'raster')
    dem_raster.setCrs(QgsCoordinateReferenceSystem(4326, QgsCoordinateReferenceSystem.EpsgCrsId))

    inundacion = riesgo
    altura_min = 734

    tipo = 'GTiff'
    extension = QgsRectangle(-4.665386, 40.400000, -4.614170, 40.411210)
    filas = dem_raster.height()
    columnas = dem_raster.width()
    filename = "{0}_{1}_{2}.tif".format(location, riesgo, random())

    capa_salida = str(pathlib.Path(str(pathlib.Path().resolve()) + "/maps/{0}".format(filename)))
    capas_entrada = []  # Lista vacia que almacenará los objetos de entrada
    transform_context = QgsCoordinateTransformContext()
    transform_context.addCoordinateOperation(QgsCoordinateReferenceSystem("EPSG:4326"),
                                             QgsCoordinateReferenceSystem("EPSG:4326"),
                                             "")

    # Crear y dar valores al objeto de entrada sobre el que realizar operaciones
    capa1 = QgsRasterCalculatorEntry()  # Crear el objeto de entrada
    capa1.ref = 'capa1@1'  # Crear referencia para la expresion
    capa1.raster = dem_raster  # Seleccion de la capa
    capa1.bandNumber = 1  # Definir el nº de banda a usar

    capas_entrada.append(capa1)  # Añadir a la lista vacia

    # Mask.
    # Para todos los valores de la capa que con la altura mínima y la inundación sean mayores que 0, se define un nuevo
    # valor que es la altura de la capa + la altura mínima + la inundación.
    expresion = '(capa1@1 * - 1 + {0} + {1} > 0) * (-1 * capa1@1 + {0} + {1})'.format(int(altura_min),
                                                                                      inundacion)

    calculos = QgsRasterCalculator(expresion, capa_salida, tipo, extension, columnas, filas, capas_entrada,
                                   transform_context)
    calculos.processCalculation()

    out_layer = QgsRasterLayer(capa_salida, "raster")
    out_layer.loadNamedStyle("estilo.qml")

    capa_write = str(pathlib.Path(str(pathlib.Path().resolve()) + "/maps/{0}{1}".format("computed_", filename)))

    file_writer = QgsRasterFileWriter(capa_write)
    pipe = QgsRasterPipe()
    provider = out_layer.dataProvider()
    if not pipe.set(provider.clone()):
        msg = "Cannot set pipe provider"
        print(msg)

    file_writer.writeRaster(
        pipe,
        provider.xSize(),
        provider.ySize(),
        provider.extent(),
        provider.crs())
    return "computed_" + filename


def getEvo():
    r = randint(1, 3)
    if r == 1:
        return Evolucion.ASCENDENTE
    elif r == 2:
        return Evolucion.CONSTANTE
    else:
        return Evolucion.DESCENDENTE


j = 0
case_map_dict = {}
dict_case_idx = {}
for riesgoWDL in Riesgo:
    for riesgoRG in Riesgo:
        for riesgoRF in Riesgo:
            evoWDL = getEvo()
            evoRG = getEvo()
            evoRF = getEvo()
            # Insert case and get case id
            case_id = insert_case(location)
            # Create time series
            tss = []
            case_ts = []

            # case_ts[0] es WDL ts
            for wdl in wdls:
                wdl_ts = generate_ts_values(get_intervalos(Tipo.WDL), evoWDL,
                                            get_init_value(Tipo.WDL, riesgoWDL, evoWDL) / 1000,
                                            get_end_value(Tipo.WDL, riesgoWDL, evoWDL) / 1000)
                case_ts.append(wdl_ts)

            # case_ts[1] es RG ts
            for rg in rgs:
                rg_ts = generate_ts_values(get_intervalos(Tipo.RG), evoRG,
                                           get_init_value(Tipo.RG, riesgoRG, evoRG),
                                           get_end_value(Tipo.RG, riesgoRG, evoRG))
                case_ts.append(rg_ts)

            # case_ts[2] es RF ts
            for rf in rfs:
                rf_ts = generate_ts_values(get_intervalos(Tipo.FORECAST), evoRF,
                                           get_init_value(Tipo.FORECAST, riesgoRF, evoRF),
                                           get_end_value(Tipo.FORECAST, riesgoRF, evoRF))
                case_ts.append(rf_ts)

            # Insert time series and get ids
            for sensor_id in wdls:
                tss.append(insert_ts(sensor_id, case_id))
            for sensor_id in rgs:
                tss.append(insert_ts(sensor_id, case_id))
            for sensor_id in rfs:
                tss.append(insert_ts(sensor_id, case_id))

            first_id = tss[0]
            inserted_ts_id = first_id
            for ts in case_ts:
                for i in range(0, len(ts)):
                    insert_ts_value(inserted_ts_id, ts[i], 0, 0)
                inserted_ts_id += 1
            # Load map for each time horizon

            dict_case_idx[j] = case_id
            case_map_dict[j] = []
            for i in range(0, 6):
                pred_horizon = i + 1
                risk_factor = get_risk_factor(riesgoWDL, riesgoRG, riesgoRF, evoRF, pred_horizon)
                water_level_value = case_ts[0][i]
                inundation_level = (risk_factor * water_level_value)
                case_map_dict[j].append(get_map(inundation_level))
            j += 1

with open("dict_case_idx.json", "w") as outfile:
    json.dump(dict_case_idx, outfile)

with open("dict_case_map.json", "w") as outfile:
    json.dump(case_map_dict, outfile)
