from pathlib import Path

import psycopg2
from config import config

def import_and_insert_image_procedure(blob_name, case_id, prediction_horizon, path="maps/"):
    conn = None
    try:
        params = config()
        conn = psycopg2.connect(**params)
        with conn:
            with conn.cursor() as db_cursor:
                lob = conn.lobject(0, 'w', 0, str(Path(path,  blob_name)))
                insert_string = "INSERT INTO web_map_blob(name, case_id, prediction_horizon, data) VALUES " \
                                "('" + blob_name + "'," + str(case_id) + ", " + str(prediction_horizon) + ", " + str(lob.oid) + ");"
                db_cursor.execute(insert_string)
            db_cursor.close
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()


# store blob in custom table with insert
def insert_image(blob_name, case_id, prediction_horizon, blob_data):
    conn = None
    try:
        # read connection parameters
        params = config()

        # connect to the PostgreSQL server
        conn = psycopg2.connect(**params)

        with conn:
            with conn.cursor() as db_cursor:
                s = "INSERT INTO web_map_blob(name, case_id, prediction_horizon, data) VALUES ('{0}', {1}, {2}, lo_import('{3}'));".format(blob_name, case_id, prediction_horizon, blob_data)
                db_cursor.execute(s)
        db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()




def recover_blob(blob_name):
    conn = None
    try:
        # read connection parameters
        params = config()

        # connect to the PostgreSQL server
        conn = psycopg2.connect(**params)

        with conn:
            with conn.cursor() as db_cursor:
                # Query
                s = "SELECT blob_data FROM t_blob_test WHERE t_blob_test.name = '{0}'".format(blob_name)
                # Execute query
                db_cursor.execute(s)
                # Obtain response from cursor
                response = db_cursor.fetchall()
                loid = response[0][0]  # [(id, )]
                # Recover blob form loid
                lo = conn.lobject(loid)
                lo.export('./maps/{0}exported.tif'.format(loid))
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()


# Insert case into database
# Returns inserted case id
def insert_case(case_location):
    conn = None
    case_id = 0
    try:
        # read connection parameters
        params = config()

        # connect to the PostgreSQL server
        print('Connecting to the PostgreSQL database...')
        conn = psycopg2.connect(**params)

        with conn:
            with conn.cursor() as db_cursor:
                # create a cursor
                print('Select location')
                s = "SELECT conf_location_id FROM web_conf_location WHERE web_conf_location.name = '{0}'".format(
                    case_location)
                db_cursor.execute(s)
                location_id = db_cursor.fetchall()
                print("Insert into web_case values {0}".format(location_id[0][0]))
                s = "INSERT INTO web_case(conf_location_id) VALUES ({0}) RETURNING case_id".format(location_id[0][0])
                db_cursor.execute(s)
                case_id = db_cursor.fetchone()[0]
        db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()
            print('Database connection closed.')
    return case_id


# Insert time series for a given sensor identifier and case
# Returns inserted ts id
def insert_ts(sensor_identifier, case_id):
    conn = None
    ts_id = 0
    try:
        # read connection parameters
        params = config()

        # connect to the PostgreSQL server
        conn = psycopg2.connect(**params)

        with conn:
            with conn.cursor() as db_cursor:
                # create a cursor
                query = "SELECT location_sensor_id FROM web_location_sensor WHERE sensor_identifier = '{0}'".format(
                    sensor_identifier)
                db_cursor.execute(query)
                sensor_id = db_cursor.fetchone()[0]
                s = "INSERT INTO web_ts(location_sensor_id, case_id) VALUES ({0}, {1}) RETURNING ts_id".format(
                    sensor_id, case_id)
                db_cursor.execute(s)
                ts_id = db_cursor.fetchone()[0]
        db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()
        return ts_id


# Insert time series value for a given time series and timestamp
def insert_ts_value(ts_id, value, converted_value, timestamp):
    conn = None
    try:
        # read connection parameters
        params = config()

        # connect to the PostgreSQL server
        conn = psycopg2.connect(**params)

        with conn:
            with conn.cursor() as db_cursor:
                # create a cursor
                s = "INSERT INTO web_ts_value(ts_id, value, converted_value, timestamp) VALUES ({0}, {1}, {2}, {3})"\
                    .format(ts_id, value, converted_value, timestamp)
                db_cursor.execute(s)
        db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()


def insert_simulation(gid, timestamp_value, rg_value, wdl_cambo_value, wdl_cambo_converted_value, wdl_osses_value,
                      wdl_osses_converted_value, simulation_graph_location, map_location):
    conn = None
    try:
        # read connection parameters
        params = config()

        # connect to the PostgreSQL server
        conn = psycopg2.connect(**params)

        with conn:
            with conn.cursor() as db_cursor:
                lob = conn.lobject(0, 'w', 0, simulation_graph_location)
                lob_map_location = conn.lobject(0, 'w', 0, map_location)
                # create a cursor
                # s = "INSERT INTO web_simulation(gid, timestamp, rg_value, wdl_cambo_value, wdl_cambo_converted_value," \
                #     " wdl_osses_value, wdl_osses_converted_value, " \
                #     " simulation_graph_id, map_id) " \
                #     "VALUES ({0}, {1}, {2}, {3}, {4}, {5}, {6}," + str(lob.oid) + "," + str(lob_map_location.oid) + ")"\
                #     .format(gid, timestamp_value, rg_value, wdl_cambo_value, wdl_cambo_converted_value, wdl_osses_value,
                #             wdl_osses_converted_value, simulation_graph_location, map_location)
                s = "INSERT INTO web_simulation(gid, timestamp, rg_value, wdl_cambo_value, wdl_cambo_converted_value, wdl_osses_value, wdl_osses_converted_value, simulation_graph_id, map_id) VALUES (" + str(gid) + "," + str(timestamp_value) + "," + str(rg_value) + "," + str(wdl_cambo_value) + "," + str(wdl_cambo_converted_value) + "," + str(wdl_osses_value) + "," + str(wdl_osses_converted_value) + "," + str(lob.oid) + "," + str(lob_map_location.oid) + ");"
                db_cursor.execute(s)
        db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            conn.close()

