from application.cbr.distance_strat.compute_distance_strategy import ComputeDistanceStrategy
from application.cbr.distance_strat.distance_helper import get_ts_values
from database.case_queries import get_weights, is_avg_np
import numpy as np

from model.cbr_constants import WATER_DISCHARGE_LEVEL, RAINFALL_GAUGE, RAINFALL_FORECAST
from model.dimension import Dimension


class ParetoDominanceDistanceStrategy(ComputeDistanceStrategy):

    def get_average_sensors(self, dimension_list, location_id):
        wdl_is_avg = False
        rg_is_avg = False
        rf_is_avg = False
        for dimension in dimension_list:
            s_type = dimension.sensor_type
            if s_type == WATER_DISCHARGE_LEVEL:
                wdl_is_avg = is_avg_np(s_type, location_id)
                if wdl_is_avg[0][0] == 'None':
                    wdl_is_avg = False
            elif s_type == RAINFALL_GAUGE:
                rg_is_avg = is_avg_np(s_type, location_id)
                if rg_is_avg[0][0] == 'None':
                    rg_is_avg = False
            elif s_type == RAINFALL_FORECAST:
                rf_is_avg = is_avg_np(s_type, location_id)
                if rf_is_avg[0][0] == 'None':
                    rf_is_avg = False
        return wdl_is_avg, rg_is_avg, rf_is_avg

    def get_number_of_sensors(self, dimension_list):
        wdl_n_sensors = 0
        rg_n_sensors = 0
        rf_n_sensors = 0
        for dimension in dimension_list:
            s_type = dimension.sensor_type
            if s_type == WATER_DISCHARGE_LEVEL:
                wdl_n_sensors += dimension.sensors
            elif s_type == RAINFALL_GAUGE:
                rg_n_sensors += dimension.sensors
            elif s_type == RAINFALL_FORECAST:
                rf_n_sensors += dimension.sensors
        return wdl_n_sensors, rg_n_sensors, rf_n_sensors

    def get_intervals(self, dimension_list):
        wdl_int = 0
        rg_int = 0
        rf_int = 0
        for dimension in dimension_list:
            s_type = dimension.sensor_type
            if s_type == WATER_DISCHARGE_LEVEL:
                wdl_int = wdl_int + dimension.intervals
            elif s_type == RAINFALL_GAUGE:
                rg_int = rg_int + dimension.intervals
            elif s_type == RAINFALL_FORECAST:
                rf_int = rf_int + dimension.intervals
        return wdl_int, rg_int, rf_int

    def compute_distance_numpy_arrays(self, current_case_numpy_array, cases_numpy_array, weights_numpy_array,
                                      dimension_list, location_id):
        wdl_int, rg_int, rf_int = self.get_intervals(dimension_list)
        wdl_n_sensor, rg_n_sensor, rf_n_sensor = self.get_number_of_sensors(dimension_list)
        wdl_avg, rg_avg, rf_avg = self.get_average_sensors(dimension_list, location_id[0])

        if wdl_avg:
            # There are 2 sensors defined but only an average value, so it is not correct to multiply by the number of sensors
            wdl_n_values = wdl_int * wdl_n_sensor
        else:
            wdl_n_values = wdl_int
        if rg_avg:
            rg_n_values = rg_int * rg_n_sensor
        else:
            rg_n_values = rg_int

        for dimension in dimension_list:
            s_type = dimension.sensor_type
            if s_type == RAINFALL_GAUGE:
                rg_cases_numpy_array = [x[wdl_n_values: wdl_n_values + rg_n_values] for x in cases_numpy_array]
                rg_current_case_numpy_array = current_case_numpy_array[wdl_n_values: wdl_n_values + rg_n_values]
                rg_weights_numpy_array = weights_numpy_array[wdl_int: wdl_int + rg_int]
                rg_max_numpy_array = np.amax(rg_cases_numpy_array, axis=1)
                if len(rg_current_case_numpy_array) > len(rg_weights_numpy_array):
                    rg_current_case_numpy_array = rg_current_case_numpy_array[0:rg_int]
                    rg_cases_numpy_array = [x[0:rg_int] for x in rg_cases_numpy_array]
                dimension.distance = self.calculate_distance_numpy_array(rg_cases_numpy_array,
                                                                         rg_current_case_numpy_array,
                                                                         rg_max_numpy_array,
                                                                         rg_weights_numpy_array)

            elif s_type == WATER_DISCHARGE_LEVEL:
                wdl_cases_numpy_array = [x[0: wdl_n_values] for x in cases_numpy_array]
                wdl_current_case_numpy_array = current_case_numpy_array[0: wdl_n_values]
                wdl_weights_numpy_array = weights_numpy_array[0: wdl_int]

                empty_case = [0, 0, 0, 0, 0, 0, 0, 0, 0]
                for i in range(len(wdl_cases_numpy_array)):
                    if location_id == 3:
                        if len(wdl_cases_numpy_array[i] < 9):
                            wdl_cases_numpy_array[i] = empty_case

                wdl_max_numpy_array = np.amax(wdl_cases_numpy_array, axis=1)
                if len(wdl_current_case_numpy_array) > len(wdl_weights_numpy_array):
                    wdl_current_case_numpy_array = wdl_current_case_numpy_array[0:wdl_int]
                    wdl_cases_numpy_array = [x[0:wdl_int] for x in wdl_cases_numpy_array]
                dimension.distance = self.calculate_distance_numpy_array(wdl_cases_numpy_array,
                                                                         wdl_current_case_numpy_array,
                                                                         wdl_max_numpy_array,
                                                                         wdl_weights_numpy_array)
            elif s_type == RAINFALL_FORECAST:
                # Esto es porque solo hay 3 valores en el caso cuando debería de haber 6 para rgs

                rf_cases_numpy_array = [x[wdl_n_values + rg_n_values:] for x in cases_numpy_array]
                rf_current_case_numpy_array = current_case_numpy_array[wdl_n_values + rg_n_values:]
                rf_weights_numpy_array = weights_numpy_array[wdl_int + rg_int:]
                rf_max_numpy_array = np.amax(rf_cases_numpy_array, axis=1)
                if len(rf_current_case_numpy_array) > len(rf_weights_numpy_array):
                    rf_current_case_numpy_array = rf_current_case_numpy_array[0:rf_int]
                    rf_cases_numpy_array = [x[0:rf_int] for x in rf_cases_numpy_array]
                dimension.distance = self.calculate_distance_numpy_array(rf_cases_numpy_array,
                                                                         rf_current_case_numpy_array,
                                                                         rf_max_numpy_array,
                                                                         rf_weights_numpy_array)
        return dimension_list

    def calculate_distance_numpy_array(self, cases, current, vector_max, weights):
        p = 2
        current_elems = [float(elem) for elem in current]
        a = np.abs(np.array(current_elems) - np.array(cases))
        b = vector_max
        b[b == 0] = 0.0000000000000001
        c = np.power(b.astype(float), -1)
        c = np.reshape(c, (len(c), 1))
        d = np.multiply(a, c)
        e = np.power(d, p)
        f = np.multiply(np.power(weights, p), e)
        g = np.sum(f, axis=1)
        h = np.sum(weights)
        return g / h

    # Compute distance for a case
    def compute_distance(self, current_vector_wdl, current_vector_rf, current_vector_rg, case, vector_max,
                         prediction_horizon, dimensions):
        # Each dim will have values to compute distance
        vector_dim_wdl = []
        vector_dim_rg = []
        vector_dim_rf = []
        # Each dim will have weights associated
        weights_dim_wdl = []
        weights_dim_rg = []
        weights_dim_rf = []
        # Each dimension with his weigths is stored in this array
        dimension_list = []
        # For each time serie in case
        for ts in case.ts:
            # And for each dimension
            for dimension in dimensions:
                # Get dimension information
                dim_location_sensor_id = dimension[2]

                # If timeseries is for the same sensor as dimension
                if ts.location_sensor_id == dim_location_sensor_id:
                    dim_weights = get_weights(dim_location_sensor_id, prediction_horizon)
                    dim_is_avg = dimension[3]
                    dim_type = dimension[0]
                    # If avg, mean dimension will be computed later
                    if dim_is_avg == 'avg':
                        if dim_type == WATER_DISCHARGE_LEVEL:
                            vector_dim_wdl.append(ts.ts_values)
                            weights_dim_wdl.append(dim_weights)
                        elif dim_type == RAINFALL_FORECAST:
                            vector_dim_rf.append(ts.ts_values)
                            weights_dim_rf.append(dim_weights)
                        elif dim_type == RAINFALL_GAUGE:
                            vector_dim_rg.append(ts.ts_values)
                            weights_dim_rg.append(dim_weights)
                    # If not, a new dimension will be added
                    elif dim_is_avg != 'avg':
                        dimension_list.append(Dimension(dim_type, dim_location_sensor_id, dim_weights,
                                                        list(map(self.map_function, ts.ts_values))))

        # Convert avg dimensions into single vector values and weights
        if len(vector_dim_wdl) > 0:
            single_wdl_vector = self.concatenate_vector(vector_dim_wdl)
            single_wdl_weights = self.concatenate_weights(weights_dim_wdl)
            dimension_list.append(Dimension(WATER_DISCHARGE_LEVEL, -1, single_wdl_weights, single_wdl_vector))
        if len(vector_dim_rg) > 0:
            single_rg_vector = self.concatenate_vector(vector_dim_rg)[0]
            single_rg_weights = self.concatenate_weights(weights_dim_rg)[0]
            dimension_list.append(Dimension(RAINFALL_GAUGE, -1, single_rg_weights, single_rg_vector))
        if len(vector_dim_rf) > 0:
            single_rf_vector = self.concatenate_vector(vector_dim_rg)[0]
            single_rf_weights = self.concatenate_weights(weights_dim_rf)[0]
            dimension_list.append(Dimension(RAINFALL_FORECAST, -1, single_rf_weights, single_rf_vector))

        # Compute distance for each dimension
        for dimension in dimension_list:
            sensor_type = dimension.sensor_type
            distance = 0
            if sensor_type == WATER_DISCHARGE_LEVEL:
                distance = self.calculate_distance(dimension.values, current_vector_wdl, np.amax(dimension.values),
                                                   dimension.weights)
            elif sensor_type == RAINFALL_GAUGE:
                distance = self.calculate_distance(dimension.values, current_vector_rg, np.amax(dimension.values),
                                                   dimension.weights)
            elif sensor_type == RAINFALL_FORECAST:
                distance = self.calculate_distance(dimension.values, current_vector_rf, np.amax(dimension.values),
                                                   dimension.weights)
            dimension.distance = distance
        return dimension_list

    def map_function(self, value):
        return value.value

    # Calculate distance value
    def calculate_distance(self, vector, current_vector, vector_max, weights):
        sum = 0
        p = 2
        weight_tuple_position = 1
        for i in range(0, len(vector)):
            if len(current_vector) <= 1:
                if len(current_vector[0]) > 1:
                    sum += weights[i][weight_tuple_position] * (
                            (abs((current_vector[0][i] - vector[i]) / vector_max) ** p) / weights[i][
                        weight_tuple_position])
            else:
                if len(weights) > 1:
                    sum += weights[i][weight_tuple_position] * (
                            (abs((current_vector[i] - vector[i]) / vector_max) ** p) / weights[i][
                        weight_tuple_position])
                else:
                    sum += weights[weight_tuple_position] * (
                            (abs((current_vector[i] - vector[i]) / vector_max) ** p) / weights[
                        weight_tuple_position])
        return sum

    # Transforms TS_Values vector into single vector created with means or single vector itself id length is 1
    def concatenate_vector(self, vector):
        array = np.array(vector)
        if len(array) < 0:
            print("fail")
            return "fail"
        elif len(array) == 1:
            return np.array(get_ts_values(array))
        else:
            ts_values = get_ts_values(array)
            return list(np.mean(ts_values, axis=0))

    def concatenate_weights(self, weights):
        array = np.array(weights)
        if len(array) < 0:
            print("fail")
            return "fail"
        elif len(array) == 1:
            return array
        else:
            result = []
            for i in range(0, len(array[0])):
                result.append((i + 1, np.mean(array)))
            return result
