from application.cbr.selection_criteria_strat.selection_criteria_strategy import SelectionCriteriaStrategy
import numpy as np

from database.case_queries import get_load_line, get_sensor_type_for_sensor_id


class ParetoDominanceStrategy(SelectionCriteriaStrategy):

    def __init__(self, selection_mode, prediction_horizon):
        self.selection_mode = selection_mode
        self.prediction_horizon = prediction_horizon

    def apply_criteria_numpy(self, location_id, prediction_horizon, dimension_list, cases):
        self.compute_distance_to_load_line_numpy(location_id, prediction_horizon, dimension_list)
        size = len(dimension_list[0].distance)
        # Comparations matrix for picking best case
        comparations = np.zeros((size, size))
        for i in range(size):
            for j in range(size):
                if self.dominate_numpy(dimension_list, i, j):
                    comparations[i][j] = 1
                    comparations[j][i] = -1
                elif self.dominate_numpy(dimension_list, j, i):
                    comparations[j][i] = 1
                    comparations[i][j] = -1

        distances = self.compute_pareto_load_line_distance_numpy(dimension_list)
        best_cases = [k for k in range(len(comparations)) if self.is_best_case(comparations[k])]
        [self.set_distance(case, distances[k]) for case, k in zip(cases, range(len(cases)))]

        selected_cases = self.selection_mode.select(np.asarray(cases)[best_cases])

        return selected_cases

    def set_distance(self, case, distance):
        case.distance_value = distance

    def compute_pareto_load_line_distance_numpy(self, dimension_list):
        dimension_sum = 0
        for dim in dimension_list:
            dimension_sum += (dim.distance - dim.load_line_value) ** 2
        dimension_sums = np.asarray(dimension_sum)
        return np.sqrt(dimension_sums)
        # Update distance_value and load_line_distance.
        # Load line because is for pareto strat
        # Distance value because it is used for selection threshold strategy filter
        # self.distance_value = sqrt(dimension_sum)
        # self.load_line_distance = sqrt(dimension_sum)

    def compute_distance_to_load_line_numpy(self, location_id, prediction_horizon, dimension_list):
        load_line = []
        case_load_line = get_load_line(location_id, prediction_horizon)
        dim_load_line = []
        for ll in case_load_line:
            id_position = 3
            value_position = 4
            ll_sensor_type = get_sensor_type_for_sensor_id(ll[id_position])[0]
            ll_value = ll[value_position]
            idx = list(map(lambda x: x.sensor_type, dimension_list)).index(ll_sensor_type)
            dimension_list[idx].load_line_value = ll_value
        for dim in dimension_list:
            dim_load_line.append(dim.load_line_value)
        load_line.append(dim_load_line)
        return load_line

    def apply_criteria(self, cases):
        # Get load line. [[],[],[]] List for each case. Each case will have stored load_line_values for its dimension
        self.compute_distance_to_load_line(cases)
        # Comparations matrix for picking best case
        comparations = np.zeros((len(cases), len(cases)))
        i = 0
        j = 0
        # Iterate over every case.
        for a in cases:
            for b in cases:
                # Case a dominate case b
                if self.dominate(a, b):
                    comparations[i][j] = 1
                    comparations[j][i] = -1
                # Case b dominate case a
                elif self.dominate(b, a):
                    comparations[j][i] = 1
                    comparations[i][j] = -1
                j += 1
            i += 1
            j = 0
        k = 0
        best_cases = []
        for case in cases:
            case.compute_pareto_load_line_distance()
            if self.is_best_case(comparations[k]):
                best_cases.append(case)
            k += 1
        return self.selection_mode.select(best_cases)

    def dominate_numpy(self, dimension_list, i, j):
        if self.first_condition_numpy(dimension_list, i, j):
            if self.second_condition_numpy(dimension_list, i, j):
                return True
            else:
                return False
        else:
            return False


    def dominate(self, a, b):
        if self.first_condition(a, b):
            if self.second_condition(a, b):
                return True
            else:
                return False
        else:
            return False


    def first_condition_numpy(self, dimension_list, i, j):
        for dim in dimension_list:
            if dim.distance[i] > dim.distance[j]:
                return False
        return True

    def second_condition_numpy(self, dimension_list, i, j):
        for dim in dimension_list:
            if dim.distance[i] >= dim.distance[j]:
                return False
        return True

    def first_condition(self, a, b):
        for (a_dimension, b_dimension) in zip(a.dimensions, b.dimensions):
            if a_dimension.distance > b_dimension.distance:
                return False
        return True

    def second_condition(self, a, b):
        for (a_dimension, b_dimension) in zip(a.dimensions, b.dimensions):
            if a_dimension.distance >= b_dimension.distance:
                return False
        return True

    # Compute distance factored
    def compute_distance_to_load_line(self, cases):
        location_id = cases[0].location_id
        load_line = []
        case_load_line = get_load_line(location_id, self.prediction_horizon)
        for case in cases:
            dim_load_line = []
            for ll in case_load_line:
                ll_sensor_type = get_sensor_type_for_sensor_id(ll[2])[0]
                ll_value = ll[3]
                idx = list(map(lambda x: x.sensor_type, case.dimensions)).index(ll_sensor_type)
                case.dimensions[idx].load_line_value = ll_value
            for dim in case.dimensions:
                dim_load_line.append(dim.load_line_value)
            case.load_line = dim_load_line
            load_line.append(dim_load_line)
        return load_line

    def is_best_case(self, values):
        is_best = True
        for value in values:
            if value < 0:
                is_best = False
        return is_best
