import numpy as np
import json
from database.case_queries import get_wdl_ts, get_rg_ts, get_simulation_values
from services.init_data_service import LocationEnum, load_cases_np, init_variables
from services.cbr_service import compute_map
import pandas as pd
from skimage import io, metrics, transform

WDL_HIGH_VALUE = 0.1255
# WDL_HIGH_VALUE = 0.1555
RG_HIGH_VALUE = 0.038308
# RG_HIGH_VALUE = 0.068308

WDL_LOW_VALUE = 0.0
RG_LOW_VALUE = 0.0

start_gid_list = [6, 716, 1347, 1981, 3467, 4493, 5498, 6145, 6752, 7601, 8228]
end_gid_list = [707, 1338, 1972, 3458, 4484, 5489, 6136, 6743, 7592, 8219, 9224]
simulation_name_list = ['11-12', '21-22', '31-32', '41-42-43', '51-52-53', '61', '71', '81-82', '91-92', '101',
                        '111-112']


def get_simulation(i):
    return simulation_name_list[i]


def get_image_name(simulation_id):
    return "simul_class_" + str(simulation_id).zfill(4) + ".tif"


def get_simulation_id(last_value, value, combinations_with_values):
    return combinations_with_values[value][combinations_with_values[value]['value_x'] ==
                                           combinations_with_values[value]['value_x'].nlargest(2).iloc[last_value]][
        'simulation_id'].values[0]


if __name__ == '__main__':
    print("ey!")
    init_variables()
    # Defino los casos según tabla de pruebas y asigno imagenes a cada caso
    wdl_ts_pd = pd.DataFrame(get_wdl_ts())
    rg_ts_pd = pd.DataFrame(get_rg_ts())

    # High values according to defined table.
    high_wdl_ts_pd = wdl_ts_pd.loc[wdl_ts_pd[2] >= WDL_HIGH_VALUE]
    high_wdl_ts_pd = high_wdl_ts_pd.rename(columns={0: 'simulation_id', 1: 'timestamp', 2: 'value'})
    high_rg_ts_pd = rg_ts_pd.loc[rg_ts_pd[2] >= RG_HIGH_VALUE]
    high_rg_ts_pd = high_rg_ts_pd.rename(columns={0: 'simulation_id', 1: 'timestamp', 2: 'value'})

    # Normal values according to defined table
    normal_wdl_ts_pd = wdl_ts_pd.loc[(wdl_ts_pd[2] < WDL_HIGH_VALUE) & (wdl_ts_pd[1] > WDL_LOW_VALUE)]
    normal_wdl_ts_pd = normal_wdl_ts_pd.rename(columns={0: 'simulation_id', 1: 'timestamp', 2: 'value'})
    normal_rg_ts_pd = rg_ts_pd.loc[(rg_ts_pd[2] < RG_HIGH_VALUE) & (rg_ts_pd[1] >= RG_LOW_VALUE)]
    normal_rg_ts_pd = normal_rg_ts_pd.rename(columns={0: 'simulation_id', 1: 'timestamp', 2: 'value'})

    # Low values according to defined table
    low_wdl_ts_pd = wdl_ts_pd.loc[wdl_ts_pd[2] <= WDL_LOW_VALUE]
    low_wdl_ts_pd = low_wdl_ts_pd.rename(columns={0: 'simulation_id', 1: 'timestamp', 2: 'value'})
    low_rg_ts_pd = rg_ts_pd.loc[rg_ts_pd[2] <= RG_LOW_VALUE]
    low_rg_ts_pd = low_rg_ts_pd.rename(columns={0: 'simulation_id', 1: 'timestamp', 2: 'value'})

    combinations = {}
    value_names = []
    # Combine list in order to get images to fill table.
    # alto alto alto
    high_high_high_pd = pd.merge(high_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner')
    combinations['high-high-high'] = high_high_high_pd
    value_names.append('high-high-high')
    # alto alto medio
    high_high_normal_pd = pd.merge(pd.merge(high_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner'),
                                   normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['high-high-normal'] = high_high_normal_pd
    value_names.append('high-high-normal')

    # alto alto bajo
    high_high_low_pd = pd.merge(pd.merge(high_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner'), low_rg_ts_pd,
                                on='simulation_id', how='inner')
    combinations['high-high-low'] = high_high_low_pd
    value_names.append('high-high-low')

    # alto medio alto
    high_normal_high_pd = pd.merge(pd.merge(high_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                   high_rg_ts_pd, on='simulation_id', how='inner')
    combinations['high-normal-high'] = high_normal_high_pd
    value_names.append('high-normal-high')

    # alto medio medio
    high_normal_normal_pd = pd.merge(pd.merge(high_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                     normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['high-normal-normal'] = high_normal_normal_pd
    value_names.append('high-normal-normal')
    # alto medio bajo
    high_normal_low_pd = pd.merge(pd.merge(high_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                  low_rg_ts_pd, on='simulation_id', how='inner')
    combinations['high-normal-low'] = high_normal_low_pd
    value_names.append('high-normal-low')
    # alto bajo alto
    high_low_high_pd = pd.merge(pd.merge(high_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'), high_rg_ts_pd,
                                on='simulation_id', how='inner')
    combinations['high-low-high'] = high_low_high_pd
    value_names.append('high-low-high')
    # alto bajo medio
    high_low_normal_pd = pd.merge(pd.merge(high_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'),
                                  normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['high-low-normal'] = high_low_normal_pd
    value_names.append('high-low-normal')
    # alto bajo bajo
    high_low_low_pd = pd.merge(pd.merge(high_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'), low_rg_ts_pd,
                               on='simulation_id', how='inner')
    combinations['high-low-low'] = high_low_low_pd
    value_names.append('high-low-low')
    # medio alto alto
    normal_high_high_pd = pd.merge(pd.merge(normal_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner'),
                                   high_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal-high-high'] = normal_high_high_pd
    value_names.append('normal-high-high')
    # medio alto medio
    normal_high_normal_pd = pd.merge(pd.merge(normal_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner'),
                                     normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal-high-normal'] = normal_high_normal_pd
    value_names.append('normal-high-normal')
    # medio alto bajo
    normal_high_low_pd = pd.merge(pd.merge(normal_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner'),
                                  low_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal-high-low'] = normal_high_low_pd
    value_names.append('normal-high-low')

    # medio medio alto
    normal_normal_high_pd = pd.merge(pd.merge(normal_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                     high_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal-normal-high'] = normal_normal_high_pd
    value_names.append('normal-normal-high')

    # medio medio medio
    normal_normal_normal_pd = pd.merge(pd.merge(normal_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                       normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal_normal_normal'] = normal_normal_normal_pd
    value_names.append('normal_normal_normal')

    # medio medio bajo
    normal_normal_low_pd = pd.merge(pd.merge(normal_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                    low_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal_normal_low'] = normal_normal_low_pd
    value_names.append('normal_normal_low')

    # medio bajo alto
    normal_low_high_pd = pd.merge(pd.merge(normal_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'),
                                  high_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal_low_high'] = normal_low_high_pd
    value_names.append('normal_low_high')

    # medio bajo medio
    normal_low_normal_pd = pd.merge(pd.merge(normal_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'),
                                    normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal_low_normal'] = normal_low_normal_pd
    value_names.append('normal_low_normal')

    # medio bajo bajo
    normal_low_low_pd = pd.merge(pd.merge(normal_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'),
                                 low_rg_ts_pd, on='simulation_id', how='inner')
    combinations['normal_low_low'] = normal_low_low_pd
    value_names.append('normal_low_low')

    # bajo alto alto
    low_high_high_pd = pd.merge(pd.merge(low_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner'),
                                high_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_high_high'] = low_high_high_pd
    value_names.append('low_high_high')

    # bajo alto medio
    low_high_normal_pd = pd.merge(pd.merge(low_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner'),
                                  normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_high_normal'] = low_high_normal_pd
    value_names.append('low_high_normal')

    # bajo alto bajo
    low_high_low_pd = pd.merge(pd.merge(low_wdl_ts_pd, high_rg_ts_pd, on='simulation_id', how='inner'),
                               low_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_high_low'] = low_high_low_pd
    value_names.append('low_high_low')

    # bajo medio alto
    low_normal_high_pd = pd.merge(pd.merge(low_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                  high_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_normal_high'] = low_normal_high_pd
    value_names.append('low_normal_high')

    # bajo medio medio
    low_normal_normal_pd = pd.merge(pd.merge(low_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                    normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_normal_normal'] = low_normal_normal_pd
    value_names.append('low_normal_normal')

    # bajo medio bajo
    low_normal_low_pd = pd.merge(pd.merge(low_wdl_ts_pd, normal_rg_ts_pd, on='simulation_id', how='inner'),
                                 low_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_normal_low'] = low_normal_low_pd
    value_names.append('low_normal_low')

    # bajo bajo alto
    low_low_high_pd = pd.merge(pd.merge(low_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'),
                               high_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_low_high'] = low_low_high_pd
    value_names.append('low_low_high')

    # bajo bajo medio
    low_low_normal_pd = pd.merge(pd.merge(low_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'),
                                 normal_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_low_normal'] = low_low_normal_pd
    value_names.append('low_low_normal')

    # bajo bajo bajo
    low_low_low_pd = pd.merge(pd.merge(low_wdl_ts_pd, low_rg_ts_pd, on='simulation_id', how='inner'),
                              low_rg_ts_pd, on='simulation_id', how='inner')
    combinations['low_low_low'] = low_low_low_pd
    value_names.append('low_low_low')

    combinations_with_values = {}
    image_for_combination = {}
    value_names_with_values = []
    for value in value_names:
        if len(combinations[value]) > 0:
            combinations_with_values[value] = combinations[value]
            value_names_with_values.append(value)
            last_value = -1
            simulation_id = get_simulation_id(last_value, value, combinations_with_values)

            image_for_combination[value] = get_image_name(simulation_id)
    # WDL (ALTO) ; RG (ALTO) ; RF (ALTO) ; IMAGEN (simul_class_0055.tif)
    print(image_for_combination)
    # Genero x casos de prueba a partir de las simulaciones que cumplan tabla
    # Casos con load_cases_np

    # cases_cambo = load_cases_np(LocationEnum.CAMBO_LES_BAINS.value)
    # print(cases_cambo)

    # Casos con simulaciones (mas casos, pero hay que formatearlos)
    # a = get_simulation_values(7)
    # print(a)

    ssim_values = {}
    mse_values = {}
    for i in range(1):
        # start_gid = start_gid_list[i]
        start_gid = 750
        # end_gid = end_gid_list[i]
        end_gid = 775

        idx = start_gid
        while idx < end_gid:
            print("Index: ", idx)
            ssim_values[idx] = {}
            mse_values[idx] = {}
            for pred_hor in range(1, 6):
                print("prediction horizon: ", pred_hor)
                compute_map("Cambo les Bains", pred_hor, "Automatic", "_Test_" + str(idx),
                            # a=None, b=None, c=None, simulation="11-12"get_simulation(i), gid=idx)
                            a=None, b=None, c=None, simulation="11-12", gid=idx)
                img1 = io.imread(
                    'static/images/' + 'Cambo les BainsAutomatic_Test_' + str(idx) + str(pred_hor) + '.tif',
                    as_gray=True)
                img2 = io.imread('static/images/fr/' + get_image_name(idx), as_gray=True)
                img1 = transform.resize(img1, img2.shape, anti_aliasing=True)

                ssim = metrics.structural_similarity(img1, img2, data_range=img1.max() - img1.min())
                mse = np.mean((img1 - img2) ** 2)
                ssim_values[idx][pred_hor] = ssim
                mse_values[idx][pred_hor] = mse
            idx += 1
    print("------------ SSIM Values -------------")
    print(ssim_values)

    print("------------ MSE Values -------------")
    print(mse_values)
    with open('ssim.json', 'w') as f:
        json.dump(ssim_values, f)

    with open('mse.json', 'w') as f:
        json.dump(mse_values, f)
    # Hago media de imágenes para representar combinación de casos?
    # Elijo una de forma pseudoaleatorio?
    # Comparo imagen de CBR para caso generado frente a imagen tabla esperada
    # Scikit-image MSE
