from application.cbr.cbrImplementation import CBRImplementation
from database.case_queries import get_conf_location_id
from model.Enums import get_location_enum_value
from services.cbr_service import get_current_case_dimensions, create_current_case_numpy_array
from services.init_data_service import get_location_variables, init_variables
import numpy as np
from sklearn.metrics import accuracy_score, recall_score, f1_score, precision_score

# location sensor id 10 -> Vega C23 -> WDL
# location sensor id 11 -> pending -> Rainfall gauge
# location sensor id 9 -> ParatronicLPN8/1 -> WDL
# location sensor id 15 -> FR ForecastVega C23 -> forecast


WDL_HIGH_VALUE = 0.1255
RG_HIGH_VALUE = 0.038308


def categorize_case(case):
    wdl = False
    for i in range(0, len(case.ts)):
        values = []
        for value in case.ts[i].ts_values:
            values.append(value.value)
        if case.ts[i].location_sensor_id == 10:
            wdl = categorize_time_series("wdl", values) and wdl
        elif case.ts[i].location_sensor_id == 9:
            wdl = categorize_time_series("wdl", values) and wdl
        elif case.ts[i].location_sensor_id == 11:
            rg = categorize_time_series("rg", values)
        elif case.ts[i].location_sensor_id == 15:
            rf = categorize_time_series("rf", values)
    return [wdl, rg, rf]

    return 0

def categorize_time_series(type, values):
    if type == "wdl":
        if np.mean(values) > WDL_HIGH_VALUE:
            return True
        return False
    else:
        if np.mean(values) > RG_HIGH_VALUE:
            return True
        return False


if __name__ == '__main__':
    location_name = "Cambo les Bains"
    location_id = get_conf_location_id(location_name)
    location = get_location_enum_value("Cambo les Bains")

    init_variables()
    results = {}
    for gid in range(6, 707, 1):
        results[gid] = {}
        for pred_hor in range(1, 7):
            results[gid][pred_hor] = {}
            cases_ids, cases_numpy_array, weights_numpy_array, dimension_list, cases, images_path = get_location_variables(
                "Cambo les Bains", pred_hor)
            cbr = CBRImplementation(location, pred_hor, "Automatic", "_Test_",
                                    None, None, None, "11-12", gid)
            current_case = cbr.get_current_case()
            current_vector_wdl, current_vector_rg, current_vector_rf = get_current_case_dimensions(
                current_case.model_dimensions)
            current_case_numpy_array = create_current_case_numpy_array(current_vector_wdl, current_vector_rg,
                                                                       current_vector_rf)
            # ------------------ Retrieve ------------------
            # Compute distance between cases and current case
            computed_distance = cbr.compute_distance_strategy.compute_distance_numpy_arrays(current_case_numpy_array,
                                                                                            cases_numpy_array,
                                                                                            weights_numpy_array,
                                                                                            dimension_list, location_id)
            ids, distances = cases_ids, computed_distance

            # Filter cases
            filtered_cases = cbr.selection_criteria_strategy.apply_criteria_numpy(location_id, pred_hor,
                                                                                  dimension_list, cases)

            filtered = np.array(list(filtered_cases))

            if len(filtered) > 5:
                filtered = filtered[:5]

            true_labels = [categorize_time_series("wdl", current_vector_wdl), categorize_time_series("rg", current_vector_rg), categorize_time_series("rf", current_vector_rf)]
            predicted = []
            for filtered_case in filtered:
                predicted.append(categorize_case(filtered_case))

            # predicted_flat = [val for sublist in predicted for val in sublist]
            true_labels = [true_labels] * len(predicted)
            accuracy = accuracy_score(true_labels, predicted)
            recall = recall_score(true_labels, predicted, average="samples")
            f1 = f1_score(true_labels, predicted, average="samples")
            precision = precision_score(true_labels, predicted, average="samples")
            results[gid][pred_hor]['accuracy'] = accuracy
            results[gid][pred_hor]['recall'] = recall
            results[gid][pred_hor]['f1'] = f1
            results[gid][pred_hor]['precision'] = precision

