import json
import numpy as np
with open('results.json') as f:
    data = json.load(f)

# Collect all the metrics across different evaluations
accuracy_list = []
recall_list = []
f1_list = []
for model_evals in data.values():
    for eval_metrics in model_evals.values():
        accuracy_list.append(eval_metrics['accuracy'])
        recall_list.append(eval_metrics['recall'])
        f1_list.append(eval_metrics['f1'])

# Calculate the mean of each metric
mean_accuracy = np.mean(accuracy_list)
stdev_accuracy = np.nanstd(accuracy_list)
quantiles_accuracy = np.nanpercentile(accuracy_list, [25, 50, 75])

mean_recall = np.mean(recall_list)
stdev_recall = np.nanstd(recall_list)
quantiles_recall = np.nanpercentile(recall_list, [25, 50, 75])

mean_f1 = np.mean(f1_list)
stdev_f1 = np.nanstd(f1_list)
quantiles_f1 = np.nanpercentile(f1_list, [25, 50, 75])

print("Mean accuracy: ", mean_accuracy)
print("STD accuracy: ", stdev_accuracy)
print("Quantiles accuracy: ", quantiles_accuracy)

print("Mean recall: ", mean_recall)
print("STD recall: ", stdev_recall)
print("Quantiles recall: ", quantiles_recall)

print("Mean F1-score: ", mean_f1)
print("STD F1-score: ", stdev_f1)
print("Quantiles F1-score: ", quantiles_f1)

keys_order = ["1", "2", "3", "4", "5"]
metrics = ["accuracy", "recall", "f1"]
# Compute the mean for each key in the order defined
results = {}
mse_results = {}
means = {}
mse_means = {}
for key in keys_order:
    accuracy_values = [data[k][key]['accuracy'] for k in data]
    recall_values = [data[k][key]['recall'] for k in data]
    f1_values = [data[k][key]['f1'] for k in data]
    results[key] = {}
    results[key]['accuracy'] = {}
    results[key]['recall'] = {}
    results[key]['f1'] = {}
    results[key]['accuracy']['mean'] = np.mean(accuracy_values)
    results[key]['accuracy']['std'] = np.nanstd(accuracy_values)
    results[key]['accuracy']['quantiles'] = np.nanpercentile(accuracy_values, [25, 50, 75])
    results[key]['recall']['mean'] = np.mean(recall_values)
    results[key]['recall']['std'] = np.nanstd(recall_values)
    results[key]['recall']['quantiles'] = np.nanpercentile(recall_values, [25, 50, 75])
    results[key]['f1']['mean'] = np.mean(f1_values)
    results[key]['f1']['std'] = np.nanstd(f1_values)
    results[key]['f1']['quantiles'] = np.nanpercentile(f1_values, [25, 50, 75])

print("a")
