import os

import psycopg2
import json
from database.config import config


def get_connection():
    # read connection parameters
    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    params = config(os.path.join(BASE_DIR, "static", "database.ini"), section="postgresql")

    # connect to the PostgreSQL server
    conn = psycopg2.connect(**params)
    return conn


def close_connection(connection):
    if connection:
        connection.close()


def get_queries():
    BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))
    queries = os.path.join(BASE_DIR, "static", "queries.json")
    with open(queries, 'r') as j:
        contents = json.loads(j.read())
    return contents


def get_conf_location_id(location_name):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_conf_location_id"], {'location_name': location_name})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_cases(location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_cases"], {'location_id': location_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_ts(case_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_ts"], {'case_id': case_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_ts_no_params():
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_ts_no_params"])
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_ts_values(ts_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_ts_values"], {'ts_id': ts_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_ts_values_no_params():
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_ts_values_no_params"])
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_sensors(location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_sensors"], {'location_id': location_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_sensor_aggregation(sensor_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_sensor_aggregation"], {'sensor_id': sensor_id})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_number_of_intervals(sensor_type_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_number_of_intervals"], {'sensor_type_id': sensor_type_id})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_sensor_type(sensor_type_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_sensor_type"], {'sensor_type_id': sensor_type_id})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_sensor_type(sensor_type_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_sensor_aggregation_id"], {'sensor_type_id': sensor_type_id})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_sensor_type_for_sensor_id(location_sensor_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_sensor_type_for_sensor_id"], {'location_sensor_id': location_sensor_id})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_time_horizon_configuration(location_id, prediction_horizon):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_time_horizon_configuration"],
                                  {'location_id': location_id, 'prediction_horizon': prediction_horizon})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_prorated_sensor(location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_prorated_sensor"], {'location_id': location_id})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_prorated_sensor_type(location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_prorated_sensor_type"], {'location_id': location_id})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_sigmoid(location_id, prediction_horizon):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_sigmoid"],
                                  {'location_id': location_id, 'prediction_horizon': prediction_horizon})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_shift(location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_shift"], {'location_id': location_id})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_load_line(location_id, prediction_horizon):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_load_line"],
                                  {'location_id': location_id, 'prediction_horizon': prediction_horizon})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_weights(location_sensor_id, prediction_horizon):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_weights"],
                                  {'location_sensor_id': location_sensor_id, 'prediction_horizon': prediction_horizon})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_dimension(location_id, aggregation_type=""):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                if aggregation_type == "":
                    db_cursor.execute(queries["get_dimension_not_avg"],
                                      {'location_id': location_id})
                else:
                    db_cursor.execute(queries["get_dimension"],
                                      {'aggregation_type': aggregation_type, 'location_id': location_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_map(case_id, prediction_horizon):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                name = get_map_name(case_id, prediction_horizon)[0]
                my_path = os.path.abspath(os.path.dirname(__file__))
                path_to_dir = os.path.join(my_path, "..", "static", "images", name)
                db_cursor.execute(queries["get_map_blob"],
                                  {'path': path_to_dir, 'case_id': case_id, 'prediction_horizon': prediction_horizon})
                response = db_cursor.fetchone()
                lo = conn.lobject(response[0])
                lo.export(path_to_dir)
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_map_info(case_id, prediction_horizon):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_map"],
                                  {'case_id': case_id, 'prediction_horizon': prediction_horizon})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_map_name(case_id, prediction_horizon):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_map_name"],
                                  {'case_id': case_id, 'prediction_horizon': prediction_horizon})
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


# Cambo Current Case Queries

def get_simulation_values(gid):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                min_gid = int(gid) - 6
                max_gid = int(gid) + 3
                queries = get_queries()
                db_cursor.execute(queries["get_simulation"],
                                  {'min_gid': min_gid, 'max_gid': max_gid})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


# -----------------------    Venero Claro Current Case Queries    ----------------------------------

def get_venero_claro_rainfall_gauge(intervals):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_venero_claro_rainfall_gauge_timeserie"],
                                  {'intervals': intervals})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_venero_claro_water_discharge_level(intervals):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_venero_claro_water_discharge_level_timeserie"],
                                  {'intervals': intervals})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_number_of_rain_gauge_sensors():
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_number_of_rain_gauge_sensors"])
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_number_of_rain_gauge_sensors_cambo():
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_number_of_rain_gauge_sensors_cambo"])
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_number_of_water_discharge_level_sensors_cambo():
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_number_of_water_discharge_level_sensors_cambo"])
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_number_of_water_discharge_level_sensors():
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_number_of_water_discharge_level_sensors"])
                response = db_cursor.fetchone()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


# NP

def get_cases_np(location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_cases_np"], {'location_id': location_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_weights_np(location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_weights_np"], {'location_id': location_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_dimension_intervals_np(location_sensor_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_dimension_intervals_np"], {'location_sensor_id': location_sensor_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_dimension_intervals_avg_np(conf_sensor_type_id, conf_location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_dimension_intervals_avg_np"],
                                  {'conf_sensor_type_id': conf_sensor_type_id,
                                   'conf_location_id': conf_location_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_avg_sensor_id_np(conf_sensor_type, location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_avg_sensor_id_np"], {'conf_sensor_type': conf_sensor_type,
                                                                    'location_id': location_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_weights_avg_np(location_sensor_id, prediction_horizon):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_weights_avg_np"], {'location_sensor_id': location_sensor_id,
                                                                  'prediction_horizon': prediction_horizon})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def is_avg_np(conf_sensor_type_id, conf_location_id):
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_is_avg_np"], {'conf_sensor_type_id': conf_sensor_type_id,
                                                             'conf_location_id': conf_location_id})
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


# Test queries
def get_wdl_ts():
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_wdl_ts"])
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response


def get_rg_ts():
    conn = None
    response = None
    try:
        conn = get_connection()
        with conn:
            with conn.cursor() as db_cursor:
                queries = get_queries()
                db_cursor.execute(queries["get_rg_ts"])
                response = db_cursor.fetchall()
            db_cursor.close()
    except (Exception, psycopg2.DatabaseError) as error:
        print(error)
    finally:
        if conn is not None:
            close_connection(conn)
        if response is not None:
            return response
