import numpy as np
from numpy import sqrt

from application.cbr.distance_strat.distance_helper import get_ts_values
from database.case_queries import get_ts, get_map, get_map_info
from model.cbr_constants import WATER_DISCHARGE_LEVEL, RAINFALL_GAUGE, RAINFALL_FORECAST
from model.map import Map
from model.ts import TS


class Case:
    def __init__(self, case_id, location_id):
        self.case_id = case_id
        self.location_id = location_id
        self.ts = []
        # Distance values initialized with high value
        self.distance_value = 10000
        self.distance_wdl = 10000
        self.distance_rg = 10000
        self.distance_rf = 10000

        self.dimensions = []
        self.load_line = []

        self.load_line_distance = 0

        self.map = 0
        # Map shift
        self.weight_retrieved_case = 0

    def __str__(self):
        return "Case with id {0} for location {1}".format(self.case_id, self.location_id)

    def get_ts(self):
        {self.ts.append(TS(x, y, self.case_id)) for x, y, z in get_ts(self.case_id)}

    def set_distance_wdl(self, value):
        self.distance_wdl = value

    def set_distance_rg(self, value):
        self.distance_rg = value

    def set_distance_rf(self, value):
        self.distance_rf = value


    # Compute distance for pareto strategy. Distance = sqrt((dim_distance-load_line_dim_value)**2)
    def compute_pareto_load_line_distance(self):
        dimension_sum = 0
        for dim in self.dimensions:
            dimension_sum += (dim.distance - dim.load_line_value) ** 2
        # Update distance_value and load_line_distance.
        # Load line because is for pareto strat
        # Distance value because it is used for selection threshold strategy filter
        self.distance_value = sqrt(dimension_sum)
        self.load_line_distance = sqrt(dimension_sum)

    def load_map(self, prediction_horizon):
        map = get_map_info(self.case_id, prediction_horizon)
        self.map = Map(map[0], map[2], map[3], self.location_id, map[1])
        # Download map from db. Wrong in server as maps are saved on /static/images/ folder
        get_map(self.case_id, prediction_horizon)


    # Numpy
    def map_function(self, value):
        return value.value

    def concatenate_vector(self, vector):
        array = np.array(vector)
        if len(array) < 0:
            print("fail")
            return "fail"
        elif len(array) == 1:
            return np.array(get_ts_values(array))
        else:
            ts_values = get_ts_values(array)
            return list(np.mean(ts_values, axis=0))

    def concatenate_weights(self, weights):
        array = np.array(weights)
        if len(array) < 0:
            print("fail")
            return "fail"
        elif len(array) == 1:
            return array
        else:
            result = []
            for i in range(0, len(array[0])):
                result.append((i + 1, np.mean(array)))
            return result

    def to_numpy_array(self, dimension_list):
        current_vector_wdl = []
        current_vector_rg = []
        current_vector_rf = []

        for dimension in dimension_list:
            if dimension.sensor_type == WATER_DISCHARGE_LEVEL:
                current_vector_wdl = current_vector_wdl + dimension.values
            elif dimension.sensor_type == RAINFALL_GAUGE:
                current_vector_rg = current_vector_rg + dimension.values
            elif dimension.sensor_type == RAINFALL_FORECAST:
                current_vector_rf = current_vector_rf + dimension.values
        return np.concatenate((current_vector_wdl, current_vector_rg, current_vector_rf))

    def case_weights_to_numpy_array(self, dimension_list):
        current_weights_wdl = []
        current_weights_rg = []
        current_weights_rf = []

        for dimension in dimension_list:
            if dimension.sensor_type == WATER_DISCHARGE_LEVEL:
                dimension.weights.sort()
                dimension.weights = [x[0] for x in dimension.weights]
                current_weights_wdl = current_weights_wdl + dimension.weights
            elif dimension.sensor_type == RAINFALL_GAUGE:
                dimension.weights.sort()
                dimension.weights = [x[0] for x in dimension.weights]
                current_weights_rg = current_weights_rg + dimension.weights
            elif dimension.sensor_type == RAINFALL_FORECAST:
                dimension.weights.sort()
                dimension.weights = [x[1] for x in dimension.weights]
                current_weights_rf = current_weights_rf + dimension.weights
        return np.concatenate((current_weights_wdl, current_weights_rg, current_weights_rf))