from application.cbr.distance_strat.distance_helper import get_dimensions
from database.case_queries import get_sensors, get_simulation_values, get_dimension_intervals_np, \
    get_dimension_intervals_avg_np, get_avg_sensor_id_np
from model.cbr_constants import WATER_DISCHARGE_LEVEL, RAINFALL_FORECAST, RAINFALL_GAUGE
from model.current_case import Current_case
from model.dimension import Dimension
from model.sensor import Sensor
import numpy as np


class Current_case_cambo_les_bains(Current_case):
    def __init__(self, location_id, hour, simulation, gid):
        self.location_id = location_id
        self.hour = hour
        self.simulation = simulation
        self.gid = gid
        self.dimensions = get_dimensions(self.location_id)
        self.sensors = get_sensors(self.location_id)
        self.model_dimensions = self.get_model_dimensions_from_TS()

    def get_location_name(self):
        return self.location_name

    def get_model_dimensions_from_TS(self):
        model_sensors = []
        for sensora in self.sensors:
            model_sensors.append(Sensor(sensora[0], sensora[1], sensora[2], sensora[3], sensora[4], sensora[5]))
        dimension_model_list = []
        simulation_rows = get_simulation_values(self.gid)

        for dimension in self.dimensions:
            sensor_type = dimension[0]
            sensor_id = dimension[2]
            sensor_aggregation_type = dimension[3]
            if sensor_type == RAINFALL_GAUGE:
                number_of_intervals = self.get_number_of_intervals(sensor_id)[0]
                number_of_sensors = 1
                # Current ts values
                rain_gauge_values = []
                for i in range(number_of_sensors):
                    only_values_list = []
                    for simulation_row in simulation_rows:
                        only_values_list.append(simulation_row[0])
                    rain_gauge_values.append(only_values_list[3:number_of_intervals+3])
                if sensor_aggregation_type == "avg":
                    mean_values = np.mean(rain_gauge_values, axis=0)
                    n_intervals = get_dimension_intervals_avg_np(RAINFALL_GAUGE, self.location_id)
                    sensor_id = get_avg_sensor_id_np(RAINFALL_GAUGE, self.location_id)
                    n_sensor = len(sensor_id)
                    dimension_model_list.append(Dimension(n_intervals, n_sensor, sensor_type, dimension[2], [], mean_values))
                else:
                    dimension_model_list.append(Dimension(number_of_intervals, 1, sensor_type, dimension[2], [], rain_gauge_values[0]))
            elif sensor_type == WATER_DISCHARGE_LEVEL:
                number_of_intervals = self.get_number_of_intervals(sensor_id)[0]
                number_of_sensors = 1
                water_discharge_level_values = []
                for i in range(number_of_sensors):
                    only_values_list = []
                    for simulation_row in simulation_rows:
                        # i + 1+i valores de los sensores de lluvia en la simulación
                        only_values_list.append(simulation_row[i + (1+i)])
                    water_discharge_level_values.append(only_values_list[0:number_of_intervals])
                if sensor_aggregation_type == "avg":
                    mean_values = list(np.mean(water_discharge_level_values, axis=0))
                    n_intervals = get_dimension_intervals_avg_np(WATER_DISCHARGE_LEVEL, self.location_id)
                    sensor_id = get_avg_sensor_id_np(WATER_DISCHARGE_LEVEL, self.location_id)
                    n_sensor = len(sensor_id)
                    dimension_model_list.append(Dimension(n_intervals, n_sensor, sensor_type, dimension[2], [], mean_values))
                else:
                    dim_intervals = get_dimension_intervals_np(sensor_id)[0][0]
                    dimension_model_list.append(
                        Dimension(dim_intervals, 1, sensor_type, dimension[2], [], water_discharge_level_values[0]))
            # elif sensor_type == "Rainfall Forecast":
            elif sensor_type == RAINFALL_FORECAST:
                number_of_intervals = self.get_number_of_intervals(sensor_id)[0]
                only_values_list = []
                for simulation_row in simulation_rows:
                    only_values_list.append(simulation_row[0])
                forecast = only_values_list[6:number_of_intervals+6]
                dim_intervals = get_dimension_intervals_np(sensor_id)[0][0]
                dimension_model_list.append(
                    Dimension(dim_intervals, 1, sensor_type, dimension[2], [], forecast))
        return dimension_model_list
