from application.api.weather import get_venero_claro_weather_forecast
from application.cbr.distance_strat.distance_helper import get_dimensions
from application.cbr.manual_weather.manual_weather import get_manual_weather_forecast
from database.case_queries import get_venero_claro_rainfall_gauge, get_venero_claro_water_discharge_level, get_sensors,\
    get_number_of_rain_gauge_sensors, get_number_of_water_discharge_level_sensors, get_avg_sensor_id_np, \
    get_dimension_intervals_avg_np
from model.cbr_constants import WATER_DISCHARGE_LEVEL, RAINFALL_GAUGE, RAINFALL_FORECAST
from model.current_case import Current_case
from model.dimension import Dimension
from model.sensor import Sensor
import numpy as np


class Current_case_venero_claro(Current_case):
    def __init__(self, location_id, prediction_horizon, hour, weather_type, a, b, c):
        self.location_id = location_id
        self.prediction_horizon = prediction_horizon
        self.weather_type = weather_type
        self.a = a
        self.b = b
        self.c = c
        self.hour = hour
        self.dimensions = get_dimensions(self.location_id)
        self.sensors = get_sensors(self.location_id)
        self.model_dimensions = self.get_model_dimensions_from_TS()

    def get_location_name(self):
        return self.location_name

    def get_prediction_horizon(self):
        return self.prediction_horizon

    def get_model_dimensions_from_TS(self):
        model_sensors = []
        for sensora in self.sensors:
            model_sensors.append(Sensor(sensora[0], sensora[1], sensora[2], sensora[3], sensora[4], sensora[5]))
        dimension_model_list = []
        for dimension in self.dimensions:
            sensor_type = dimension[0]
            sensor_id = dimension[2]
            sensor_aggregation_type = dimension[3]
            if sensor_type == RAINFALL_GAUGE:
                number_of_intervals = self.get_number_of_intervals(sensor_id)[0]
                number_of_sensors = get_number_of_rain_gauge_sensors()[0]
                ts_rainfall_gauge = get_venero_claro_rainfall_gauge(number_of_intervals * number_of_sensors)
                rain_gauge_values = []
                for i in range(0, number_of_sensors):
                    new_list = ts_rainfall_gauge[i * number_of_intervals:i * number_of_intervals + number_of_intervals]
                    only_values_list = []
                    for ts_value in new_list:
                        only_values_list.append(ts_value[2])
                    rain_gauge_values.append(only_values_list)
                if sensor_aggregation_type == "avg":
                    mean_values = np.mean(rain_gauge_values, axis=0)
                    sensor_id = get_avg_sensor_id_np(RAINFALL_GAUGE, self.location_id)
                    n_sensor = len(sensor_id)
                    dimension_model_list.append(Dimension(number_of_intervals, n_sensor, sensor_type, dimension[2], [], mean_values))
                else:
                    dimension_model_list.append(Dimension(number_of_intervals, 1, sensor_type, dimension[2], [], rain_gauge_values))
            elif sensor_type == WATER_DISCHARGE_LEVEL:
                number_of_intervals = self.get_number_of_intervals(sensor_id)[0]
                number_of_sensors = get_number_of_water_discharge_level_sensors()[0]
                ts_water_discharge_level = get_venero_claro_water_discharge_level(
                    number_of_intervals * number_of_sensors)
                water_discharge_level_values = []
                for i in range(0, number_of_sensors):
                    new_list = ts_water_discharge_level[
                               i * number_of_intervals:i * number_of_intervals + number_of_intervals]
                    only_values_list = []
                    for ts_value in new_list:
                        only_values_list.append(ts_value[0])
                    water_discharge_level_values.append(only_values_list)
                if sensor_aggregation_type == "avg":
                    mean_values = np.mean(water_discharge_level_values, axis=0)
                    sensor_id = get_avg_sensor_id_np(WATER_DISCHARGE_LEVEL, 3)
                    n_intervals = get_dimension_intervals_avg_np(WATER_DISCHARGE_LEVEL, 3)[0][0]
                    n_sensor = len(sensor_id)
                    dimension_model_list.append(Dimension(n_intervals, n_sensor, sensor_type, dimension[2], [], mean_values))
                else:
                    dimension_model_list.append(
                        Dimension(number_of_intervals, 1, sensor_type, dimension[2], [], water_discharge_level_values[0]))
            elif sensor_type == RAINFALL_FORECAST:
                if self.weather_type == "Automatic":
                    forecast = get_venero_claro_weather_forecast()
                    forecast = self.get_forecast_time_series(forecast)
                else:
                    forecast = get_manual_weather_forecast(self.weather_type, self.a, self.b, self.c)
                dimension_model_list.append(
                    Dimension(len(forecast), 1, sensor_type, dimension[2], [], forecast))

        return dimension_model_list

    def get_forecast_time_series(self, forecast):
        # Next
        if int(self.hour) + self.prediction_horizon > 23:
            next_index = int(self.hour) + self.prediction_horizon - 23
        else:
            next_index = int(self.hour) + self.prediction_horizon
        # Last
        if int(self.hour) + 1 + self.prediction_horizon > 23:
            last_index = next_index + 1 - 23
        else:
            last_index = next_index + 1

        current_hour_forecast = 0
        next_hour_forecast = forecast[1]["precipitacion"][next_index]["value"]
        last_hour_forecast = forecast[1]["precipitacion"][last_index]["value"]

        next_hour_forecast = self.check_value(forecast, next_hour_forecast, next_index)
        last_hour_forecast = self.check_value(forecast, last_hour_forecast, last_index)
        return self.linear_interpolate(float(current_hour_forecast), next_hour_forecast, last_hour_forecast)

    def check_value(self, forecast, forecast_value, index_value):
        while str(forecast_value).lower() == "ip":
            if len(forecast[1]["precipitacion"]) > index_value + 1:
                index_value = index_value + 1
            else:
                index_value = 0
            forecast_value = forecast[1]["precipitacion"][index_value]["value"]
        return float(forecast_value)

    def linear_interpolate(self, init, next, end):
        x = [0, 6, 12]
        y = [init, next, end]
        interpolate_time_serie = []
        for i in range(0, 9):
            y_new = np.interp(i, x, y)
            interpolate_time_serie.append(y_new)

        return interpolate_time_serie
