import os
from flask import jsonify, send_from_directory, make_response, request, Flask, Blueprint

from data.shared_data import shared_data
from model.Enums import WeatherEnum, LocationEnum
from services.cbr_service import compute_map
from services.init_data_service import init_variables

from database.case_queries import get_map_info, get_map

from model.map import Map

# Setting flask blueprint

my_routes = Blueprint('my_routes', __name__)

# Load values from database
@my_routes.route('/update', methods=['GET'])
def update():
    print("NEW CBR ENDPOINT CALLED: " + "/update")
    init_variables()


# Get simulations from database
@my_routes.route('/simulation', methods=['GET'])
def get_french_images():
    print("NEW CBR ENDPOINT CALLED: " + "/simulation")
    cases_ids = []
    for case_id in cases_ids:
        for i in range(0, 6):
            map = get_map_info(case_id, i)
            map = Map(map[0], map[2], map[3], "Cambo les Bains", map[1])
            get_map(case_id, i)


# Get image from static images folder
@my_routes.route('/static/images/<string:name>.tif', methods=['GET'])
def get_image(name):
    print("NEW CBR ENDPOINT CALLED: " + "/static/images/" + name)
    save_filename = name + ".tif"
    img_path = os.path.join(shared_data["CLIENT_IMAGES"], save_filename)
    if os.path.exists(img_path):
        response = make_response(
            send_from_directory(directory=shared_data["CLIENT_IMAGES"], path=save_filename, as_attachment=True),
            {'mimeType': 'image/tiff'})
        response.headers.add('Access-Control-Allow-Origin', '*')
        return response
    else:
        print("There was an error loading image. Image does not exist.")
        return jsonify({'msg': 'Error'})


# Main input. Do cbr process and return generated filenames
@my_routes.route('/cbr/input', methods=['GET'])
def main_input():
    print("NEW CBR ENDPOINT CALLED: " + "/cbr/input")
    # Get params
    location = request.args.get("location")
    hour = request.args.get("hour")
    weather_type = request.args.get("weather")
    prediction_horizon = int(request.args.get("horizon"))
    # Init filenames array to be returned
    filenames = []

    if location == LocationEnum.VENERO_CLARO.value:
        # Append cbr result to array
        if weather_type == WeatherEnum.AUTOMATIC.value:
            filenames.append(compute_map(location, prediction_horizon, weather_type, hour))
        else:
            a = request.args.get("a")
            b = request.args.get("b")
            c = request.args.get("c")
            filenames.append(compute_map(location, prediction_horizon, weather_type, hour, a=a, b=b, c=c))
    else:
        simulation = request.args.get("simulation")
        gid = request.args.get("gid")

        filenames.append(compute_map(location, prediction_horizon, weather_type, hour, simulation=simulation, gid=gid))

    response = make_response(jsonify({'code': '200', 'filenames': filenames}))
    # CORS
    response.headers.add('Access-Control-Allow-Origin', '*')
    return response


