import numpy as np

from application.cbr.distance_strat.distance_helper import get_ts_values
from data.shared_data import shared_data
from database.case_queries import get_cases, get_conf_location_id, get_ts_no_params, get_ts_values_no_params, \
    get_dimension, get_weights, get_dimension_intervals_np, get_avg_sensor_id_np, \
    get_dimension_intervals_avg_np, get_weights_avg_np
from model.Enums import LocationEnum
from model.case import Case
from model.cbr_constants import WATER_DISCHARGE_LEVEL, RAINFALL_FORECAST, RAINFALL_GAUGE
from model.dimension import Dimension
from model.ts import TS
from model.ts_value import TS_value


def get_cases_global(location_name):
    cases = []
    {cases.append(Case(x, y)) for x, y in get_cases(get_conf_location_id(location_name=location_name)[0])}
    return cases


# Add ts to cases and to values list
def add_ts_np(case, x, y, z):
    case.ts.append(TS(x, y, z))


# Add ts values to ts and to values list
def add_ts_value_np(ts, values, y, z, k):
    ts.ts_values.append(TS_value(ts.ts_id, y, z, k))
    values.append(y)


# Load cases and return a tuple with a numpy array and cases list object
def load_cases_np(location_name):
    cases_list = []
    cases = get_cases_global(location_name)
    ts_all = get_ts_no_params()
    ts_values_all = get_ts_values_no_params()

    for case in cases:
        values = []
        case_ts = filter(lambda x: x[2] == case.case_id, ts_all)
        {add_ts_np(case, x, y, z) for x, y, z in list(case_ts)}
        for ts in case.ts:
            case_ts_values = filter(lambda x: x[0] == ts.ts_id, ts_values_all)
            {add_ts_value_np(ts, values, y, z, k) for x, y, z, k in list(case_ts_values)} # El :18 porque hay un valor mal en bd que saca 23 para una lista

        if location_name == LocationEnum.CAMBO_LES_BAINS.value:
            cases_list.append(values[:18])
        else:
            cases_list.append(values)
    return [np.array(cases_list), cases]


def get_dimensions(location_id):
    dimensions = []
    avgs = get_dimension(location_id, 'Avg')
    for dim in avgs:
        dim = dim + ('avg',)
        dimensions.append(dim)
    no_avgs = get_dimension(location_id)
    for dim in no_avgs:
        dim = dim + ('no_avg',)
        dimensions.append(dim)
    dimensions.sort(reverse=True)
    return dimensions


def map_function(value):
    return value.value


def concatenate_vector(vector):
    array = np.array(vector)
    if len(array) < 0:
        print("fail")
        return "fail"
    elif len(array) == 1:
        return np.array(get_ts_values(array))
    else:
        ts_values = get_ts_values(vector)
        return list(np.mean(ts_values, axis=0))


def concatenate_weights(weights):
    array = np.array(weights)
    if len(array) < 0:
        print("fail")
        return "fail"
    elif len(array) == 1:
        return array
    else:
        result = []
        for i in range(0, len(array[0])):
            result.append((i + 1, np.mean(array)))
        return result


def get_dimension_list(prediction_horizon, location_name, case, dimensions):
    # Each dim will have values to compute distance
    vector_dim_wdl = []
    vector_dim_rg = []
    vector_dim_rf = []
    # Each dimension with his weigths is stored in this array
    dimension_list = []
    # For each time serie in case
    for ts in case.ts:
        # And for each dimension
        for dimension in dimensions:

            # Get dimension information
            dim_location_sensor_id = dimension[2]

            # If timeseries is for the same sensor as dimension
            if ts.location_sensor_id == dim_location_sensor_id:
                dim_weights = get_weights(dim_location_sensor_id, prediction_horizon)

                dim_is_avg = dimension[3]
                dim_type = dimension[0]
                dim_intervals = get_dimension_intervals_np(dim_location_sensor_id)[0][0]
                # If avg, mean dimension will be computed later
                if dim_is_avg == 'avg':
                    if dim_type == WATER_DISCHARGE_LEVEL:
                        vector_dim_wdl.append(ts.ts_values)
                    elif dim_type == RAINFALL_FORECAST:
                        vector_dim_rf.append(ts.ts_values)
                    elif dim_type == RAINFALL_GAUGE:
                        vector_dim_rg.append(ts.ts_values)
                # If not, a new dimension will be added
                elif dim_is_avg != 'avg':
                    dimension_list.append(Dimension(dim_intervals, 1, dim_type, dim_location_sensor_id, dim_weights,
                                                    list(map(map_function, ts.ts_values))))
    location_id = get_conf_location_id(location_name)
    # Convert avg dimensions into single vector values and weights
    if len(vector_dim_wdl) > 0:
        single_wdl_vector = concatenate_vector(vector_dim_wdl)
        sensor_id = get_avg_sensor_id_np(WATER_DISCHARGE_LEVEL, location_id)
        n_sensor = len(sensor_id)
        single_wdl_weights = get_weights_avg_np(sensor_id[0], prediction_horizon)
        n_intervals = get_dimension_intervals_avg_np(WATER_DISCHARGE_LEVEL, location_id)[0][0]
        dimension_list.append(
            Dimension(n_intervals, n_sensor, WATER_DISCHARGE_LEVEL, -1, single_wdl_weights, single_wdl_vector))
    if len(vector_dim_rg) > 0:
        single_rg_vector = concatenate_vector(vector_dim_rg)
        if len(single_rg_vector) > 0:
            single_rg_vector = single_rg_vector[0]
        sensor_id = get_avg_sensor_id_np(RAINFALL_GAUGE, location_id)
        n_intervals = get_dimension_intervals_avg_np(RAINFALL_GAUGE, location_id)[0][0]

        n_sensor = len(sensor_id)
        single_rg_weights = get_weights_avg_np(sensor_id[0], prediction_horizon)
        dimension_list.append(
            Dimension(n_intervals, n_sensor, RAINFALL_GAUGE, -1, single_rg_weights, single_rg_vector))
    if len(vector_dim_rf) > 0:
        single_rf_vector = concatenate_vector(vector_dim_rf)[0]
        sensor_id = get_avg_sensor_id_np(RAINFALL_FORECAST, location_id)
        n_sensor = len(sensor_id)
        single_rf_weights = get_weights_avg_np(sensor_id[0], prediction_horizon)
        n_intervals = get_dimension_intervals_avg_np(RAINFALL_FORECAST, location_id)[0][0]
        dimension_list.append(
            Dimension(n_intervals, n_sensor, RAINFALL_FORECAST, -1, single_rf_weights, single_rf_vector))
    return dimension_list


def init_variables():
    shared_data["DATA"].cambo = load_cases_np(LocationEnum.CAMBO_LES_BAINS.value)
    shared_data["DATA"].cases_cambo = shared_data["DATA"].cambo[0]
    shared_data["DATA"].cases_cambo_object = shared_data["DATA"].cambo[1]

    shared_data["DATA"].venero_claro = load_cases_np(LocationEnum.VENERO_CLARO.value)
    shared_data["DATA"].cases_venero_claro = shared_data["DATA"].venero_claro[0]
    shared_data["DATA"].cases_venero_claro_object = shared_data["DATA"].venero_claro[1]

    shared_data["DATA"].dimensions_cambo = get_dimensions(get_conf_location_id(LocationEnum.CAMBO_LES_BAINS.value))
    shared_data["DATA"].dimensions_venero_claro = get_dimensions(get_conf_location_id(LocationEnum.VENERO_CLARO.value))

    shared_data["DATA"].dimension_list_cambo = []
    shared_data["DATA"].dimension_list_venero_claro = []
    for i in range(1, 7):
        dim_list = get_dimension_list(i, LocationEnum.CAMBO_LES_BAINS.value, shared_data["DATA"].cases_cambo_object[0],
                                      shared_data["DATA"].dimensions_cambo)
        shared_data["DATA"].dimension_list_cambo.append(dim_list)
        shared_data["DATA"].dimension_list_venero_claro.append(
            get_dimension_list(i, LocationEnum.VENERO_CLARO.value, shared_data["DATA"].cases_venero_claro_object[0],
                               shared_data["DATA"].dimensions_venero_claro))

    shared_data["DATA"].cases_cambo_ids = [case.case_id for case in shared_data["DATA"].cases_cambo_object]
    shared_data["DATA"].cases_venero_claro_ids = [case.case_id for case in
                                                  shared_data["DATA"].cases_venero_claro_object]

    shared_data["DATA"].weights_list_cambo = []
    shared_data["DATA"].weights_list_venero_claro = []
    for i in range(6):
        shared_data["DATA"].weights_list_cambo.append(
            shared_data["DATA"].cases_cambo_object[0].case_weights_to_numpy_array(
                shared_data["DATA"].dimension_list_cambo[i]))
        shared_data["DATA"].weights_list_venero_claro.append(
            shared_data["DATA"].cases_venero_claro_object[0].case_weights_to_numpy_array(
                shared_data["DATA"].dimension_list_venero_claro[i]))


def get_location_variables(location_name, prediction_horizon):
    if location_name == LocationEnum.VENERO_CLARO.value:
        cases_ids = shared_data["DATA"].cases_venero_claro_ids
        cases_numpy_array = shared_data["DATA"].cases_venero_claro
        weights_numpy_array = shared_data["DATA"].weights_list_venero_claro[prediction_horizon - 1]
        dimension_list = shared_data["DATA"].dimension_list_venero_claro[prediction_horizon - 1]
        cases = shared_data["DATA"].cases_venero_claro_object
    else:
        cases_ids = shared_data["DATA"].cases_cambo_ids
        cases_numpy_array = shared_data["DATA"].cases_cambo
        weights_numpy_array = shared_data["DATA"].weights_list_cambo[prediction_horizon - 1]
        dimension_list = shared_data["DATA"].dimension_list_cambo[prediction_horizon - 1]
        cases = shared_data["DATA"].cases_cambo_object
    images_path = shared_data["CLIENT_IMAGES"]
    return cases_ids, cases_numpy_array, weights_numpy_array, dimension_list, cases, images_path
