const express = require('express');
const bp = require('body-parser');
const handlebars = require('express-handlebars');

const log4js = require('log4js');
// modules
const db = require('./modules/queries')
const validations = require("./modules/validations.js");

const path = require('path');

const HandlebarsI18n = require("handlebars-i18n")
HandlebarsI18n.init()
const i18next = require('./modules/i18next');


const app = express();

// handlebars view
const hbs = handlebars.create({
    partialsDir: path.join(__dirname, '/static/partials/'),
    layoutsDir: path.join(__dirname, '/static/layouts/'),
    defaultLayout: 'main',
    extname: 'handlebars'
});
app.engine('handlebars', hbs.engine);

app.set('view engine', 'handlebars');
app.set('views', './static/');

// Configurations
log4js.configure({
    appenders: {logger: {type: 'file', filename: 'logger.log'}},
    categories: {default: {appenders: ['logger'], level: 'info'}}
});
const logger = log4js.getLogger('logger');

// Settings
//Port

app.set('port', 8000)

// Middlewares
app.use(express.static(path.join(__dirname,'/static/')));
app.use(function (req, res, next) {
    res.header("Access-Control-Allow-Origin", "*");
    res.header("Access-Control-Allow-Credentials", "true");
    res.header("Access-Control-Allow-Methods", "POST, GET, DELETE, UPDATE, PUT");
    res.header("Access-Control-Allow-Headers", "Origin, X-Requested-With, Content-Type, Accept, token"); // Debemos especificar todas las headers que se aceptan. Content-Type , token
    next();
});
app.use(bp.json());
app.use(bp.urlencoded({extended:true}))


const bcrypt = require("bcrypt");
const jwt = require('jsonwebtoken');

//session
var expressSession = require('express-session');
app.use(expressSession({
    secret: 'inundatio',
    resave: true,
    saveUninitialized: true
}));
const routerLocale = express.Router()
routerLocale.use(function(req, res, next){
    var locale = req.query.lang
    if (locale) {
        res.set("lang", locale)
        i18next.changeLanguage(locale).then(function() {
            console.log(locale)
        }).catch(function() {
            console.log(locale)
        });
    }
    next()
})
app.use("/*", routerLocale)


// Router token
const routerUserToken = express.Router()
routerUserToken.use(function(req, res, next){
    var token = req.headers['auth-token'] || req.body.token ||req.query.token || req.session.token;
    if (token != null){
        jwt.verify(token, 'inundatio', function(err, infoToken) {
            if (err || (Date.now()/1000 - infoToken.expiresIn) > 240 ){
                res.status(403);
                res.redirect("/login?message=Session is no longer available, try to log in again!&messageType=alert alert-danger alert-dismissible")
            } else {
                res.user = infoToken.user;
                res.user_id = infoToken.user_id;
                next();
            }
        });

    } else {
        res.status(403); // Forbidden
        res.redirect("/login?message=Log in first!&messageType=alert alert-danger alert-dismissible")
    }
})
app.use("/map", routerUserToken)
app.use("/cbrConfiguration", routerUserToken)
app.use("/form", routerUserToken)

// Routes

require("./routes/mapRender.js")(app, logger, path)
require("./routes/cbrRender.js")(app, logger, path)
require("./routes/rapiSensor.js")(app, db, validations, i18next )
require("./routes/rapiSensorType.js")(app, db, validations, i18next)
require("./routes/rapiShift.js")(app, db, validations, i18next)
require("./routes/rapiSigmoid.js")(app, db, validations, i18next)
require("./routes/rapiTimeHorizon.js")(app, db, validations, i18next)
require("./routes/rapiWeights.js")(app, db, validations, i18next)
require("./routes/rapiProratedVariable.js")(app, db, validations, i18next)
require("./routes/rapiLoadLine.js")(app, db, validations, i18next)
const hbs2 = require("handlebars")
require("./routes/rapiLogin.js")(app, db, bcrypt, jwt, hbs2)



// init route
app.get('/', function (req, res) {
    var token = req.headers['auth-token'] || req.body.token ||req.query.token || req.session.token;
    if (token != null){
        jwt.verify(token, 'inundatio', function(err, infoToken) {
            if (err || (Date.now()/1000 - infoToken.expiresIn) > 240 ){
                res.render('partials/widget-login');
            } else {
                res.user = infoToken.user;
                res.user_id = infoToken.user_id;
                res.render('partials/widget-form');
            }
        });

    } else {
        res.render('partials/widget-login');
    }

})

app.use((req, res, next) => {
    res.header('Access-Control-Allow-Origin', '*');
    next();
});
// starting the server
app.listen(app.get('port'), () => {
    console.log('listening on port ' + app.get('port'));
});
