const Pool = require('pg').Pool
var format = require('pg-format')

// Callback hell queries.
// TODO. Refactor into promise or async await queries

// server side
const pool = new Pool()
// local sise

const getUser = (request, response) => {
    const getUserQuery = format('SELECT * FROM user WHERE user.username = %L', request.params.username);
    pool.query(getUserQuery, (error, results) => {
        if (error) {
            response();
        } else {
            response(results.rows);
        }
    })
}

const getSensor = async (request, response) => {
    try {
        const getSensorsDataQuery = `SELECT sensor_identifier, sampling_freq, conf_sensor_type_id, url_access, username, password FROM web_conf_location cl, web_location_sensor ls, web_access_data ad, web_location_sensor_aggregation lsa WHERE ad.location_sensor_id = ls.location_sensor_id AND lsa.location_sensor_aggregation_id = ls.conf_location_sensor_aggregation_id AND ls.conf_location_id = cl.conf_location_id AND cl.name = '${request.params.location}' ORDER BY sensor_identifier`;

        const results = await pool.query(getSensorsDataQuery);

        if (results.rows.length === 0) {
            response({error: 'No data found'});
        } else {
            response(results.rows);
        }
    } catch (error) {
        response({error: 'An error occurred'});
    }
};


// Get sensor types data
const getLocationSensorTypes = async (request, response) => {
    try {
        const getSensorTypesQuery = format('SELECT DISTINCT conf_sensor_type_id FROM web_location_sensor_aggregation lsa, web_conf_location cl WHERE lsa.conf_location_id = cl.conf_location_id AND cl.conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)', request.params.location)
        const results = await pool.query(getSensorTypesQuery);
        if (results.rows.length == 0) {
            response({error: 'No data found'});
        } else {
            response(results.rows);
        }
    } catch (error) {
        response({error: 'An error occurred'});
    }
}

// Get second tab data
const getSensorType = async (request, response) => {
    try {
        const getSensorTypeDataQuery = format('SELECT number_of_intervals, conf_sensor_aggregation_id, conf_sensor_type_id FROM web_location_sensor_aggregation lsa, web_conf_location cl WHERE lsa.conf_location_id = cl.conf_location_id AND cl.name = %L ORDER BY conf_sensor_type_id ASC', request.params.location);

        const results = await new Promise((resolve, reject) => {
            pool.query(getSensorTypeDataQuery, (error, results) => {
                if (error) {
                    reject(error);
                } else {
                    resolve(results.rows);
                }
            });
        });

        response(results);
    } catch (error) {
        response('Internal Server Error');
    }
};


// Get sensor type for a given sensor
const getSensorsToUpdate = async (request, response) => {
    try {
        const getSensorTypeFromSensorQuery = format(
            'SELECT location_sensor_id FROM web_location_sensor ls, web_conf_location cl, web_location_sensor_aggregation lsa WHERE ls.conf_location_sensor_aggregation_id = lsa.location_sensor_aggregation_id AND lsa.conf_sensor_aggregation_id = %L AND ls.conf_location_id = cl.conf_location_id AND cl.name = %L AND lsa.conf_sensor_type_id = (SELECT conf_sensor_type_id FROM web_location_sensor_aggregation lsaa, web_location_sensor lss, web_conf_location cll WHERE lss.conf_location_id = cll.conf_location_id AND cl.name = %L AND lss.sensor_identifier = %L AND lss.conf_location_sensor_aggregation_id = lsaa.location_sensor_aggregation_id);',
            'Avg',
            request.params.location,
            request.params.location,
            request.params.sensor
        );

        const results = await pool.query(getSensorTypeFromSensorQuery);

        if (results.rows.length === 0) {
            const getOneSensorQuery = format(
                "SELECT location_sensor_id FROM web_location_sensor WHERE sensor_identifier = %L",
                request.params.sensor
            );

            const oneSensorResults = await pool.query(getOneSensorQuery);
            response(oneSensorResults.rows);
        } else {
            response(results.rows);
        }
    } catch (error) {
        response();
    }
};


const updateSensorType = async (request, response) => {
    const client = await pool.connect();
    try {
        await client.query('BEGIN');

        const updateQuery = format(
            'UPDATE web_location_sensor_aggregation SET number_of_intervals = %L, conf_sensor_aggregation_id = %L, conf_sensor_type_id = (SELECT type FROM web_conf_sensor_type WHERE type = %L) WHERE conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L) AND number_of_intervals = %L AND conf_sensor_aggregation_id = %L AND conf_sensor_type_id = %L',
            request.sensorTypeHourInterval,
            request.sensorTypeAggregationMode,
            request.sensorTypeType,
            request.location,
            request.oldSensorNumberOfIntervals,
            request.oldSensorAggregation,
            request.oldSensorType
        );

        await client.query(updateQuery);

        if (request.oldSensorNumberOfIntervals !== request.sensorTypeHourInterval) {
            const getPredictionHorizonAndMaxNumberOfIntervalsConf = format(
                'SELECT cph.max_prediction_horizon, cph.max_number_of_intervals FROM web_conf_prediction_horizon cph, web_location_time_horizon lth WHERE lth.conf_prediction_horizon_id = cph.conf_prediction_horizon_id AND lth.conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L);',
                request.location
            );

            const predictionHorizonAndMaxNumberOfIntervalsResult = await client.query(getPredictionHorizonAndMaxNumberOfIntervalsConf);

            const getSensorsToUpdateWeightsQuery = format(
                'SELECT location_sensor_id FROM web_location_sensor WHERE conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L) AND conf_location_sensor_aggregation_id = (SELECT location_sensor_aggregation_id FROM web_location_sensor_aggregation WHERE conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L) AND conf_sensor_type_id = %L)',
                request.location,
                request.location,
                request.sensorTypeType
            );

            const location_sensor_ids = await client.query(getSensorsToUpdateWeightsQuery);

            const getLocationIdQuery = format('SELECT conf_location_id FROM web_conf_location WHERE name = %L', request.location);
            const conf_location_id_results = await client.query(getLocationIdQuery);

            if (parseInt(request.sensorTypeHourInterval) > parseInt(request.oldSensorNumberOfIntervals)) {
                let nestList = [];
                for (let i = 0; i < predictionHorizonAndMaxNumberOfIntervalsResult.rows[0].max_prediction_horizon; i++) {
                    for (let j = parseInt(request.oldSensorNumberOfIntervals); j < parseInt(request.sensorTypeHourInterval); j++) {
                        for (let k = 0; k < location_sensor_ids.rows.length; k++) {
                            nestList.push([parseInt(j) + 1, parseInt(i + 1), conf_location_id_results.rows[0].conf_location_id, location_sensor_ids.rows[k].location_sensor_id, 0.1]);
                        }
                    }
                }
                let insertString = format("INSERT INTO web_weights(interval, prediction_horizon, conf_location_id, location_sensor_id, weight) VALUES %L", nestList);
                let insert_sensor_weights_results = await client.query(insertString);
                await client.query('COMMIT');
                response(insert_sensor_weights_results.rowCount);
            } else {
                let h = parseInt(request.oldSensorNumberOfIntervals);
                let l = 0;
                while (h > parseInt(request.sensorTypeHourInterval)) {
                    while (l < location_sensor_ids.rows.length) {
                        let query = deleteWeightQuery(h, location_sensor_ids.rows[l].location_sensor_id);
                        await client.query(query);
                        l++;
                    }
                    l = 0;
                    h--;
                }
                await client.query('COMMIT');
                response(1);
            }
        } else {
            let results = await client.query('COMMIT');
            response(results.rowCount);
        }
    } catch (error) {
        console.log(error)
        await client.query('ROLLBACK');
        response();
    } finally {
        client.release();
    }
};


const deleteWeightQuery = (interval, sensor) => {
    return format('DELETE FROM web_weights WHERE interval = %L AND location_sensor_id = %L', interval, sensor)
}


const createSensorType = async (request, response) => {
    try {
        const createSensorTypeQuery = format('INSERT INTO web_location_sensor_aggregation(number_of_intervals, conf_location_id, conf_sensor_aggregation_id, conf_sensor_type_id) VALUES (%L, (SELECT conf_location_id FROM web_conf_location WHERE name = %L), %L, %L)', request.sensorTypeHourInterval, request.location, request.sensorTypeAggregationMode, request.sensorTypeType);

        const results = await pool.query(createSensorTypeQuery);
        response(results.rowCount);
    } catch (error) {
        response();
    }
};
const deleteSensorType = async (request, response) => {
    try {
        const deleteSensorQuery = format(
            'DELETE FROM web_location_sensor_aggregation WHERE location_sensor_aggregation_id = (SELECT location_sensor_aggregation_id FROM web_location_sensor_aggregation WHERE number_of_intervals = %L AND conf_sensor_type_id = %L AND conf_sensor_aggregation_id = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L));',
            request.sensorTypeNumberOfIntervals,
            request.sensorTypeType,
            request.sensorTypeAggregationMode,
            request.location
        );

        await pool.query(deleteSensorQuery);

        const deleteSensorTypeQuery = format(
            'DELETE FROM web_location_sensor_aggregation WHERE number_of_intervals = %L AND conf_sensor_type_id = %L AND conf_sensor_aggregation_id =%L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L);',
            request.sensorTypeNumberOfIntervals,
            request.sensorTypeType,
            request.sensorTypeAggregationMode,
            request.location
        );

        await pool.query(deleteSensorTypeQuery);

        let deleted = 1;
        response(deleted);
    } catch (error) {
        response();
    }
};


// Get third tab data
const getTimeHorizonData = async (request, response) => {
    try {
        const getTimeHorizonDataQuery = format('SELECT prediction_horizon, number_of_neighbour, distance_threshold, conf_distance_function, conf_selection_mode, conf_selection_criteria FROM web_location_time_horizon lth, web_conf_location cl WHERE lth.conf_location_id = cl.conf_location_id AND cl.name = %L ORDER BY prediction_horizon ASC', request.params.location)

        let results = await pool.query(getTimeHorizonDataQuery);
        response(results.rows);
    } catch (error) {
        response();
    }
}
const updateTimeHorizon = async (request, response) => {
    try {
        const updateTimeHorizonQuery = format('UPDATE web_location_time_horizon SET prediction_horizon = %L, number_of_neighbour = %L, distance_threshold = %L, conf_distance_function = %L, conf_selection_mode = %L, conf_selection_criteria = %L WHERE conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L) AND prediction_horizon = %L', request.timeHorizonId, request.timeHorizonNeighbour, request.timeHorizonDistanceThreshold, request.timeHorizonDistanceFunction, request.timeHorizonSelectionMode, request.timeHorizonSelectionCriteria, request.location, request.oldTimeHorizon)

        let results = await pool.query(updateTimeHorizonQuery);
        response(results.rowCount);
    } catch (error) {
        response();
    }
}

// Get forth tab data
const getWeightData = async (request, response) => {
    try {
        const getWeightDataQuery = format('SELECT ls.sensor_identifier, lsa.conf_sensor_type_id, w.interval, w.weight FROM web_weights w, web_location_sensor ls, web_location_sensor_aggregation lsa WHERE w.conf_location_id = (SELECT conf_location_id FROM web_conf_location www WHERE www.name = %L) AND ls.conf_location_id = w.conf_location_id AND w.location_sensor_id = ls.location_sensor_id AND lsa.location_sensor_aggregation_id = ls.conf_location_sensor_aggregation_id AND w.prediction_horizon = %L ORDER BY sensor_identifier, interval ASC', request.params.location, request.params.timeHorizon)
        let results = await pool.query(getWeightDataQuery);
        response(results.rows);
    } catch (error) {
        response();
    }
}

function getUpdateWeightQuery(request, i, sensorId) {
    let weightsValues = []
    for (let i = 0; i < request.length; i++) {
        weightsValues.push(request[i])
    }
    return format('UPDATE web_weights SET weight = %L WHERE conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L) AND location_sensor_id = %L AND interval = %L AND prediction_horizon = %L;', weightsValues[i], request.location, sensorId, i + 1, request.predictionHorizon)
}

const updateWeight = async (request, response) => {
    try {
        for (let i = 0; i < request.sensorNumber; i++) {
            let j = 0;
            while (j < request.length) {
                let query = getUpdateWeightQuery(request, j, request.sensors[i])
                await pool.query(query);
                j++
            }
        }
        response(1)
    } catch (error) {
        response(error);
    }
}

const getProratedVariable = async (request, response) => {
    try {
        const getProratedVariableQuery = format('SELECT prorated_variable FROM web_conf_location cl WHERE cl.name = %L', request.params.location)

        let results = await pool.query(getProratedVariableQuery);
        response(results.rows);
    } catch (error) {
        response(error);
    }
}

const getProratedData = async (request, response) => {
    try {
        const getProratedDataQuery = format('SELECT sensor_identifier, conf_sensor_type_id, conf_sensor_aggregation_id FROM web_conf_location cl, web_location_sensor ls, web_location_sensor_aggregation lsa WHERE cl.conf_location_id = ls.conf_location_id AND ls.conf_location_sensor_aggregation_id = lsa.location_sensor_aggregation_id AND cl.name = %L ORDER BY prorated_variable_sensor DESC, sensor_identifier ASC', request.params.location)
        let results = pool.query(getProratedDataQuery);
        response(results.rows);
    } catch (error) {
        response();
    }
}

const removeProratedSensors = async (request, response) => {
    try {
        const removeProratedSensorQuery = format('UPDATE web_location_sensor SET prorated_variable_sensor = false WHERE conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)', request)

        let removeProratedSensorResults = await pool.query(removeProratedSensorQuery);
        response(removeProratedSensorResults);
    } catch (error) {
        response();
    }
}

const addProratedSensors = async (request, response) => {
    try {
        const addProratedSensorQuery = format('UPDATE web_location_sensor SET prorated_variable_sensor = true WHERE conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L) AND conf_location_sensor_aggregation_id = (SELECT location_sensor_aggregation_id FROM web_location_sensor_aggregation WHERE conf_sensor_type_id = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L))', request.location, request.proratedVariable, request.location)

        let addProratedSensorResults = await pool.query(addProratedSensorQuery);
        response(addProratedSensorResults.rowCount);
    } catch (error) {
        response();
    }
}

const updateProratedVariable = async (request, response) => {
    try {
        const updateOldProratedSensor = format('UPDATE web_location_sensor SET prorated_variable_sensor = false WHERE sensor_identifier = %L', request.oldProratedSensorIdentifier)
        await pool.query(updateOldProratedSensor);
        const updateProratedSensor = format('UPDATE web_location_sensor SET prorated_variable_sensor = true WHERE sensor_identifier = %L', request.proratedSensorIdentifier);
        await pool.query(updateProratedSensor);
        const updateProratedVariable = format('UPDATE web_conf_location SET prorated_variable = %L WHERE name = %L', request.proratedVariable, request.location);
        let updateProratedVariableResults = await pool.query(updateProratedVariable);
        response(updateProratedVariableResults.rowCount);
    } catch (error) {
        response();
    }

}

const getSigmoidData = async (request, response) => {
    try {
        const getSigmoidDataQuery = format('SELECT merging_parameter_a, merging_parameter_b, merging_parameter_c, merging_parameter_delta FROM web_location_time_horizon ltc, web_conf_location cl WHERE cl.conf_location_id = ltc.conf_location_id AND ltc.prediction_horizon = %L AND cl.name = %L', request.params.timeHorizon, request.params.location)
        let results = await pool.query(getSigmoidDataQuery);
        response(results.rows);
    } catch (error) {
        response();
    }
}

const updateSigmoid = async (request, response) => {
    try {
        const updateSigmoidQuery = format('UPDATE web_location_time_horizon SET merging_parameter_a = %L, merging_parameter_b = %L,merging_parameter_c = %L,merging_parameter_delta = %L WHERE prediction_horizon = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)', request.sigmoidA, request.sigmoidB, request.sigmoidC, request.sigmoidDelta, request.timeHorizon, request.location)
        let results = await pool.query(updateSigmoidQuery);
        response(results.rows);
    } catch (e) {
        response();
    }
}

const getShiftData = async (request, response) => {
    try {
        const getShiftDataQuery = format('SELECT lh, mh, mhs FROM web_conf_location cl WHERE cl.name = %L', request.params.location)
        let results = await pool.query(getShiftDataQuery);
        response(results.rows);

    } catch (error) {
        response();
    }
}

const updateShift = async (request, response) => {
    try {
        const updateShiftQuery = format('UPDATE web_conf_location SET mhs = %L, mh = %L, lh = %L WHERE name = %L', request.shiftMHS, request.shiftMH, request.shiftLH, request.location)
        let results = await pool.query(updateShiftQuery);
        response(results.rowCount);

    } catch (error) {
        response();
    }
}


// Saves updated sensor data
const updateSensor = async (request, response) => {
    const client = await pool.connect();
    try {
        await client.query('BEGIN');

        // GET locaiton id
        const getLocationIdQuery = format('SELECT conf_location_id FROM web_conf_location WHERE name = %L', request.location)
        let conf_location_id_results = await pool.query(getLocationIdQuery);

        // Obtener agregación
        const getAggregationQuery = format('SELECT location_sensor_aggregation_id FROM web_location_sensor_aggregation WHERE conf_location_id = %L AND conf_sensor_type_id = %L', conf_location_id_results.rows[0].conf_location_id, request.sensorType);
        let conf_location_sensor_aggregation_id_results = pool.query(getAggregationQuery);

        // Get sensor id
        const getLocationSensorIdQuery = format('SELECT location_sensor_id FROM web_location_sensor WHERE sensor_identifier = %L', request.oldSensorIdentifier)
        let location_sensor_id_results = await pool.query(getLocationSensorIdQuery);

        // Update type
        const updateLocationSensorAggregation = format('UPDATE web_location_sensor_aggregation SET conf_sensor_type_id = %L WHERE location_sensor_aggregation_id = %L', request.sensorType, conf_location_sensor_aggregation_id_results.rows[0].location_sensor_aggregation_id)
        await pool.query(updateLocationSensorAggregation);

        // Update sensor
        const updateSensorQuery = format('UPDATE web_location_sensor SET sensor_identifier = %L, sampling_freq = %L, prorated_variable_sensor = %L, conf_location_id = %L, conf_location_sensor_aggregation_id = %L WHERE location_sensor_id = %L', request.sensorId, request.sensorSamplingFreq, 't', conf_location_id_results.rows[0].conf_location_id, conf_location_sensor_aggregation_id_results.rows[0].location_sensor_aggregation_id, location_sensor_id_results.rows[0].location_sensor_id)
        let update_sensor_results = await pool.query(updateSensorQuery);

        // Update web access data
        const updateAccessDataQuery = format('UPDATE web_access_data SET url_access = %L, username = %L, password = %L WHERE web_access_data.location_sensor_id = %L', request.sensorURL, request.sensorUsername, request.sensorPassword, location_sensor_id_results.rows[0].location_sensor_id)
        let access_data_update_results = await pool.query(updateAccessDataQuery);

        // Commit and response
        await client.query('COMMIT');
        response(access_data_update_results.rowCount)
    } catch (error) {
        // If error rollback and response
        await client.query('ROLLBACK');
        response();
    } finally {
        client.release();
    }
}

const createSensor = async (request, response) => {
    const client = await pool.connect();

    try {
        await client.query('BEGIN');


        // Get prediction horizon conf
        const getPredictionHorizonConf = format('SELECT cph.max_prediction_horizon, cph.max_number_of_intervals FROM web_conf_prediction_horizon cph, web_location_time_horizon lth WHERE lth.conf_prediction_horizon_id = cph.conf_prediction_horizon_id AND lth.conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L);', request.location)
        let get_prediction_horizon_conf_results = await pool.query(getPredictionHorizonConf);

        // Get location id query
        const getLocationIdQuery = format('SELECT conf_location_id FROM web_conf_location WHERE name = %L', request.location);
        let conf_location_id_results = await pool.query(getLocationIdQuery);

        // Get aggregation info
        const getAggregationQuery = format('SELECT location_sensor_aggregation_id, conf_sensor_aggregation_id, number_of_intervals FROM web_location_sensor_aggregation WHERE conf_location_id = %L AND conf_sensor_type_id = %L', conf_location_id_results.rows[0].conf_location_id, request.sensorType);
        let conf_location_sensor_aggregation_id_results = await pool.query(getAggregationQuery);

        // Insert sensor data
        const insertLocationSensorQuery = format('INSERT INTO web_location_sensor(sensor_identifier, sampling_freq, prorated_variable_sensor, conf_location_id, conf_location_sensor_aggregation_id) VALUES (%L, %L, %L, %L, %L) RETURNING location_sensor_id', request.sensorId, request.sensorSamplingFreq, 'f', conf_location_id_results.rows[0].conf_location_id, conf_location_sensor_aggregation_id_results.rows[0].location_sensor_aggregation_id);
        let location_sensor_id_results = await pool.query(insertLocationSensorQuery);

        // Insert access data
        const insertAccessDataQuery = format('INSERT INTO web_access_data(url_access, username, password, location_sensor_id) VALUES(%L, %L, %L, %L)', request.sensorURL, request.sensorUsername, request.sensorPassword, location_sensor_id_results.rows[0].location_sensor_id);
        let insert_access_data_query_results = await pool.query(insertAccessDataQuery);


        let aggregationType = conf_location_sensor_aggregation_id_results.rows[0].conf_sensor_aggregation_id;
        let numberOfIntervals = conf_location_sensor_aggregation_id_results.rows[0].number_of_intervals;
        if (aggregationType !== "Avg") {
            var nestList = []
            for (let i = 0; i < get_prediction_horizon_conf_results.rowCount; i++) {
                for (let j = 0; j < numberOfIntervals; j++) {
                    nestList.push([j + 1, i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, 0.1])
                }
                const addDefaultLoadLineQuery = format("INSERT INTO web_load_line VALUES (%L, %L, %L, %L)", i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, 1)
                await pool.query(addDefaultLoadLineQuery);
            }
            var insertString = format("INSERT INTO web_weights(interval, prediction_horizon, conf_location_id, location_sensor_id, weight) VALUES %L", nestList);
            let insert_sensor_weights_results = await pool.query(insertString);
            await client.query('COMMIT');
            response(insert_sensor_weights_results.rowCount);
        } else {
            // Obtener load line
            const getLoadLineValueQuery = format("SELECT wll.value FROM web_load_line wll, web_location_sensor wls, web_location_sensor_aggregation wlsa WHERE wll.location_sensor_id = wls.location_sensor_id AND wls.conf_location_sensor_aggregation_id = wlsa.location_sensor_aggregation_id AND wlsa.conf_sensor_aggregation_id = %L AND wll.conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)", conf_location_sensor_aggregation_id_results.rows[0].conf_sensor_aggregation_id, request.location);
            let results = await pool.query(getLoadLineValueQuery);
            values = results.rows;
            // Obtener pesos
            const selectTypeWeightsQuery = format('SELECT weight FROM web_weights w, web_location_sensor ls  WHERE w.location_sensor_id = ls.location_sensor_id AND ls.conf_location_sensor_aggregation_id = (SELECT location_sensor_aggregation_id FROM web_location_sensor_aggregation WHERE conf_sensor_aggregation_id = %L AND conf_sensor_type_id = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)) ORDER BY prediction_horizon, interval', conf_location_sensor_aggregation_id_results.rows[0].conf_sensor_aggregation_id, request.sensorType, request.location)
            let insert_type_weights_results = await pool.query(selectTypeWeightsQuery);

            let newNestedList = []
            if (insert_type_weights_results.rowCount == 0) {
                for (let i = 0; i < get_prediction_horizon_conf_results.rowCount; i++) {
                    for (let j = 0; j < numberOfIntervals; j++) {
                        newNestedList.push([j + 1, i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, 0.1])
                    }
                    const addDefaultLoadLineQuery = format("INSERT INTO web_load_line VALUES (%L, %L, %L, %L)", i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, values[i]["value"])
                    await pool.query(addDefaultLoadLineQuery)
                }
                let insertNewWeightStringQuery = format("INSERT INTO web_weights(interval, prediction_horizon, conf_location_id, location_sensor_id, weight) VALUES %L", newNestedList)
                let insert_results = await pool.query(insertNewWeightStringQuery)
                await client.query('COMMIT');
                response(insert_results)
            } else {
                weightsRows = insert_type_weights_results.rows
                let weightsRowNestedList = []
                let k = 0
                for (let i = 0; i < get_prediction_horizon_conf_results.rowCount; i++) {
                    for (let j = 0; j < numberOfIntervals; j++) {
                        weightsRowNestedList.push([j + 1, i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, weightsRows[k].weight])
                        k += 1
                    }
                    const addDefaultLoadLineQuery = format("INSERT INTO web_load_line VALUES (%L, %L, %L, %L)", i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, values[i]["value"])
                    await pool.query(addDefaultLoadLineQuery)
                }
                let insertNewWeightStringQuery = format("INSERT INTO web_weights(interval, prediction_horizon, conf_location_id, location_sensor_id, weight) VALUES %L", weightsRowNestedList)
                let insert_results = pool.query(insertNewWeightStringQuery)
                await client.query('COMMIT');
                response(insert_results)
            }
        }
    } catch
        (error) {
        await client.query('ROLLBACK');
        response();
    } finally {
        client.release();
    }
}


// const createSensor = (request, response) => {
//     const getPredictionHorizonConf = format('SELECT cph.max_prediction_horizon, cph.max_number_of_intervals FROM web_conf_prediction_horizon cph, web_location_time_horizon lth WHERE lth.conf_prediction_horizon_id = cph.conf_prediction_horizon_id AND lth.conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L);', request.location)
//     pool.query(getPredictionHorizonConf, (error, get_prediction_horizon_conf_results) => {
//         if (error) {
//             response()
//         } else {
//             const getLocationIdQuery = format('SELECT conf_location_id FROM web_conf_location WHERE name = %L', request.location)
//             pool.query(getLocationIdQuery, (error, conf_location_id_results) => {
//                 if (error) {
//                     response()
//                 } else {
//                     const getAggregationQuery = format('SELECT location_sensor_aggregation_id, conf_sensor_aggregation_id, number_of_intervals FROM web_location_sensor_aggregation WHERE conf_location_id = %L AND conf_sensor_type_id = %L', conf_location_id_results.rows[0].conf_location_id, request.sensorType);
//                     pool.query(getAggregationQuery, (error, conf_location_sensor_aggregation_id_results) => {
//                         if (error) {
//                             response()
//                         } else {
//                             const insertLocationSensorQuery = format('INSERT INTO web_location_sensor(sensor_identifier, sampling_freq, prorated_variable_sensor, conf_location_id, conf_location_sensor_aggregation_id) VALUES (%L, %L, %L, %L, %L) RETURNING location_sensor_id', request.sensorId, request.sensorSamplingFreq, 'f', conf_location_id_results.rows[0].conf_location_id, conf_location_sensor_aggregation_id_results.rows[0].location_sensor_aggregation_id)
//                             pool.query(insertLocationSensorQuery, (error, location_sensor_id_results) => {
//                                     if (error) {
//                                         response()
//                                     } else {
//                                         const insertAccessDataQuery = format('INSERT INTO web_access_data(url_access, username, password, location_sensor_id) VALUES(%L, %L, %L, %L)', request.sensorURL, request.sensorUsername, request.sensorPassword, location_sensor_id_results.rows[0].location_sensor_id)
//                                         pool.query(insertAccessDataQuery, (error, insert_access_data_query_results) => {
//                                             if (error) {
//                                                 response()
//                                             } else {
//                                                 var aggregationType = conf_location_sensor_aggregation_id_results.rows[0].conf_sensor_aggregation_id
//                                                 var numberOfIntervals = conf_location_sensor_aggregation_id_results.rows[0].number_of_intervals
//                                                 if (aggregationType != "Avg") {
//                                                     var nestList = []
//                                                     for (let i = 0; i < get_prediction_horizon_conf_results.rowCount; i++) {
//                                                         for (let j = 0; j < numberOfIntervals; j++) {
//                                                             nestList.push([j + 1, i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, 0.1])
//                                                         }
//                                                         const addDefaultLoadLineQuery = format("INSERT INTO web_load_line VALUES (%L, %L, %L, %L)", i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, 1)
//                                                         pool.query(addDefaultLoadLineQuery, (error, results) => {
//                                                             if (error) {
//                                                                 response();
//                                                             }
//                                                         })
//                                                     }
//                                                     // return results.rowCount
//                                                     var insertString = format("INSERT INTO web_weights(interval, prediction_horizon, conf_location_id, location_sensor_id, weight) VALUES %L", nestList)
//                                                     pool.query(insertString, (error, insert_sensor_weights_results) => {
//                                                         if (error) {
//                                                             response()
//                                                         } else {
//                                                             response(insert_sensor_weights_results.rowCount)
//                                                         }
//                                                     })
//                                                 } else {
//                                                     const getLoadLineValueQuery = format("SELECT wll.value FROM web_load_line wll, web_location_sensor wls, web_location_sensor_aggregation wlsa WHERE wll.location_sensor_id = wls.location_sensor_id AND wls.conf_location_sensor_aggregation_id = wlsa.location_sensor_aggregation_id AND wlsa.conf_sensor_aggregation_id = %L AND wll.conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)", conf_location_sensor_aggregation_id_results.rows[0].conf_sensor_aggregation_id, request.location);
//                                                     pool.query(getLoadLineValueQuery, (error, results) => {
//                                                         if (error) {
//                                                             return -1;
//                                                         } else {
//                                                             values = results.rows
//                                                             const selectTypeWeightsQuery = format('SELECT weight FROM web_weights w, web_location_sensor ls  WHERE w.location_sensor_id = ls.location_sensor_id AND ls.conf_location_sensor_aggregation_id = (SELECT location_sensor_aggregation_id FROM web_location_sensor_aggregation WHERE conf_sensor_aggregation_id = %L AND conf_sensor_type_id = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)) ORDER BY prediction_horizon, interval', conf_location_sensor_aggregation_id_results.rows[0].conf_sensor_aggregation_id, request.sensorType, request.location)
//                                                             pool.query(selectTypeWeightsQuery, (error, insert_type_weights_results) => {
//                                                                 if (error) {
//                                                                     response()
//                                                                 } else {
//                                                                     var newNestedList = []
//                                                                     if (insert_type_weights_results.rowCount == 0) {
//                                                                         for (let i = 0; i < get_prediction_horizon_conf_results.rowCount; i++) {
//                                                                             for (let j = 0; j < numberOfIntervals; j++) {
//                                                                                 newNestedList.push([j + 1, i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, 0.1])
//                                                                             }
//                                                                             const addDefaultLoadLineQuery = format("INSERT INTO web_load_line VALUES (%L, %L, %L, %L)", i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, values[i]["value"])
//                                                                             pool.query(addDefaultLoadLineQuery, (error, results) => {
//                                                                                 if (error) {
//                                                                                     response();
//                                                                                 }
//                                                                             })
//                                                                         }
//                                                                         var insertNewWeightStringQuery = format("INSERT INTO web_weights(interval, prediction_horizon, conf_location_id, location_sensor_id, weight) VALUES %L", newNestedList)
//                                                                         pool.query(insertNewWeightStringQuery, (error, insert_results) => {
//                                                                             if (error) {
//                                                                                 response()
//                                                                             } else {
//                                                                                 response(insert_results)
//                                                                             }
//                                                                         })
//                                                                     } else {
//                                                                         weightsRows = insert_type_weights_results.rows
//                                                                         var weightsRowNestedList = []
//                                                                         let k = 0
//                                                                         for (let i = 0; i < get_prediction_horizon_conf_results.rowCount; i++) {
//                                                                             for (let j = 0; j < numberOfIntervals; j++) {
//                                                                                 weightsRowNestedList.push([j + 1, i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, weightsRows[k].weight])
//                                                                                 k += 1
//                                                                             }
//                                                                             const addDefaultLoadLineQuery = format("INSERT INTO web_load_line VALUES (%L, %L, %L, %L)", i + 1, conf_location_id_results.rows[0].conf_location_id, location_sensor_id_results.rows[0].location_sensor_id, values[i]["value"])
//                                                                             pool.query(addDefaultLoadLineQuery, (error, results) => {
//                                                                                 if (error) {
//                                                                                     response();
//                                                                                 } else {
//
//                                                                                 }
//                                                                             })
//                                                                         }
//                                                                         var insertNewWeightStringQuery = format("INSERT INTO web_weights(interval, prediction_horizon, conf_location_id, location_sensor_id, weight) VALUES %L", weightsRowNestedList)
//                                                                         pool.query(insertNewWeightStringQuery, (error, insert_results) => {
//                                                                             if (error) {
//                                                                                 response()
//                                                                             } else {
//                                                                                 response(insert_results)
//                                                                             }
//                                                                         })
//                                                                     }
//                                                                 }
//                                                             })
//                                                         }
//                                                     })
//                                                 }
//                                             }
//                                         })
//                                     }
//                                 }
//                             )
//                         }
//                     })
//                 }
//             })
//         }
//     })
// }

const deleteSensor = async (request, response) => {
    const client = await pool.connect();
    try {
        await client.query('BEGIN');
        // Select location id
        const selectLocationSensorIdQuery = format('SELECT location_sensor_id FROM web_location_sensor WHERE sensor_identifier = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)', request.sensorId, request.location)
        let select_results = await pool.query(selectLocationSensorIdQuery);
        // Delete access data
        const deleteAccessDataLocationSensorQuery = format('DELETE FROM web_access_data WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id);
        let delete_access_data_results = await pool.query(deleteAccessDataLocationSensorQuery);
        // Delete weights
        const deleteWeightsLocationSensorQuery = format('DELETE FROM web_weights WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
        let delete_weights_results = await pool.query(deleteWeightsLocationSensorQuery);
        // Delete load line
        const deleteLoadLineQuery = format('DELETE FROM web_load_line WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
        let results = pool.query(deleteLoadLineQuery);
        // Delete TS data
        const deleteTSLocationSensorQuery = format('DELETE FROM web_ts WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
        let delete_ts_results = await pool.query(deleteTSLocationSensorQuery);
        // Delete web current TS data
        const deleteCurrentTSLocationSensorQuery = format('DELETE FROM web_current_ts WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
        let delete_current_ts_results = await pool.query(deleteCurrentTSLocationSensorQuery);
        // Check if it is prorated
        const isProratedSensorQuery = format('SELECT ls.prorated_variable_sensor, lsa.conf_sensor_aggregation_id, lsa.conf_sensor_type_id FROM web_location_sensor ls, web_location_sensor_aggregation lsa WHERE location_sensor_id = %L AND ls.conf_location_sensor_aggregation_id = lsa.location_sensor_aggregation_id', select_results.rows[0].location_sensor_id)
        let isProratedSensorResults = await pool.query(isProratedSensorQuery);




    } catch
        (error) {
        await client.query('ROLLBACK');
        response();
    } finally {
        client.release();
    }
}

// const deleteSensor = (request, response) => {
//     // get sensor id
//     const selectLocationSensorIdQuery = format('SELECT location_sensor_id FROM web_location_sensor WHERE sensor_identifier = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)', request.sensorId, request.location)
//     pool.query(selectLocationSensorIdQuery, (error, select_results) => {
//         if (error) {
//             response()
//         } else {
//             // delete from access data
//             const deleteAccessDataLocationSensorQuery = format('DELETE FROM web_access_data WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
//             pool.query(deleteAccessDataLocationSensorQuery, (error, delete_access_data_results) => {
//                 if (error) {
//                     response()
//                 } else {
//                     // delete from weights
//                     const deleteWeightsLocationSensorQuery = format('DELETE FROM web_weights WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
//                     pool.query(deleteWeightsLocationSensorQuery, (error, delete_weights_results) => {
//                         if (error) {
//                             response()
//                         } else {
//                             //delete load line value
//                             const deleteLoadLineQuery = format('DELETE FROM web_load_line WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
//                             pool.query(deleteLoadLineQuery, (error, results) => {
//                                 if (error) {
//                                     response()
//                                 } else {
//                                     // delete from ts
//                                     const deleteTSLocationSensorQuery = format('DELETE FROM web_ts WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
//                                     pool.query(deleteTSLocationSensorQuery, (error, delete_ts_results) => {
//                                         if (error) {
//                                             response()
//                                         } else {
//                                             // delete from current_ts
//                                             const deleteCurrentTSLocationSensorQuery = format('DELETE FROM web_current_ts WHERE location_sensor_id = %L', select_results.rows[0].location_sensor_id)
//                                             pool.query(deleteCurrentTSLocationSensorQuery, (error, delete_current_ts_results) => {
//                                                 if (error) {
//                                                     response()
//                                                 } else {
//                                                     // check if it is prorated sensor
//                                                     const isProratedSensorQuery = format('SELECT ls.prorated_variable_sensor, lsa.conf_sensor_aggregation_id, lsa.conf_sensor_type_id FROM web_location_sensor ls, web_location_sensor_aggregation lsa WHERE location_sensor_id = %L AND ls.conf_location_sensor_aggregation_id = lsa.location_sensor_aggregation_id', select_results.rows[0].location_sensor_id)
//                                                     pool.query(isProratedSensorQuery, (error, isProratedSensorResults) => {
//                                                         if (error) {
//                                                             response()
//                                                         } else {
//                                                             let isProratedItem = isProratedSensorResults.rows
//                                                             if (isProratedItem[0].prorated_variable_sensor) {
//                                                                 if (isProratedItem[0].conf_sensor_aggregation_id != 'Avg') {
//                                                                     const findAnotherProratedSensor = format('SELECT location_sensor_id FROM web_location_sensor ls, web_location_sensor_aggregation lsa WHERE ls.conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L) AND ls.conf_location_sensor_aggregation_id = lsa.location_sensor_aggregation_id AND lsa.conf_sensor_type_id = %L AND ls.location_sensor_id != %L', request.location, isProratedItem[0].conf_sensor_type_id, select_results.rows[0].location_sensor_id)
//                                                                     pool.query(findAnotherProratedSensor, (error, sameAggregationTypeSensorsResults) => {
//                                                                         if (error) {
//                                                                             response()
//                                                                         } else {
//                                                                             if (sameAggregationTypeSensorsResults.rows.length > 0) {
//                                                                                 const updateProratedSensorQuery = format('UPDATE web_location_sensor SET prorated_variable_sensor = true WHERE location_sensor_id = %L', sameAggregationTypeSensorsResults.rows[0].location_sensor_id)
//                                                                                 pool.query(updateProratedSensorQuery, (error, response) => {
//                                                                                     if (error) {
//                                                                                         response()
//                                                                                     } else {
//                                                                                         // delete sensor from location_sensor table
//                                                                                         const deleteSensorQuery = format('DELETE FROM web_location_sensor WHERE sensor_identifier = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)', request.sensorId, request.location)
//                                                                                         pool.query(deleteSensorQuery, (error, results) => {
//                                                                                             if (error) {
//                                                                                                 response()
//                                                                                             } else {
//                                                                                             }
//                                                                                         })
//                                                                                     }
//                                                                                 })
//                                                                             } else {
//                                                                                 response(-1)
//                                                                             }
//                                                                         }
//                                                                     })
//                                                                 } else {
//                                                                     // delete sensor from location_sensor table
//                                                                     const deleteSensorQuery = format('DELETE FROM web_location_sensor WHERE sensor_identifier = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)', request.sensorId, request.location)
//                                                                     pool.query(deleteSensorQuery, (error, results) => {
//                                                                         if (error) {
//                                                                             response()
//                                                                         } else {
//                                                                             response(results.rowCount);
//                                                                         }
//                                                                     })
//                                                                 }
//                                                             } else {
//                                                                 // delete sensor from location_sensor table
//                                                                 const deleteSensorQuery = format('DELETE FROM web_location_sensor WHERE sensor_identifier = %L AND conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L)', request.sensorId, request.location)
//                                                                 pool.query(deleteSensorQuery, (error, results) => {
//                                                                     if (error) {
//                                                                         response()
//                                                                     } else {
//                                                                         response(results.rowCount);
//                                                                     }
//                                                                 })
//                                                             }
//                                                         }
//                                                     })
//                                                 }
//                                             })
//                                         }
//                                     })
//                                 }
//                             })
//                         }
//                     })
//                 }
//             })
//         }
//     })
// }

const getLoadLineData = async (request, response) => {
    try {
        const getLoadLineDataQuery = format('SELECT ls.sensor_identifier, lsa.conf_sensor_aggregation_id,  wll.value, wll.prediction_horizon FROM web_load_line wll, web_location_sensor ls, web_location_sensor_aggregation lsa WHERE wll.conf_location_id = (SELECT conf_location_id FROM web_conf_location www WHERE www.name = %L) AND ls.conf_location_id = wll.conf_location_id AND wll.location_sensor_id = ls.location_sensor_id AND lsa.location_sensor_aggregation_id = ls.conf_location_sensor_aggregation_id ORDER BY wll.prediction_horizon, sensor_identifier ASC', request.params.location)
        let results = await pool.query(getLoadLineDataQuery);
        response(results.rows)

    } catch (error) {
        response();
    }
}

const updateLoadLineData = async (request, response) => {
    try {
        values = []

        for (const value in request) {
            if (!isNaN(value)) {
                values.push(request[value])
            }
        }
        let max_time_horizon = 6 + 1;
        for (i = 1; i < max_time_horizon; i++) {
            var query = getUpdateLoadLineQuery(request, i, values);
            await pool.query(query);
        }
        return response();
    } catch (error) {
        response(error);
    }
}
// function updateLoadLineData(request, response) {
//     values = []
//     for (const value in request) {
//         if (!isNaN(value)) {
//             values.push(request[value])
//         }
//     }
//     let max_time_horizon = 6 + 1
//     for (i = 1; i < max_time_horizon; i++) {
//         var query = getUpdateLoadLineQuery(request, i, values)
//         pool.query(query, (error, results) => {
//             if (error) {
//                 response(error)
//             }
//         })
//     }
//     return response()
// }
function getUpdateLoadLineQuery(request, i, values) {
    return format('UPDATE web_load_line SET value = %L WHERE conf_location_id = (SELECT conf_location_id FROM web_conf_location WHERE name = %L) AND location_sensor_id = (SELECT location_sensor_id from web_location_sensor where sensor_identifier = %L) AND prediction_horizon = %L ;', values[i - 1], request.location, request.sensorId, i)
}

//
function addLoadLineValue(predictionHorizon, confLocationId, locationSensorId, value = 1) {
    const addDefaultLoadLineQuery = format("INSERT INTO web_load_line VALUES (%L, %L, %L, %L)", predictionHorizon, confLocationId, locationSensorId, value)
    pool.query(addDefaultLoadLineQuery, (error, results) => {
        if (error) {
            return -1;
        } else {
            return results.rowCount
        }
    })
}

const findUsername = async (request, response) => {
    try {
        const findUsernameQuery = format("SELECT user_id, username, password FROM web_users WHERE username = (%L)", request);

        let results = await pool.query(findUsernameQuery);
        response(results.rows);
    } catch (error) {
        return null;
    }
}

const insertUser = async (request, response) => {
    try {
        const insertUserQuery = format("INSERT INTO web_users values(2, 'kiketfm22', (%L))", request)
        let results = await pool.query(insertUserQuery);
        response(results);
    } catch (error) {
        return null
    }
}

module.exports = {
    getSensor,
    getLocationSensorTypes,
    updateSensor,
    createSensor,
    deleteSensor,

    getSensorType,
    updateSensorType,
    createSensorType,
    deleteSensorType,

    getTimeHorizonData,
    updateTimeHorizon,

    getSensorsToUpdate,

    getWeightData,
    updateWeight,

    getSigmoidData,
    updateSigmoid,

    getShiftData,
    updateShift,

    getProratedVariable,
    getProratedData,
    removeProratedSensors,
    addProratedSensors,
    updateProratedVariable,

    getLoadLineData,
    updateLoadLineData,


    findUsername,
    insertUser
}