module.exports = {

    validateLoadLineData: function (loadLineData, i18next, functionCallback) {
        for (const value in loadLineData) {
            if (!isNaN(value)) {
                if (loadLineData[value] < 0) {
                    let response = {
                        message: i18next.t("VALIDATION_LOAD_LINE_TIME_HORIZON_INVALID"),
                        messageType: "alert-danger"
                    }
                    functionCallback(response)
                }
            }
        }
        if (loadLineData.sensorId === null || loadLineData.sensorId === undefined) {
            let response = {
                message: i18next.t("VALIDATION_LOAD_LINE_SENSOR_ID_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else {
            functionCallback("")
        }
    },
    validateSensor: function validateSensor(sensor, i18next, functionCallback) {
        if (sensor.sensorType != "Water Discharge Level" && sensor.sensorType != "Rainfall Gauge" && sensor.sensorType != "Rainfall Forecast") {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_SENSOR_TYPE_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (sensor.sensorURL.trim() === "") {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_SENSOR_URL_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (sensor.sensorUsername.trim() === "") {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_SENSOR_USERNAME_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (sensor.sensorPassword.trim() === "") {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_SENSOR_PASSWORD_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (sensor.sensorId.trim() === "") {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_SENSOR_ID_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (sensor.sensorSamplingFreq <= 0) {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_SAMPLING_FREQUENCY_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else functionCallback("")
    },

    validateSensorType: function validateSensorType(sensorTypeItem, i18next, functionCallback) {
        const sensorTypeValue = sensorTypeItem.sensorTypeType
        const sensorIntervalValue = sensorTypeItem.sensorTypeHourInterval
        const sensorAggregationMode = sensorTypeItem.sensorTypeAggregationMode
        if (sensorTypeValue != "Water Discharge Level" && sensorTypeValue != "Rainfall Gauge" && sensorTypeValue != "Rainfall Forecast") {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_TYPE_SENSOR_TYPE_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (isNaN(sensorIntervalValue) || sensorIntervalValue <= 0) {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_TYPE_SENSOR_INTERVAL_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (sensorAggregationMode != "Max" && sensorAggregationMode != "Avg" && sensorAggregationMode != "None" ) {
            let response = {
                message: i18next.t("VALIDATION_SENSOR_TYPE_AGGREGATION_MODE_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else {
            functionCallback("")
        }
    },

    validateTimeHorizon: function validateTimeHorizon(timeHorizonItem, i18next, functionCallback) {
        const timeHorizonId = timeHorizonItem.timeHorizonId
        const timeHorizonNeighbour = timeHorizonItem.timeHorizonNeighbour
        const timeHorizonDistanceThreshold = timeHorizonItem.timeHorizonDistanceThreshold
        const timeHorizonDistanceFunction = timeHorizonItem.timeHorizonDistanceFunction
        const timeHorizonSelectionMode = timeHorizonItem.timeHorizonSelectionMode
        const timeHorizonSelectionCriteria = timeHorizonItem.timeHorizonSelectionCriteria

        if (timeHorizonId.trim() === "") {
            let response = {
                message: i18next.t("VALIDATION_TIME_HORIZON_TIME_HORIZON_ID_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (timeHorizonNeighbour <= 0) {
            let response = {
            message: i18next.t("VALIDATION_TIME_HORIZON_NEIGHBOUR_INVALID"),
            messageType: "alert-danger"
        }
            functionCallback(response)
        }
        else if (timeHorizonDistanceThreshold <= 0) {
            let response = {
                message: i18next.t("VALIDATION_TIME_HORIZON_DISTANCE_THRESHOLD_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (timeHorizonDistanceFunction != "Sobol Indexes" && timeHorizonDistanceFunction != "Weighted Sum") {
            let response = {
                message: i18next.t("VALIDATION_TIME_HORIZON_DISTANCE_FUNCTION_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (timeHorizonSelectionMode != "Number of Neighbour" && timeHorizonSelectionMode != "Distance Threshold") {
            let response = {
                message: i18next.t("VALIDATION_TIME_HORIZON_SELECTION_MODE_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (timeHorizonSelectionCriteria != "Minimizing a Single Criteria"
            && timeHorizonSelectionCriteria != "Pareto Dominance") {
            let response = {
                message: i18next.t("VALIDATION_TIME_HORIZON_SELECTION_CRITERIA_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else { functionCallback("") }
    },

    validateWeights: function validateWeights(weights, i18next, functionCallback) {
        for (const weight in weights) {
            if (weights[weight] < 0) {
                let response = {
                    message: i18next.t("VALIDATION_WEIGHTS_INVALID"),
                    messageType: "alert-danger"
                }
                functionCallback(response)
            }
        }
        functionCallback("")
    },

    validateSigmoid: function validateSigmoid(sigmoid, i18next, functionCallback) {
        const sigmoidA = sigmoid.sigmoidA
        const sigmoidB = sigmoid.sigmoidB
        const sigmoidC = sigmoid.sigmoidC
        const sigmoidDelta = sigmoid.sigmoidDelta

        if (isNaN(sigmoidA) || sigmoidA < 0) {
            let response = {
                message: i18next.t("VALIDATION_SIGMOID_A_PARAMETER_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (isNaN(sigmoidB) || sigmoidB < 0) {
            let response = {
                message: i18next.t("VALIDATION_SIGMOID_B_PARAMETER_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (isNaN(sigmoidC) || sigmoidC < 0) {
            let response = {
                message: i18next.t("VALIDATION_SIGMOID_C_PARAMETER_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (isNaN(sigmoidDelta) || sigmoidDelta < 0) {
            let response = {
                message: i18next.t("VALIDATION_SIGMOID_DELTA_PARAMETER_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else {
            functionCallback("")
        }
    },

    validateShift: function validateShift(shift, i18next, functionCallback) {
        const shiftLH = shift.shiftLH
        const shiftMH = shift.shiftMH
        const shiftMHS = shift.shiftMHS
        if (isNaN(shiftLH) || shiftLH < 0) {
            let response = {
                message: i18next.t("VALIDATION_SHIFT_LH_PARAMETER_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (isNaN(shiftMH) || shiftMH < 0) {
            let response = {
                message: i18next.t("VALIDATION_SHIFT_MH_PARAMETER_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else if (isNaN(shiftMHS) || shiftMHS < 0) {
            let response = {
                message: i18next.t("VALIDATION_SHIFT_MHS_PARAMETER_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else {
            functionCallback("")
        }
    },

    validateProratedVariable: function validateProratedVariable(proratedData, i18next, functionCallback) {
        if (proratedData.proratedVariable != "Water Discharge Level" && proratedData.proratedVariable != "Rainfall Gauge" && proratedData.proratedVariable != "Rainfall Forecast") {
            let response = {
                message: i18next.t("VALIDATION_PRORATED_VARIABLE_INVALID"),
                messageType: "alert-danger"
            }
            functionCallback(response)
        }
        else {
            functionCallback("")
        }
    },
}


