module.exports = function (app, db, validations, i18next) {

    /**
     * API Route: "/api/proratedVariable/:location"
     * API Method: GET
     * Get prorated variable for a given location.
     */
    app.get("/api/proratedVariable/:location", function (req, res) {
        // get data
        db.getProratedVariable(req, function (proratedVariable) {
            // if error
            if (proratedVariable == null) {
                res.status(500);
                res.json({error: i18next.t("API_GET_PRORATED_VARIABLE_ERROR")});
            }
            // if ok
            else {
                res.status(200);
                res.send(JSON.stringify(proratedVariable[0]["prorated_variable"]));
            }
        })
    })

    /**
     * API Route: "/api/proratedData/:location"
     * API Method: GET
     * Get prorated data for a given location.
     */
    app.get("/api/proratedData/:location/:proratedVariable", function (req, res) {
        // get data
        db.getProratedData(req, function (proratedData) {
            // if error
            if (proratedData == null) {
                res.status(500);
                res.json({error: i18next.t("API_GET_PRORATED_VARIABLE_ERROR")});
            }
            // if ok
            else {
                res.status(200);
                res.send(JSON.stringify(proratedData));
            }
        })
    })

    /**
     *  API Route: "/api/proratedVariable"
     *  API Method: PUT
     *  Update prorated variable in the db.
     */
    app.put("/api/updateProratedVariable", function(req, res) {
        // get data
        var proratedSensorIdentifier = req.body.proratedSensorIdentifier
        var oldProratedSensorIdentifier = req.body.oldProratedSensorIdentifier
        var proratedVariable = req.body.proratedVariable
        var location = req.body.location

        // build object
        var proratedData = {
            "proratedSensorIdentifier": proratedSensorIdentifier,
            "oldProratedSensorIdentifier": oldProratedSensorIdentifier,
            "proratedVariable": proratedVariable,
            "location": location
        }
        // validate
        validations.validateProratedVariable(proratedData, i18next,function (response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                // Remove prorated sensors tags
                db.removeProratedSensors(proratedData.location, function(response) {
                    // if ok
                    if (response != null) {
                        // Tag sensors as prorated sensor
                        db.addProratedSensors(proratedData, function(response) {
                            // if ok
                            if (response != null) {
                                // Update prorated variable in db
                                db.updateProratedVariable(proratedData, function (response) {
                                    // if ok
                                    if (response != null) {
                                        res.status(200)
                                        res.json({
                                            message: i18next.t("API_PUT_PRORATED_VARIABLE_SUCCESS"),
                                            messageType: "alert alert-primary alert-dismissible"
                                        })
                                    }
                                    // if error
                                    else {
                                        res.status(400)
                                        res.json({
                                            message: i18next.t("API_PUT_PRORATED_VARIABLE_ERROR"),
                                            messageType: "alert alert-danger alert-dismissible"
                                        })
                                    }
                                })
                            }
                            else {
                                res.status(400)
                                res.json({
                                    message: i18next.t("API_PUT_PRORATED_VARIABLE_ERROR"),
                                    messageType: "alert alert-danger alert-dismissible"
                                })
                            }
                        })
                    }
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_PUT_PRORATED_VARIABLE_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                })
            }
        })
    })
}
