// CRUD for Sensor
module.exports = function (app, db, validations, i18next) {

    /**
     * API Route: "/api/sensor"
     * API Method: POST
     * Save new sensor data in db.
     */
    app.post("/api/sensor/", function(req, res) {
        var sensor = req.body.sensor
        sensor.oldSensorIdentifier = req.body.sensorIdentifier
        sensor.location = req.body.location
        // validate
        validations.validateSensor(sensor, i18next,function (response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                // save in db
                db.createSensor(sensor, function (response) {
                    // if ok
                    if (response != null) {
                        res.status(200)
                        res.json({
                            message: i18next.t("API_SENSOR_POST_SUCCESS"),
                            messageType: "alert alert-primary alert-dismissible"
                        })
                    }
                    // if error
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_POST_SENSOR_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                })
            }
        })
    })

    /**
     * API Route: "/api/sensor/:location"
     * API Method: GET
     * Get sensor data for a given location.
     */
    app.get("/api/sensor/:location", function (req, res) {
        // get data
        db.getSensor(req, function (sensorData) {
            // if error
            if (sensorData == null) {
                res.status(500);
                res.json({error: i18next.t("API_GET_SENSOR_ERROR")});
            }
            // if ok
            else {
                res.status(200);
                res.send(JSON.stringify(sensorData));
            }
        })
    })

    /**
     * API Route: "/api/sensor/types/:location"
     * API Method: GET
     * Get all distinct sensor types for a given location.
     */
    app.get("/api/sensor/types/:location", function (req, res) {
        // get data
        db.getLocationSensorTypes(req, function (types) {
            // if error
            if (types == null) {
                res.status(500)
                res.json({error: i18next.t("API_GET_SENSOR_ERROR")});
            }
            // if ok
            else {
                res.status(200)
                res.json(JSON.stringify(types))
            }
        })
    })

    /**
     * API Route: "/api/sensor/"
     * API Method: PUT
     * Update sensor data.
     */
    app.put("/api/sensor/", function(req, res) {
        var sensor = req.body.sensor
        sensor.oldSensorIdentifier = req.body.sensorIdentifier
        sensor.location = req.body.location
        // validate
        validations.validateSensor(sensor, i18next, function (response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                // update in db
                db.updateSensor(sensor, function (response) {
                    // if ok
                    if (response != null) {
                        res.status(200)
                        res.json({
                            message: i18next.t("API_PUT_SENSOR_SUCCESS"),
                            messageType: "alert alert-primary alert-dismissible"
                        })
                    }
                    // if error
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_PUT_SENSOR_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                })
            }
        })
    })

    /**
     * API Route: "/api/sensor"
     * API Method: DELETE
     * Delete sensor data.
     */
    app.delete("/api/sensor", function (req, res)  {
        // delete
        db.deleteSensor(req.body.sensor, function(response) {
            // if ok
            if (response != null) {
                res.status(200)
                res.json({
                    message: i18next.t("API_DELETE_SENSOR_SUCCESS"),
                    messageType: "alert alert-primary alert-dismissible"
                })
            }
            // if error
            else {
                res.status(400)
                res.json({
                    message: i18next.t("API_DELETE_SENSOR_ERROR"),
                    messageType: "alert alert-danger alert-dismissible"
                })
            }
        })
    })
}