// CRUD for Sensor Type
module.exports = function (app, db, validations, i18next) {

    /**
     * API Route: "/api/sensorType/"
     * API Method: POST
     * Add new sensor type.
     */
    app.post("/api/sensorType/", function (req,res) {
        var sensorType = req.body.sensorType
        sensorType.location = req.body.location
        // validate
        validations.validateSensorType(sensorType, i18next,function (response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                // save
                db.createSensorType(sensorType, function (response) {
                    // if ok
                    if (response != null) {
                        res.status(200)
                        res.json({
                            message: i18next.t("API_POST_SENSOR_TYPE_SUCCESS"),
                            messageType: "alert alert-primary alert-dismissible"
                        })
                    }
                    // if error
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_POST_SENSOR_TYPE_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                })
            }
        })
    })

    /**
     * API Route: "/api/sensorType/:location"
     * API Method: GET
     * Get sensor type data for a given location.
     */
    app.get("/api/sensorType/:location", function(req, res) {
        //get data
        db.getSensorType(req, function(sensorTypeData) {
            // if error
            if (sensorTypeData == null) {
                res.status(500);
                res.json({error: i18next.t("API_GET_SENSOR_TYPE_ERROR")});
            }
            // if ok
            else {
                res.status(200);
                res.send(JSON.stringify(sensorTypeData));
            }
        })
    })

    /**
     * API Route: "/api/sensorType/"
     * API Method: PUT
     * Update sensor type.
     */
    app.put("/api/sensorType/", function(req, res) {
        var sensorType = req.body.sensorType
        sensorType.location = req.body.location
        sensorType.oldSensorNumberOfIntervals = req.body.oldSensorNumberOfIntervals
        sensorType.oldSensorAggregation = req.body.oldSensorAggregation
        sensorType.oldSensorType = req.body.oldSensorType
        // validate
        validations.validateSensorType(sensorType, i18next, function (response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                // get sensors to update
                // update
                db.updateSensorType(sensorType, function (response) {
                    // if ok
                    if (response != null) {
                        res.status(200)
                        res.json({
                            message: i18next.t("API_PUT_SENSOR_TYPE_SUCCESS"),
                            messageType: "alert alert-primary alert-dismissible"
                        })
                    }
                    // if error
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_PUT_SENSOR_TYPE_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                }).then(r => {
                })
            }
        })
    })

    /**
     * API Route: "/api/sensorType/"
     * API Method: DELETE
     * DELETE sensor type.
     */
    app.delete("/api/sensorType", function (req, res)  {
        // delete
        db.deleteSensorType(req.body.sensorType, function(response) {
            // if ok
            if (response != null) {
                res.status(200)
                res.json({
                    message: i18next.t("API_DELETE_SENSOR_TYPE_SUCCESS"),
                    messageType: "alert alert-primary alert-dismissible"
                })
            }
            // if error
            else {
                res.status(400)
                res.json({
                    message: i18next.t("API_DELETE_SENSOR_TYPE_ERROR"),
                    messageType: "alert alert-danger alert-dismissible"
                })
            }
        })
    })
}