// CRUD for Shift
module.exports = function (app, db, validations, i18next) {

    /**
     * API Route: "/api/shift/:location"
     * API Method: GET
     * Get shift data for a given location.
     */
    app.get("/api/shift/:location", function (req, res) {
        // read
        db.getShiftData(req, function (shiftData) {
            // if error
            if (shiftData == null) {
                res.status(500);
                res.json({error: i18next.t("API_GET_SHIFT_ERROR")});
            }
            // if ok
            else {
                res.status(200);
                res.send(JSON.stringify(shiftData))
            }
        })
    })

    /**
     * API Route: "/api/shift/"
     * API Method: PUT
     * Update existing shift data.
     */
    app.put("/api/shift/", function(req, res) {
        let shift = req.body.shift
        shift.location = req.body.location
        // validate
        validations.validateShift(shift, i18next, function (response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                // update
                db.updateShift(shift, function (response) {
                    // if ok
                    if (response != null) {
                        res.status(200)
                        
                        res.json({
                            message: i18next.t("API_PUT_SHIFT_SUCCESS"),
                            messageType: "alert alert-primary alert-dismissible"
                        })
                    }
                    // if error
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_PUT_SHIFT_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                })
            }
        })
    })
}