// CRUD for sigmoid
module.exports = function (app, db, validations, i18next) {

    /**
     * API Route: "/api/sigmoid/:location/:timeHorizon"
     * API Method: GET
     * Get sigmoid data for a given location and time horizon.
     */
    app.get("/api/sigmoid/:location/:timeHorizon", function (req, res) {
        // read
        db.getSigmoidData(req, function (sigmoidData) {
            // if error
            if (sigmoidData == null) {
                res.status(500);
                //res.json({error: "error getting sigmoid data"});
                res.json({
                    message: i18next.t("API_GET_SIGMOID_ERROR"),
                    messageType: "alert alert-danger alert-dismissible"
                })
            }
            // if ok
            else {
                res.status(200);
                res.send(JSON.stringify(sigmoidData))
            }
        })
    })


    /**
     * API Route: "/api/sigmoid/"
     * API Method: PUT
     * Update sigmoid data.
     */
    app.put("/api/sigmoid/", function (req, res) {
        var sigmoid = req.body.sigmoid
        sigmoid.timeHorizon = req.body.timeHorizon
        sigmoid.location = req.body.location
        // validate
        validations.validateSigmoid(sigmoid, i18next,function (response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                // update
                db.updateSigmoid(sigmoid, function (response) {
                    // if ok
                    if (response != null) {
                        res.status(200)
                        res.json({
                            message: i18next.t("API_PUT_SIGMOID_SUCCESS"),
                            messageType: "alert alert-primary alert-dismissible"
                        })
                    }
                    // if error
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_PUT_SIGMOID_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                })
            }
        })
    })
}
