// CRUD for time horizon
module.exports = function (app, db, validations, i18next) {

    /**
     * API Route: "/api/timeHorizon/:location"
     * API Method: GET
     * Get time horizon data for a given location.
     */
    app.get("/api/timeHorizon/:location", function(req, res) {
        // read
        db.getTimeHorizonData(req, function(timeHorizonData) {
            // if error
            if (timeHorizonData == null) {
                res.status(400);
                res.json({error: i18next.t("API_GET_TIME_HORIZON_ERROR")});
            }
            // if ok
            else {
                res.status(200);
                res.send(JSON.stringify(timeHorizonData));
            }
        })
    })


    /**
     * API Route: "/api/timeHorizon/"
     * API Method: PUT
     * Update time horizon data.
     */
    app.put("/api/timeHorizon/", function(req, res) {
        var timeHorizon = req.body.timeHorizon
        timeHorizon.location = req.body.location
        timeHorizon.oldTimeHorizon = req.body.oldTimeHorizonId
        // validate
        validations.validateTimeHorizon(timeHorizon, i18next, function (response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                // update
                db.updateTimeHorizon(timeHorizon, function (response) {
                    // if ok
                    if (response != null) {
                        res.status(200)
                        res.json({
                            message: i18next.t("API_PUT_TIME_HORIZON_SUCCESS"),
                            messageType: "alert alert-primary alert-dismissible"
                        })
                    }
                    // if error
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_PUT_TIME_HORIZON_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                })
            }
        })
    })
}