// CRUD for weights
module.exports = function (app, db, validations, i18next) {

    /**
     * API Route: "/api/weight/:location/:timeHorizon"
     * API Method: GET
     * Get weights data for a given location and time horizon.
     */
    app.get("/api/weight/:location/:timeHorizon", function(req, res) {
        // get
        db.getWeightData(req, function(weightData) {
            // if error
            if (weightData == null) {
                res.status(500);
                res.json({error: i18next.t("API_GET_WEIGHTS_ERROR")});
            }
            // if ok
            else {
                res.status(200);
                res.send(JSON.stringify(weightData));
            }
        })
    })


    /**
     * API Route: "/api/weight/"
     * API Method: PUT
     * Update weight data.
     */
    app.put("/api/weight/", function (req, res) {
        var weights = req.body.weights
        // validate
        validations.validateWeights(weights, i18next,function(response) {
            // if error
            if (response !== "") {
                res.status(400)
                res.json({
                    message: response.message,
                    messageType: response.messageType
                })
            }
            // if ok
            else {
                weights.location = req.body.location
                weights.predictionHorizon = req.body.predictionHorizon
                weights.length = req.body.length
                weights.sensors = req.body.sensors
                weights.sensorNumber = req.body.sensorNumber
                // update
                db.updateWeight(weights, function (response) {
                    // if ok
                    if (response != null) {
                        res.status(200)
                        res.json({
                            message: i18next.t("API_PUT_WEIGHTS_SUCCESS"),
                            messageType: "alert alert-primary alert-dismissible"
                        })
                    }
                    // if error
                    else {
                        res.status(400)
                        res.json({
                            message: i18next.t("API_PUT_WEIGHTS_ERROR"),
                            messageType: "alert alert-danger alert-dismissible"
                        })
                    }
                })
            }
        })
    })


    app.get("/api/weights/:sensor/:location", function (req, res) {
        db.getSensorsToUpdate(req, function(response) {
            if (response != null) {
                res.status(200)
                res.json({
                    message: i18next.t("API_GET_WEIGHTS_SUCCESS"),
                    messageType: "alert alert-primary alert-dismissible",
                    queryResult: response
                })
            }
            else {
                res.status(400)
                res.json({
                    message: i18next.t("API_GET_WEIGHTS_ERROR"),
                    messageType: "alert alert-danger alert-dismissible"
                })
            }
        })
    })
}