var server = true
function getBaseURL() {
    return server ? "http://localhost:5000/" : "http://localhost:5000/"
}

/**
 * Get x basin coordinate for the given location
 * @param location. String that matches with a valid location
 * @returns {number}. X coordinate for that location
 */
function getXBasinCoordinate(location) {
    switch (location.toLowerCase()) {
        case VENERO_CLARO:
            return -4.64;
            break;
        case CAMBO_LES_BAINS:
            return -1.40;
            break;
        default:
            return -4.64;
    }
}

/**
 * Get y basin coordinate for the given location
 * @param location. String that matches with a valid location
 * @returns {number}. Y coordinate for that location.
 */
function getYBasinCoordinate(location) {
    switch (location.toLowerCase()) {
        case VENERO_CLARO:
            return 40.39;
            break;
        case CAMBO_LES_BAINS:
            return 43.34;
            break;
        default:
            return 40.39;
    }
}

var filenames=[]

function getWeatherFromString(cadena) {
    if (cadena.toLowerCase().includes('linéaire') || cadena.toLowerCase().includes('linear') || cadena.toLowerCase().includes('lineal')) {
        return 'lineal'
    }
    else if (cadena.toLowerCase().includes('gaussian') || cadena.toLowerCase().includes('gaussiana') || cadena.toLowerCase().includes('gaussian')) {
        return 'gaussian';
    }
    else {
        return 'quadratic';
    }
}
async function getTifLayer(map) {
    var myInit = {
        method: 'GET',
        mode: 'cors'
    };
    let horizon = $("#currentValue").val()
    // let weather = "Automatic"
    let weather = $("#inputRainfall").val()
    let simulation = 0
    let gid = 0
    let location = $("#inputArea").val()
    if (location.toLowerCase().includes(CAMBO_LES_BAINS)) {
        simulation = $("#inputSimulation").val()
        gid = $("#inputGID").val()
    }
    let hour = new Date($("#datePicker").val()).getHours()
    if (location.toLowerCase().includes(VENERO_CLARO)) {
        weather = $("#inputRainfall").val()
    }

    var url = new URL(getBaseURL() + "cbr/input?");
    if (weather.toLowerCase().includes('automatic') || weather.toLowerCase().includes('automático') || weather.toLowerCase().includes('automatique')) {
        const params = {
            horizon: horizon,
            location: location,
            hour: hour,
            weather: weather,
            simulation: simulation,
            gid: gid
        }
        Object.keys(params).forEach(key => url.searchParams.append(key, params[key]))
    }
    else {
        weather = getWeatherFromString($("input[name='curveType']:checked").val());
        a = $("#aValue").val()
        b = $("#bValue").val()
        c = $("#cValue").val()


        const params = {
            horizon: horizon,
            location: location,
            hour: hour,
            weather: weather,
            a: a,
            b: b,
            c: c
        }
        Object.keys(params).forEach(key => url.searchParams.append(key, params[key]))
    }
    var myRequest = new Request(url, myInit);
    fetch(myRequest).then(function(response){
        response.json().then(function(json) {
            // If no images...
            if (json.code == 500) {
                $("#spinner").hide()
            }
            // If there are images
            else {
                filenames = json.filenames
                loadFirstImage(map, filenames)
            }
        })
    })
}

async function loadFirstImage(map, filenames) {
    const url = getBaseURL() + "static/images/" + filenames[0]
    var myInit = {
        method: 'GET',
        mode: "cors"
    };
    var req = new Request(url, myInit);
    fetch(req)
        .then(function(response) {
            return response.blob();
        })
        .then(function(myBlob) {
            var objectURL = URL.createObjectURL(myBlob);
            var sourceT = new ol.source.GeoTIFF({
                sources: [
                    {
                        url: objectURL
                    },
                ],
            });
            var layerT = new ol.layer.WebGLTile({source: sourceT})

            map.addLayer(layerT);
            $("#spinner").hide()
        });
}

/**
 * Function that loads the layer of the map for the given location
 * @param location. Location of the map.
 * @returns {GeoCanvas}. TIF format layer.
 */
function getLayer( map) {
    getTifLayer(map);

}

/**
 * Function that creates and configures a map that will be returned and added to the html.
 * @param location. Location of the map.
 * @returns {MAP}. It will return the loaded map.
 */
function getMap(location) {
    // Open Street Map Layer
    let osmLayer = new ol.layer.Tile({
        source: new ol.source.OSM()
    });
    // Configure zoom options
    let zoom = 13;
    let maxZoom = 15;
    let minZoom = 10;
    // Getting x,y center coordinates and building center value
    let xBasinCoordinate = getXBasinCoordinate(location);
    let yBasinCoordinate = getYBasinCoordinate(location);
    let center = ol.proj.transform([xBasinCoordinate, yBasinCoordinate], 'EPSG:4326', 'EPSG:4326');

    // Set the view of the map with previous values
    let view = new ol.View({
        center: center,
        zoom: zoom,
        maxZoom: maxZoom,
        minZoom: minZoom,
        projection: 'EPSG:4326'
    });

    // Defining html target element id to be occupied by map
    let target = 'map';

    // Create the map
    let map = new ol.Map({
        layers: [
            osmLayer,
        ],
        target: target,
        view: view
    });

    // Get tif format layer
    getLayer(map);
    return map;
}

// Getting the map
map = getMap($("#inputArea option:selected").text())
center = 0
/**
 * Center back map
 */
function centerBack(){
    map.getView().setCenter(ol.proj.transform(center, 'EPSG:4326', 'EPSG:4326'));
    map.render()

}

/**
 * Function to pan around the map
 * @param element
 */
function pan(element){
    center = map.getView().getCenter()
    const resolution = map.getView().getResolution()
    const mapSize = map.getSize()

    const mapHorizontalMove = (mapSize[0] * resolution) / 10
    const mapVerticalMove = (mapSize[1] * resolution) / 10

    switch (element.id) {
        case "down":
            center[1] -= mapVerticalMove
            break;
        case "up":
            center[1] += mapVerticalMove;
            break;
        case "left":
            center[0] -= mapHorizontalMove;
            break;
        case "right":
            center[0] += mapHorizontalMove;
            break;
    }
    map.render()
}

$("#predictionHorizonRange").change(function (){
    $("#spinner").show()
    $("#currentValue").val($("#predictionHorizonRange").val())
    getLayer(map)
})
async function loadMapImage(filename) {
    const url = new URL(getBaseURL() + "static/images/" + filenames[filename-1])
    var myInit = {
        method: 'GET'
    };
    var req = new Request(url, myInit);

    fetch(req)
        .then(function(response) {
            return response.blob();
        })
        .then(function(myBlob) {
            if (map.getLayers().getArray()[1] != null){
                map.getLayers().removeAt(1)
            }
            var objectURL = URL.createObjectURL(myBlob);
            layer = new GeoCanvas(map, objectURL, {});
            layer.init()
        });
}

