var oldProratedSensorIdentifier

/**
 * Load prorated variable types from the different sensors of the selected location.
 * @param proratedSensorData. Sensors of the selected location.
 */
function loadOptions(proratedSensorData, proratedVariable) {
    // clear out the select item
    $("#inputProratedVariable").empty()
    var sensorTypes = []
    // include the sensor types
    proratedSensorData.forEach(sensor => {
        if (!sensorTypes.includes(sensor.conf_sensor_type_id)) {
            sensorTypes.push(sensor.conf_sensor_type_id)
        }
    })
    $("#inputProratedVariable").change(function() {
        updateProratedVariable()
    })
    sensorTypes.forEach(type => {
        if (type == proratedVariable) {
            $("#inputProratedVariable").append("<option value='" + type + "' selected >" + type + "</option>")
        }
        else {
            $("#inputProratedVariable").append("<option value='" + type + "'>" + type + "</option>")
        }
    })
}

/**
 * Load sensors with the same type as the selected prorated variable.
 * @param proratedSensorData. Sensors to filter.
 */
function getSelectedValueAndLoadSensors(proratedSensorData, proratedVariable) {
    // Load prorated variable
    loadOptions(proratedSensorData, proratedVariable)
    // For each type.
    // If sensor type has an Avg aggregation mode do nothing
    // If not, select the prorated sensor
    $("#proratedVariableAggregation").empty()
    var agg = false
    for (i = 0; i < proratedSensorData.length; i++) {
        if (proratedVariable == proratedSensorData[i].conf_sensor_type_id) {
            if (proratedSensorData[i].conf_sensor_aggregation_id == "Avg") {
                agg = true
            }
        }
    }
    if (!agg) {
        loadProratedSensorConfTable(proratedSensorData, proratedVariable)
    }
}

/**
 * Create menu for prorated sensor selection
 * @param sensors as options
 */
function loadProratedSensorConfTable(sensors) {
    var appendString = "<p>" + i18next.t("TABLE_HEADER_EMPTY_PRORATED_VARIABLE") + "</p><div class='form-group row mt-3'>" +
        "   <label for='inputProratedVariable' class='input-group-text col-sm-2 col-form-label'>" + i18next.t("TABLE_LABEL_PRORATED_VARIABLE") + "</label>" +
        "   <select class='col-sm-10 form-control' id='inputProratedSensor' name='inputProratedSensor' placeHolder='" + i18next.t("TABLE_SELECT_TOOLTIP_PRORATED_VARIABLE") + "' onchange='updateProratedVariable()' onfocusin='setOldValue()' oldvalue=''>"
    appendString.concat("</select> </div>")
    $("#proratedVariableAggregation").append(appendString);
    sensors.forEach(sensor => {
        if (sensor.conf_sensor_type_id == proratedVariable) {
            $("#inputProratedSensor").append("<option value=" + sensor.sensor_identifier + ">" + sensor.sensor_identifier + "</option>");
        }
    })
}


/**
 * Create an ajax call to the API to upload prorated variable
 */
function updateProratedVariable() {
    var proratedSensorIdentifier = $("#inputProratedSensor").val()
    var location = $("#locationSelect").val()
    var proratedVariableValue = $("#inputProratedVariable").val()
    $.ajax({
        url: URLbase + "/updateProratedVariable",
        type: "PUT",
        data: {
            proratedSensorIdentifier: proratedSensorIdentifier,
            oldProratedSensorIdentifier: oldProratedSensorIdentifier,
            proratedVariable: proratedVariableValue,
            location: location
        },
        dataType: 'json',
        headers: {},
        success: function(validationResult) {
            loadProratedVariable()
            showMessage(validationResult)
        },
        error : function (error) {
            loadProratedVariable()
            showMessage(error.responseJSON)
        }
    });
}