// File for sensor data CRUD operations but READ.

// old sensor data
var oldTypeData
var oldUrlData
var oldUsername
var oldPassword
var oldSensorIdData
var oldSamplingFreqData


// ----------------------------------------- CREATE -----------------------------------------

/**
 * Create a new row in the sensor table to add a new sensor.
 * @param sensorRow. Row of the new sensor item.
 */
function newSensorRow(sensorRow) {
    // add new row to table
    $( "#sensorTableBody" ).append(
        "<tr isLast = 'true' id = 'sensorRow" + sensorRow + "'>" +
        "<td id = 'sensorType" + sensorRow + "'> </td>" +
        "<td id = 'sensorURL" + sensorRow + "'> </td>" +
        "<td id = 'sensorUsername" + sensorRow + "'> </td>" +
        "<td id = 'sensorPassword" + sensorRow + "'> </td>" +
        "<td id = 'sensorId" + sensorRow + "'> </td>" +
        "<td id = 'sensorSamplingFrequency" + sensorRow + "'> </td>" +
        "<td>" +
        "<a id = 'editSensorButton" + sensorRow + "' onclick = editSensorData(" + sensorRow + ") data-toggle='tooltip' data-placement='top' title='Edit sensor data'> <i class = 'fas fa-edit'></i> </a>" +
        "<a class = 'updateButton' id='createNewSensorButton" + sensorRow + "' onclick = saveNewSensorData(" + sensorRow + ") data-toggle='tooltip' data-placement='top' title='Save sensor data'> <i class = 'fas fa-save'></i> </a>" +
        "<a id = 'deleteSensorButton" + sensorRow + "' onclick = deleteSensorData(" + sensorRow + ") data-toggle='tooltip' data-placement='top' title='Delete sensor'> <i class = 'fas fa-trash'></i> </a>" +
        "</td>" +
        "</tr>"
    )

    createNewSensor(sensorRow)
}

/**
 * Create a new editable sensor in a new row.
 * @param sensorRow. Row of the new sensor item.
 */
function createNewSensor(sensorRow) {
    // hide edit button and show save button
    document.getElementById("editSensorButton" + sensorRow).style.display="none";
    document.getElementById("createNewSensorButton" + sensorRow).style.display="inline-block";

    // get elements and edit
    edit(sensorRow, false)

    sensorRow = parseInt(sensorRow) + 1
    // Reset new row button
    $( "#newSensorRowButton").empty()
    $( "#newSensorRowButton" ).append("<a id='newSensorButton' onclick = newSensorRow(" + sensorRow + ")> <i class = 'fas fa-plus'></i>" + i18next.t("TABLE_OPTION_BUTTON_NEW_SENSOR") + "</a>" )
}


/**
 * Save sensor data in a selected row
 * @param sensorRow to save
 */
function saveNewSensorData(sensorRow) {
    // Obtain modified sensor data
    const sensorData = obtainModifiedSensorData(sensorRow)

    // Save new sensor in db
    saveSensor(sensorData, sensorRow)

    // Modify table action buttons
    document.getElementById("editSensorButton" + sensorRow).style.display = "inline-block";
    document.getElementById("createNewSensorButton" + sensorRow).style.display = "none";
}

/**
 * Function to save sensor data in database.
 * This function is called when the user wants to save a new created sensor
 * @param newSensor Data to save.
 * @param sensorRow Sensor of the saved row.
 */
function saveSensor(newSensor, sensorRow) {
    var location = $("#locationSelect").val()
    $.ajax({
        url: URLbase + "/sensor",
        type: "POST",
        data: {
            sensor: newSensor,
            sensorIdentifier: newSensor.sensorId,
            location: location
        },
        dataType: 'json',
        headers: {},
        success: function(validationResult) {
            loadSensorData()
            showMessage(validationResult)
            updateCBRApp()
        },
        error : function (error) {
            loadSensorData()
            showMessage(error.responseJSON)
        }
    });
}

/**
 * Edit sensor data in a selected row.
 * It is executed after clicking on the edit icon for an existing row.
 * @param sensorRow to edit
 */
function editSensorData(sensorRow) {
    document.getElementById("deleteSensorButton" + sensorRow).onclick = function() { deleteSensorData(sensorRow, true)};

    // hide edit button and show save button
    document.getElementById("editSensorButton" + sensorRow).style.display="none";
    document.getElementById("updateSensorButton" + sensorRow).style.display="inline-block";

    let isEdit = true
    edit(sensorRow, isEdit)
}

/**
 * Helper function for editing a sensor element.
 * @param sensorRow. Row of the sensor that will be edited.
 */
function edit(sensorRow, isEdit) {
    // get sensor elements
    const type = document.getElementById("sensorType" + sensorRow)
    const url = document.getElementById("sensorURL" + sensorRow)
    const username = document.getElementById("sensorUsername" + sensorRow)
    const password = document.getElementById("sensorPassword" + sensorRow)
    const sensorId = document.getElementById("sensorId" + sensorRow)
    const samplingFreq = document.getElementById("sensorSamplingFrequency" + sensorRow)
    const sensorTypeText = 'sensorType_text'
    fillTypeOptions(type, sensorTypeText, sensorRow)

    // Update this at the load of the data to save the first value and update it whenever saveData has been called.
    if (isEdit) {
        oldTypeData = type.innerHTML
        oldUrlData = url.innerHTML
        oldUsername = username.innerHTML
        oldPassword = password.innerHTML
        oldSensorIdData = sensorId.innerHTML
        oldSamplingFreqData = samplingFreq.innerHTML

        url.innerHTML = "<input class='form-control' type='text' id='sensorUrl_text" + sensorRow + "' value='" + oldUrlData + "'>";
        username.innerHTML = "<input class='form-control' type='text' id='sensorUsername_text" + sensorRow + "' value='" + oldUsername + "'>";
        password.innerHTML = "<input class='form-control' type='text' id='sensorPassword_text" + sensorRow + "' value='" + oldPassword + "'>";
        sensorId.innerHTML = "<input class='form-control' size='8' type='text' id='sensorId_text" + sensorRow + "' value='" + oldSensorIdData + "'>";
        samplingFreq.innerHTML = "<input class='form-control number' size='1' type='text' id='sensorSamplingFreq_text" + sensorRow + "' value='" + oldSamplingFreqData + "'>";
    }
    else {
        url.innerHTML = "<input class='form-control' type='text' id='sensorUrl_text" + sensorRow + "' value='" + "'>";
        username.innerHTML = "<input class='form-control' type='text' id='sensorUsername_text" + sensorRow + "' value='" + "'>";
        password.innerHTML = "<input class='form-control' type='text' id='sensorPassword_text" + sensorRow + "' value='" + "'>";
        sensorId.innerHTML = "<input class='form-control' size='8' type='text' id='sensorId_text" + sensorRow + "' value='" + "'>";
        samplingFreq.innerHTML = "<input class='form-control number' size='1' type='text' id='sensorSamplingFreq_text" + sensorRow + "' value='" + "'>";
    }

}

/**
 * Helper function to fill in type select item with valid types. Types are obtained from the database through an API call.
 * @param type. Type item to fill in.
 * @param sensorTypeText. Id of the select item.
 * @param sensorRow. Sensor row to be modified.
 */
function fillTypeOptions(type, sensorTypeText, sensorRow) {
    var location = $("#locationSelect").val()
    $.ajax({
        url: URLbase + "/sensor/types/" + location,
        type: "GET",
        data: { },
        dataType: 'json',
        success: function(types) {
            var typeString = "<select class='form-control' id='" + sensorTypeText + sensorRow + "' value='" + oldTypeData + "'> "
            var parsedTypes = JSON.parse(types)
            for (i=0 ; i < parsedTypes.length; i++ ) {
                typeString = typeString.concat("<option value='" + parsedTypes[i].conf_sensor_type_id + "'>" + parsedTypes[i].conf_sensor_type_id + "</option>")
            }
            typeString = typeString.concat("</select>")
            type.innerHTML = typeString
            setSelectedOption(sensorTypeText, oldTypeData, sensorRow)

        },
        error : function (error) {
            loadSensorData()
            showMessage(error.responseJSON)
        }
    });
}



// ----------------------------------------- UPDATE -----------------------------------------

/**
 * Save sensor data in a selected row
 * @param sensorRow to save
 */
function updateSensorData(sensorRow) {
    // Obtain sensor data from the inputs
    const sensorData = obtainModifiedSensorData(sensorRow)

    // Update sensor data
    updateSensor(sensorData, sensorRow)

    // Modify table action buttons
    document.getElementById("editSensorButton" + sensorRow).style.display = "inline-block";
    document.getElementById("updateSensorButton" + sensorRow).style.display = "none";
}


/**
 * Function to update sensor data in database.
 * This function is called when an existing sensor is edited and
 * the user clicks on the save icon to save the changes.
 * @param newSensor new sensor data to save.
 * @param sensorRow Sensor of the saved row.
 */
function updateSensor(newSensor, sensorRow) {
    var location = $("#locationSelect").val()
    $.ajax({
        url: URLbase + "/sensor",
        type: "PUT",
        data: {
            sensor: newSensor,
            sensorIdentifier: oldSensorIdData,
            location: location
        },
        dataType: 'json',
        headers: {},
        success: function(validationResult) {
            loadSensorData()
            showMessage(validationResult)
            updateCBRApp()
        },
        error : function (error) {
            loadSensorData()
            showMessage(error.responseJSON)
        }
    });
}



// ----------------------------------------- DELETE -----------------------------------------

/**
 * Delete sensor data in a selected row
 * @param sensorRow to delete
 */
function deleteSensorData(sensorRow, isEditing) {
    $("#sensorDeleteModal").modal()
    $("#sensorDeleteModalButton").on("click", function () {
        $("#sensorDeleteModal").modal('hide')
        var sensor_identifier = document.getElementById("sensorId" + sensorRow).innerHTML;
        var location = $("#locationSelect").val()

        if (document.getElementById("sensorRow" + sensorRow + "").getAttribute("isLast") == "true") {
            document.getElementById("sensorRow" + sensorRow + "").setAttribute("isLast", "false")
        }
        else {
            var sensor;
            isEditing ? sensor = {
                "sensorId": oldSensorIdData,
                "location": location
            } : sensor = {
                "sensorId": sensor_identifier,
                "location": location
            }
            deleteSensor(sensor)
        }
        document.getElementById("sensorRow" + sensorRow + "").outerHTML = "";
        $("#sensorDeleteModalButton").off("click")
    })
    $("#sensorCancelModalButton").on("click", function () {
        $("#sensorDeleteModal").modal('hide')
        $("#sensorDeleteModalButton").off("click")

    })
}

/**
 * Function to delete sensor data in database.
 * This function is called when the user clicks on the delete icon for a sensor row.
 * @param sensor to delete.
 * @param sensorRow row to delete.
 */
function deleteSensor(sensor, sensorRow) {
    $.ajax({
        url: URLbase + "/sensor",
        type: "DELETE",
        data: {
            sensor: sensor
        },
        dataType: 'json',
        headers: {},
        success: function(validationResult) {
            showMessage(validationResult)
            updateCBRApp()
        },
        error : function (error) {
            showMessage(error.responseJSON)
            // If there is an error old data should be placed.
        }
    });
}


// ----------------------------------------- HELPER -----------------------------------------
/**
 * Helper function to obtain the new or modified data from a sensor.
 * @param sensorRow. Sensor row from which data will be obtained.
 * @returns {{sensorURL: *, sensorType: *, sensorUsername: *, sensorSamplingFreq: *, sensorPassword: *, sensorId: *}}. Returns object that contains recovered data.
 */
function obtainModifiedSensorData(sensorRow) {
    var type_val = document.getElementById("sensorType_text" + sensorRow).value;
    var url_val = document.getElementById("sensorUrl_text" + sensorRow).value;
    var username_val = document.getElementById("sensorUsername_text" + sensorRow).value;
    var password_val = document.getElementById("sensorPassword_text" + sensorRow).value;
    var id_val = document.getElementById("sensorId_text" + sensorRow).value;
    var samplingFreq_val = document.getElementById("sensorSamplingFreq_text" + sensorRow).value;

    const sensorData = {
        "sensorType": type_val,
        "sensorURL": url_val,
        "sensorUsername": username_val,
        "sensorPassword": password_val,
        "sensorId": id_val,
        "sensorSamplingFreq": samplingFreq_val,
    }
    return sensorData
}
