// old sensorType data
var oldSensorTypeTypeData
var oldSensorTypeHourIntervalData
var oldSensorTypeAggregationModeData

// ----------------------------------------- CREATE -----------------------------------------

/**
 * Function for adding a new sensor type row.
 * @param sensorTypeRow. Row of the new sensor type.
 */
function newSensorType(rowNumber) {
    // add new row to table
    $( "#sensorTypeBody" ).append(
        "<tr isLast = 'true' id='sensorTypeRow" + rowNumber + "'>" +
        "<td id='sensorTypeType" + rowNumber + "'> </td>" +
        "<td id='sensorTypeHourInterval" + rowNumber + "'> </td>" +
        "<td id='sensorTypeAggregationMode" + rowNumber + "'> </td>" +
        "<td>" +
        "<a id='editSensorTypeButton" + rowNumber + "' onclick = editSensorTypeData(" + rowNumber + ")> <i class='fas fa-edit'> </i></a>" +
        "<a class='updateButton' id='createNewSensorTypeButton" + rowNumber + "' onclick = saveNewSensorTypeData(" + rowNumber + ")> <i class='fas fa-save'> </i></a>" +
        "<a id='deleteSensorTypeButton" + rowNumber + "' onclick = deleteSensorTypeData(" + rowNumber + "," + false + ")> <i class='fas fa-trash'> </i></a>" +
        "</td>" +
        "</tr>")

    createNewSensorType(rowNumber)
}

/**
 * Function for creating a new sensor type row.
 * @param sensorTypeRow. Row of the new sensor type item.
 */
function createNewSensorType(sensorTypeRow) {
    // hide edit button and show save button
    document.getElementById("editSensorTypeButton" + sensorTypeRow).style.display="none";
    document.getElementById("createNewSensorTypeButton" + sensorTypeRow).style.display="inline-block";

    // get sensor elements and edit
    obtainModifiedSensorTypeData(sensorTypeRow, false)

    // Reset new row button
    $("#newSensorTypeRowButton").empty()
    $("#newSensorTypeRowButton").append("<a id='newSensorTypeButton' onclick = newSensorType(" + sensorTypeRow+1 + ") <i class = 'fas fa-plus'></i>" + i18next.t("TABLE_OPTION_BUTTON_NEW_SENSOR_TYPE") + "Add sensor type </a>" )
}

/**
 * Get modified sensor type data and call the function to save it in the database.
 * @param sensorTypeRow. Row of the new sensor type
 */
function saveNewSensorTypeData(sensorTypeRow) {
    // document.getElementById("newSensorTypeRowButton").style.display = "inline-block";

    const sensorTypeType_val = document.getElementById("sensorTypeType_text" + sensorTypeRow).value;
    const sensorTypeHourIntervalData_val = document.getElementById("sensorTypeHourInterval_text" + sensorTypeRow).value;
    const sensorTypeAggregationMode_val = document.getElementById("sensorTypeAggregationMode_text" + sensorTypeRow).value;

    const sensorTypeData = {
        "sensorTypeType": sensorTypeType_val,
        "sensorTypeHourInterval": sensorTypeHourIntervalData_val,
        "sensorTypeAggregationMode": sensorTypeAggregationMode_val
    }
    saveSensorType(sensorTypeData, sensorTypeRow)

    document.getElementById("editSensorTypeButton" + sensorTypeRow).style.display="inline-block";
    document.getElementById("createNewSensorTypeButton" + sensorTypeRow).style.display="none";
}

/**
 * Function that makes a call to the API to save a new sensor type in the database.
 * @param sensorTypeData. Sensor type data to be saved in the db.
 * @param sensorTypeRow. Row in the column of the sensor type data.
 */
function saveSensorType(sensorTypeData, sensorTypeRow) {
    var location = $("#locationSelect").val()
    $.ajax({
        url: URLbase + "/sensorType",
        type: "POST",
        data: {
            sensorType: sensorTypeData,
            location: location
        },
        dataType: 'json',
        headers: {},
        success: function(validationResult) {
            loadSensorTypeData()
            showMessage(validationResult)
            updateCBRApp()
        },
        error : function (error) {
            loadSensorTypeData()
            showMessage(error.responseJSON)
        }
    });
}



// ----------------------------------------- UPDATE -----------------------------------------
/**
 * Function for editing a sensor type row.
 * @param sensorTypeRow. Row of the sensor type that will be edited.
 */
function editSensorTypeData(sensorTypeRow) {
    document.getElementById("deleteSensorTypeButton" + sensorTypeRow).onclick = function() { deleteSensorTypeData(sensorTypeRow,true)};

    // hide edit button and show save button
    document.getElementById("editSensorTypeButton" + sensorTypeRow).style.display="none";
    document.getElementById("saveSensorTypeButton" + sensorTypeRow).style.display="inline-block";

    // get sensor elements
    // obtainModifiedSensorTypeData(sensorTypeRow)
    let isEdit = true
    obtainModifiedSensorTypeData(sensorTypeRow, isEdit)
}

/**
 * Helper function for editing a sensor type row.
 */
function obtainModifiedSensorTypeData(sensorTypeRow, isEdit) {
    const sensorTypeType = document.getElementById("sensorTypeType" + sensorTypeRow)
    const sensorTypeHourInterval = document.getElementById("sensorTypeHourInterval" + sensorTypeRow)
    const sensorTypeAggregationMode = document.getElementById("sensorTypeAggregationMode" + sensorTypeRow)

    // get sensor data
    if (isEdit) {
        oldSensorTypeTypeData = sensorTypeType.innerHTML
        oldSensorTypeHourIntervalData = sensorTypeHourInterval.innerHTML
        oldSensorTypeAggregationModeData = sensorTypeAggregationMode.innerHTML
        // setSensorTypeTypeOptions(sensorTypeType, sensorTypeRow, sensorTypeData)
        sensorTypeType.innerHTML="<select class='form-control' id='sensorTypeType_text" + sensorTypeRow + "'>" +
            "    <option id='WDLOption " + sensorTypeRow + "' value='Water Discharge Level'>Water Discharge Level</option>" +
            "    <option id='RGOption " + sensorTypeRow + "' value='Rainfall Gauge'>Rainfall Gauge</option>" +
            "    <option id='RFOption " + sensorTypeRow + "' value='Rainfall Forecast'>Rainfall Forecast</option>" +
            "</select>"
        setSelectedOption('sensorTypeType_text', oldSensorTypeTypeData, sensorTypeRow)
        sensorTypeHourInterval.innerHTML="<input class='form-control' size='1' class='number' type='text' id='sensorTypeHourInterval_text" + sensorTypeRow + "' value='" + oldSensorTypeHourIntervalData + "'>";
        sensorTypeAggregationMode.innerHTML="<select class='form-control' id='sensorTypeAggregationMode_text" + sensorTypeRow + "' value='" + oldSensorTypeAggregationModeData + "'> " +
            "    <option value='Max'> Max </option>" +
            "    <option value='Avg'> Avg </option>" +
            "    <option value='None'> None </option>" +
            "</select>"
        setSelectedOption('sensorTypeAggregationMode_text', oldSensorTypeAggregationModeData, sensorTypeRow)
    }
    else {
        sensorTypeType.innerHTML="<select class='form-control' id='sensorTypeType_text" + sensorTypeRow + "'>" +
            "    <option id='WDLOption " + sensorTypeRow + "' value='Water Discharge Level'>Water Discharge Level</option>" +
            "    <option id='RGOption " + sensorTypeRow + "' value='Rainfall Gauge'>Rainfall Gauge</option>" +
            "    <option id='RFOption " + sensorTypeRow + "' value='Rainfall Forecast'>Rainfall Forecast</option>" +
            "</select>"
        sensorTypeHourInterval.innerHTML="<input class='form-control' type='text' id='sensorTypeHourInterval_text" + sensorTypeRow + "' value='" + "'>";
        sensorTypeAggregationMode.innerHTML="<select class='form-control' id='sensorTypeAggregationMode_text" + sensorTypeRow + "' value='" + "'> " +
            "    <option value='Max'> Max </option>" +
            "    <option value='Avg'> Avg </option>" +
            "    <option value='None'> None </option>" +
            "</select>"
    }


}

/**
 * Helper function to set an option in a select item.
 * @param id. Select item id.
 * @param selectedValue. Selected value to set in the select item.
 * @param sensorTypeRow. Row of the sensor type item.
 */
function setSelectedOption(id, selectedValue, sensorTypeRow) {
}

/**
 * Get modified sensor type data and call the function to update it in the database.
 * @param sensorTypeRow. Row of the new sensor type
 */
function updateSensorTypeData(sensorTypeRow) {
    // Obtain modified sensor type data
    const sensorTypeData = obtainSensorTypeData(sensorTypeRow)

    // Save new sensor type in db
    updateSensorType(sensorTypeData, sensorTypeRow)

    // Modify table action buttons
    document.getElementById("editSensorTypeButton" + sensorTypeRow).style.display="inline-block";
    document.getElementById("saveSensorTypeButton" + sensorTypeRow).style.display="none";
}

function obtainSensorTypeData(sensorTypeRow) {
    const sensorTypeType_val = document.getElementById("sensorTypeType_text" + sensorTypeRow).value;
    const sensorTypeHourIntervalData_val = document.getElementById("sensorTypeHourInterval_text" + sensorTypeRow).value;
    const sensorTypeAggregationMode_val = document.getElementById("sensorTypeAggregationMode_text" + sensorTypeRow).value;

    const sensorTypeData = {
        "sensorTypeType": sensorTypeType_val,
        "sensorTypeHourInterval": sensorTypeHourIntervalData_val,
        "sensorTypeAggregationMode": sensorTypeAggregationMode_val
    }
    return sensorTypeData;
}

/**
 * Function to update sensor data in database.
 * @param newSensor. New sensor type data to save.
 * @param sensorRow. Row of the updated sensor.
 */
function updateSensorType(sensorTypeData, sensorRow) {
    var location = $("#locationSelect").val()
    $.ajax({
        url: URLbase + "/sensorType",
        type: "PUT",
        data: {
            sensorType: sensorTypeData,
            location: location,
            oldSensorNumberOfIntervals: oldSensorTypeHourIntervalData,
            oldSensorAggregation: oldSensorTypeAggregationModeData,
            oldSensorType: oldSensorTypeTypeData
        },
        dataType: 'json',
        headers: {},
        success: function(validationResult) {
            loadSensorTypeData()
            showMessage(validationResult)
            updateCBRApp()
        },
        error : function (error) {
            loadSensorTypeData()
            showMessage(error.responseJSON)
        }
    });
}


// ----------------------------------------- DELETE -----------------------------------------

/**
 * Delete sensor type item for a given row.
 * @param sensorTypeRow. Row of the sensor type to be deleted.
 */
function deleteSensorTypeData(sensorTypeRow, isEditing) {
    $("#sensorTypeDeleteModal").modal()
    $("#sensorTypeDeleteModalButton").on("click", function () {
        $("#sensorTypeDeleteModal").modal('hide')
        var location = $("#locationSelect").val()
        if ( document.getElementById("sensorTypeRow" + sensorTypeRow + "").getAttribute("isLast") == "true") {
            document.getElementById("sensorTypeRow" + sensorTypeRow + "").setAttribute("isLast", "false")
        }
        else {
            var sensorType;
            isEditing ? sensorType = {
                "sensorTypeType": oldSensorTypeTypeData,
                "sensorTypeAggregationMode": oldSensorTypeAggregationModeData,
                "sensorTypeNumberOfIntervals": oldSensorTypeHourIntervalData,
                "location": location
            } : sensorType = {
                "sensorTypeType": document.getElementById("sensorTypeType" + sensorTypeRow).innerHTML,
                "sensorTypeAggregationMode": document.getElementById("sensorTypeAggregationMode" + sensorTypeRow).innerHTML,
                "sensorTypeNumberOfIntervals": document.getElementById("sensorTypeHourInterval" + sensorTypeRow).innerHTML,
                "location": location
            }
            deleteSensorType(sensorType)
        }
        document.getElementById("sensorTypeRow" + sensorTypeRow + "").outerHTML="";
    })
    $("#sensorTypeCancelModalButton").on("click", function () {
        $("#sensorTypeDeleteModal").modal('hide')
        $("#sensorTypeDeleteModalButton").off("click")

    })
}

/**
 * Call the API to delete a sensor type item.
 * @param sensorType. Sensor type item to be deleted.
 * @param sensorTypeRow. Row of the sensor type
 */
function deleteSensorType(sensorType, sensorTypeRow) {
    $.ajax({
        url: URLbase + "/sensorType",
        type: "DELETE",
        data: {
            sensorType: sensorType
        },
        dataType: 'json',
        headers: {},
        success: function(validationResult) {
            showMessage(validationResult)
            updateCBRApp()
        },
        error : function (error) {
            showMessage(error.responseJSON)
            // If there is an error old data should be placed.
        }
    });
}