DROP TABLE IF EXISTS "web_users" CASCADE;
DROP TABLE IF EXISTS "web_location_sensor" CASCADE;
DROP TABLE IF EXISTS "web_conf_location" CASCADE;
DROP TABLE IF EXISTS "web_conf_basin" CASCADE;
DROP TABLE IF EXISTS "web_access_data" CASCADE;
DROP TABLE IF EXISTS "web_conf_selection_mode" CASCADE;
DROP TABLE IF EXISTS "web_conf_sensor_type" CASCADE;
DROP TABLE IF EXISTS "web_conf_selection_criteria" CASCADE;
DROP TABLE IF EXISTS "web_conf_distance_function" CASCADE;
DROP TABLE IF EXISTS "web_conf_prediction_horizon" CASCADE;
DROP TABLE IF EXISTS "web_location_time_horizon" CASCADE;
DROP TABLE IF EXISTS "web_location_sensor_aggregation" CASCADE;
DROP TABLE IF EXISTS "web_conf_sensor_aggregation" CASCADE;
DROP TABLE IF EXISTS "web_weights" CASCADE;
DROP TABLE IF EXISTS "web_case" CASCADE;
DROP TABLE IF EXISTS "web_ts" CASCADE;
DROP TABLE IF EXISTS "web_ts_value" CASCADE;
DROP TABLE IF EXISTS "web_map_blob" CASCADE;
DROP TABLE IF EXISTS "web_simulation" CASCADE;
DROP TABLE IF EXISTS "web_load_line" CASCADE;
DROP TABLE IF EXISTS "registros" CASCADE;
DROP TABLE IF EXISTS "FTP_registros" CASCADE;

DROP TYPE IF EXISTS "interval_length_enum";
CREATE TYPE "interval_length_enum" AS ENUM (
  '1_hour',
  '15_minutes'
);

DROP TYPE IF EXISTS "units_enum";
CREATE TYPE "units_enum" AS ENUM (
  'hours',
  'minutes'
);

CREATE TABLE IF NOT EXISTS "web_users" (
  "user_id" SERIAL PRIMARY KEY,
  "username" varchar(100),
  "password" varchar(200)
);

CREATE TABLE IF NOT EXISTS "web_location_sensor" (
  "location_sensor_id" SERIAL PRIMARY KEY,
  "sensor_identifier" varchar,
  "sampling_freq" real,
  "prorated_variable_sensor" boolean,
  "conf_location_sensor_aggregation_id" int,
  "conf_location_id" int
);

CREATE TABLE IF NOT EXISTS "web_conf_location" (
  "conf_location_id" SERIAL PRIMARY KEY,
  "name" varchar,
  "gpsul" int,
  "gpsdr" int,
  "mhs" int,
  "mh" int,
  "lh" int,
  "interval_length" varchar,
  "prorated_variable" varchar,
  "conf_basin_name" varchar
);

CREATE TABLE IF NOT EXISTS "web_conf_basin" (
  "name" varchar,
  UNIQUE("name")
);

CREATE TABLE IF NOT EXISTS "web_access_data" (
  "access_data_id" SERIAL PRIMARY KEY,
  "url_access" varchar,
  "username" varchar,
  "password" varchar,
  "location_sensor_id" int,
  UNIQUE("location_sensor_id")
);

CREATE TABLE IF NOT EXISTS "web_conf_selection_mode" (
  "selection_mode" varchar,
  UNIQUE("selection_mode")
);

CREATE TABLE IF NOT EXISTS "web_conf_sensor_type" (
  "type" varchar,
  UNIQUE("type")
);

CREATE TABLE IF NOT EXISTS "web_conf_selection_criteria" (
  "selection_criteria" varchar,
  UNIQUE("selection_criteria")
);

CREATE TABLE IF NOT EXISTS "web_conf_distance_function" (
  "function" varchar,
  UNIQUE("function")
);

CREATE TABLE IF NOT EXISTS "web_conf_prediction_horizon" (
  "conf_prediction_horizon_id" SERIAL PRIMARY KEY,
  "units" varchar,
  "type" int,
  "min_prediction_horizon" int,
  "max_prediction_horizon" int,
  "max_number_of_intervals" int,
  "max_number_of_neighbours" int
);

CREATE TABLE IF NOT EXISTS "web_location_time_horizon" (
  "conf_location_id" int ,
  "prediction_horizon" int,
  "number_of_neighbour" int,
  "distance_threshold" real,
  "merging_parameter_a" int,
  "merging_parameter_b" int,
  "merging_parameter_c" int,
  "merging_parameter_delta" int,
  "conf_distance_function" varchar,
  "conf_selection_mode" varchar,
  "conf_selection_criteria" varchar,
  "conf_prediction_horizon_id" int,
  PRIMARY KEY ("conf_location_id", "prediction_horizon")
);

CREATE TABLE IF NOT EXISTS "web_location_sensor_aggregation" (
  "location_sensor_aggregation_id" SERIAL PRIMARY KEY,
  "number_of_intervals" int,
  "conf_location_id" int,
  "conf_sensor_aggregation_id" varchar,
  "conf_sensor_type_id" varchar
);

CREATE TABLE IF NOT EXISTS "web_conf_sensor_aggregation" (
  "aggregation" varchar,
  UNIQUE("aggregation")
);

CREATE TABLE IF NOT EXISTS "web_weights" (
  "weights_id" SERIAL PRIMARY KEY,
  "interval" int,
  "prediction_horizon" int,
  "conf_location_id" int,
  "location_sensor_id" int,
  "weight" float
);

CREATE TABLE IF NOT EXISTS "web_case" (
  "case_id" SERIAL PRIMARY KEY,
  "conf_location_id" int
);

CREATE TABLE IF NOT EXISTS "web_ts" (
  "ts_id" SERIAL PRIMARY KEY,
  "location_sensor_id" int,
  "case_id" int
);

CREATE TABLE IF NOT EXISTS "web_ts_value" (
  "ts_id" int,
  "value" float,
  "converted_value" float,
  "timestamp" int
);

CREATE TABLE IF NOT EXISTS"web_map_blob" (
  "map_id" SERIAL PRIMARY KEY, 
  "case_id" int,
  "prediction_horizon" int,
  "data" oid,
  "name" varchar(100)
  );



CREATE TABLE IF NOT EXISTS "web_simulation" (
  "simulation_id" SERIAL PRIMARY KEY,
  "gid" int,
  "timestamp" int,
  "rg_value" float,
  "wdl_cambo_value" float,
  "wdl_cambo_converted_value" float,
  "wdl_osses_value" float,
  "wdl_osses_converted_value" float,
  "simulation_graph_id" int,
  "simulation_map_id" int,
  "map_id" int
);

CREATE TABLE IF NOT EXISTS "web_load_line" (
  "load_line_id" SERIAL PRIMARY KEY, 
  "prediction_horizon" int,
  "conf_location_id" int,
  "location_sensor_id" int,
  "value" float
);

CREATE TABLE IF NOT EXISTS "registros" (
  "id" SERIAL PRIMARY KEY,
  "station_id" int,
  "datetime" int,
  "rain_rate_mm" float
);

CREATE TABLE IF NOT EXISTS "ftp_registros" (
  "id" SERIAL PRIMARY KEY,
  "datetime" int,
  "ch01" int,
  "name" varchar(100)
);


ALTER TABLE "web_load_line" ADD FOREIGN KEY ("conf_location_id") REFERENCES "web_conf_location" ("conf_location_id");

ALTER TABLE "web_load_line" ADD FOREIGN KEY ("location_sensor_id") REFERENCES "web_location_sensor" ("location_sensor_id");

ALTER TABLE "web_location_sensor" ADD FOREIGN KEY ("conf_location_sensor_aggregation_id") REFERENCES "web_location_sensor_aggregation" ("location_sensor_aggregation_id");

ALTER TABLE "web_location_sensor" ADD FOREIGN KEY ("conf_location_id") REFERENCES "web_conf_location" ("conf_location_id");

ALTER TABLE "web_conf_location" ADD FOREIGN KEY ("prorated_variable") REFERENCES "web_conf_sensor_type" ("type");

ALTER TABLE "web_conf_location" ADD FOREIGN KEY ("conf_basin_name") REFERENCES "web_conf_basin" ("name");

ALTER TABLE "web_location_sensor" ADD FOREIGN KEY ("location_sensor_id") REFERENCES "web_access_data" ("location_sensor_id");

ALTER TABLE "web_location_time_horizon" ADD FOREIGN KEY ("conf_distance_function") REFERENCES "web_conf_distance_function" ("function");

ALTER TABLE "web_location_time_horizon" ADD FOREIGN KEY ("conf_selection_mode") REFERENCES "web_conf_selection_mode" ("selection_mode");

ALTER TABLE "web_location_time_horizon" ADD FOREIGN KEY ("conf_selection_criteria") REFERENCES "web_conf_selection_criteria" ("selection_criteria");

ALTER TABLE "web_location_time_horizon" ADD FOREIGN KEY ("conf_location_id") REFERENCES "web_conf_location" ("conf_location_id");

ALTER TABLE "web_location_time_horizon" ADD FOREIGN KEY ("conf_prediction_horizon_id") REFERENCES "web_conf_prediction_horizon" ("conf_prediction_horizon_id");

ALTER TABLE "web_location_sensor_aggregation" ADD FOREIGN KEY ("conf_location_id") REFERENCES "web_conf_location" ("conf_location_id");

ALTER TABLE "web_location_sensor_aggregation" ADD FOREIGN KEY ("conf_sensor_aggregation_id") REFERENCES "web_conf_sensor_aggregation" ("aggregation");

ALTER TABLE "web_location_sensor_aggregation" ADD FOREIGN KEY ("conf_sensor_type_id") REFERENCES "web_conf_sensor_type" ("type");

ALTER TABLE "web_weights" ADD FOREIGN KEY ("location_sensor_id") REFERENCES "web_location_sensor" ("location_sensor_id");

ALTER TABLE "web_case" ADD FOREIGN KEY ("conf_location_id") REFERENCES "web_conf_location" ("conf_location_id");

ALTER TABLE "web_ts" ADD FOREIGN KEY ("location_sensor_id") REFERENCES "web_location_sensor" ("location_sensor_id");

ALTER TABLE "web_ts" ADD FOREIGN KEY ("case_id") REFERENCES "web_case" ("case_id");

ALTER TABLE "web_ts_value" ADD FOREIGN KEY ("ts_id") REFERENCES "web_ts" ("ts_id");

ALTER TABLE "web_map_blob" ADD FOREIGN KEY ("case_id") REFERENCES "web_case" ("case_id");